#ifndef _COMMON_H
#define _COMMON_H
/******************************************************************************

                              Copyright (c) 2009
                            Lantiq Deutschland GmbH
                     Am Campeon 3; 85579 Neubiberg, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

  ****************************************************************************
   Module      : common.h
   Description :
*******************************************************************************/
/**
   \file common.c
   \date 2005-10-01
   \brief Common for tapidemo enums, structure, macros and defines.
*/
/* ============================= */
/* Includes                      */
/* ============================= */

#ifdef LINUX
#include "tapidemo_config.h"
#endif

/** system includes */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef LINUX
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
   #include <netdb.h>
#include <sys/types.h>
#include <getopt.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <signal.h>
#include <time.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <pwd.h>
#include <ctype.h>
   #include <net/if.h>
#include <syslog.h>
#endif /* LINUX */

#ifdef VXWORKS
   #include <vxWorks.h>
   #include <sockLib.h>
   #include <inetLib.h>
   #include <ioLib.h>
   #include <sys/ioctl.h>
   #include <sysLib.h>
   #include <tickLib.h>
   #include <taskLib.h>
   #include <hostLib.h>
   #include <time.h>
   #include <taskHookLib.h>
   #include <sigLib.h>
   #include <inetLib.h>
#ifdef EASY336
   #include <natProtocol.h>
#endif
#endif /* VXWORKS */

#include "ifx_types.h"
#include "td_osmap.h"

#ifdef DANUBE
/* Compilation flag OLD_BSP is used for BSP which is compiled by LXDB */
   #ifdef OLD_BSP
      #include "asm/danube/port.h"
   #else
      #include "asm/ifx/ifx_gpio.h"
   #endif
#endif

#if (defined(AR9) || defined(VR9))
   #include "asm/ifx/ifx_gpio.h"
#endif

#ifdef TD_DECT

/* Workaround - so IFX_TRUE and IFX_FALSE will not be defined for the second
   time in file ifx_types.h and ifx_common_defs.h. Workaround should be used
   until both ifx_types.h and ifx_common_defs.h must be used */
#define IFX_TRUE 
#define IFX_FALSE 
#include "ifx_common_defs.h"
#undef IFX_TRUE
#undef IFX_FALSE

#include "IFX_DECT_Stack.h"
#include "IFX_DECT_CSU.h"

#endif /* TD_DECT */

#ifdef HAVE_DRV_TAPI_HEADERS
   #include "drv_tapi_io.h"
#else
    #error "drv_tapi is missing"
#endif

#ifdef TAPI_VERSION4
   #include "ifxos_debug.h"
#endif

#ifdef QOS_SUPPORT
/* QOS_INIT_SESSION defined in drv_qos.h */
    #ifdef HAVE_DRV_TAPI_HEADERS
       #include "drv_tapi_qos_io.h"
    #else
        #error "drv_tapi is missing"
    #endif
#endif /* QOS_SUPPORT */

#ifdef HAVE_DRV_TAPI_HEADERS
   #include "drv_tapi_event_io.h"
#else
    #error "drv_tapi is missing"
#endif

#ifdef LINUX
#ifdef EASY336
#include <linux/ifx_svip_nat_io.h>
#endif
#endif
 
#if (defined(EASY336) || defined(XT16))
#include "lib_svip_rm.h"
#endif

/* early VxWorks versions did not have snprintf */
#if defined(VXWORKS) && defined(NO_SNPRINTF)
#define snprintf(res, n, args...) sprintf(res, ##args)
#endif /* VXWORKS && NO_SNPRINTF */

extern size_t strnlen (__const char *__string, size_t __maxlen);

#ifdef LINUX
extern int snprintf(char *str, size_t size, const char *format, ...);
extern int gethostname(char *name, size_t len);
#endif /* LINUX */

/* ============================= */
/* Debug interface               */
/* ============================= */

/** Trace redirection setting. */
typedef enum _TRACE_REDIRECTION_t
{
   /** No redirection */
   TRACE_REDIRECTION_NONE = 0,
   /** Redirection to file */
   TRACE_REDIRECTION_FILE = 1,
   /** Redirection to local syslog */
   TRACE_REDIRECTION_SYSLOG_LOCAL = 2,
   /** Redirection to remote syslog */
   TRACE_REDIRECTION_SYSLOG_REMOTE = 3
} TRACE_REDIRECTION_t;

/** Trace levels */
typedef enum _DEBUG_LEVEL_t
{
   /** None reporting */
   DBG_LEVEL_OFF = TD_OS_PRN_LEVEL_OFF,
   /** Error reporting */
   DBG_LEVEL_HIGH = TD_OS_PRN_LEVEL_HIGH,
   /** Status reporting */
   DBG_LEVEL_NORMAL = TD_OS_PRN_LEVEL_NORMAL,
   /** More info reporting */
   DBG_LEVEL_LOW = TD_OS_PRN_LEVEL_LOW 
} DEBUG_LEVEL_t;

/**
   Prints a trace message and sends it through communication socket.

   \param name     - Name of the trace group
   \param level    - level of this message
   \param message  - a printf compatible formated string + opt. arguments

   \return

   \remark
      The string will be redirected via printf if level is higher or equal to
      the actual level for this trace group ( please see SetTraceLevel ).
*/
#define TRACE(name,level,message) \
      do \
      { \
         TD_OS_PRN_USR_DBG_NL(name,level,message); \
         if((IFX_TRUE == g_pITM_Ctrl->fITM_Enabled) && \
            (level >= g_pITM_Ctrl->nComDbgLevel) && \
            (COM_MODULAR_TEST == g_pITM_Ctrl->nTestType)) \
         { \
            if(NO_SOCKET != g_pITM_Ctrl->nComOutSocket)\
            { \
               COM_GENERATE_STRING message;\
               COM_SendTraceMessage(level, g_buf);\
            } \
         } \
      } while(0)

/** print message */
#define TAPIDEMO_PRINTF(message) \
      do \
      { \
          printf message; \
          /* if modular test send msg to control PC */ \
          if((IFX_TRUE == g_pITM_Ctrl->fITM_Enabled) && \
             (COM_MODULAR_TEST == g_pITM_Ctrl->nTestType)) \
          { \
             if(NO_SOCKET != g_pITM_Ctrl->nComOutSocket)\
             { \
                  COM_GENERATE_STRING message;\
                  COM_SendTraceMessage(DBG_LEVEL_OFF, g_buf);\
             } \
          } \
      } while(0)

TD_OS_PRN_USR_MODULE_DECL(TAPIDEMO);

/* ============================= */
/* Debug interface               */
/* ============================= */

/* macros for asserts */
#ifdef LINUX
#define TD_ASSERT(expr, fd, pBoard, dev, ch) if(!(expr)) {                    \
      printf(#expr" failed in file \"%s\" on line %d, %s dev %d, "            \
      "ch %d\n",                                                              \
      __FILE__, __LINE__, pBoard->pszBoardName, dev, ch);                     \
      Common_ErrorHandle (fd, pBoard, dev);                                   \
   }
#if 0
   else                                                                       \
   {                                                                          \
     TRACE (TAPIDEMO, DBG_LEVEL_LOW, (#expr", %s dev %d, ch %d\n",            \
     pBoard->pszBoardName, dev, ch));                                         \
   }
#endif
#endif
#ifdef VXWORKS
#define TD_ASSERT(expr, fd, pBoard, dev, ch) if(!(expr)) {                    \
      IFX_char_t* sFileName = __FILE__;                                       \
      Common_StripPath(&sFileName);                                           \
      printf(#expr" failed in file \"%s\" on line %d, %s dev %d, "            \
      "ch %d\n",                                                              \
      sFileName, __LINE__, pBoard->pszBoardName, dev, ch);                    \
      Common_ErrorHandle (fd, pBoard, dev);                                   \
   }
#if 0
   else                                                                       \
   {                                                                          \
     TRACE (TAPIDEMO, DBG_LEVEL_LOW, (#expr", %s dev %d, ch %d\n",            \
     pBoard->pszBoardName, dev, ch));                                         \
   }
#endif
#endif

/** Check if pointer is NULL.
       ptr  - pointer to check
       ret   - value to retunr in case of error. */
#define TD_PTR_CHECK(ptr, ret) if(ptr == IFX_NULL) {                           \
        TRACE(TAPIDEMO, DBG_LEVEL_HIGH,                                        \
              ("Error, Invalid pointer ("#ptr"). (File: %s, line: %d)\n",      \
               __FILE__, __LINE__));                                           \
        return ret;                                                            \
        }

/** Check if pointer is NULL for void functions.
       ptr  - pointer to check. */
#define TD_PTR_CHECK_VOID(ptr) if(ptr == IFX_NULL) {                           \
        TRACE(TAPIDEMO, DBG_LEVEL_HIGH,                                        \
              ("Error, Invalid pointer ("#ptr"). (File: %s, line: %d)\n",      \
               __FILE__, __LINE__));                                           \
        return;                                                                \
        }

/** Check condition and print value of parameters that will help in debug.
       expr  - condition to check, if true then return ret
       param - parameter to print in case of expr being false
       ret   - value to return in case of error. */
#define TD_PARAMETER_CHECK(expr, param, ret) if(expr) {                        \
        TRACE(TAPIDEMO, DBG_LEVEL_HIGH,                                        \
              ("Error, Invalid parameter ("#param"=%d). "                      \
              "(File: %s, line: %d)\n",                                        \
               param, __FILE__, __LINE__));                                    \
        return ret;                                                            \
        }

/* Check if driver's function returned error. In case of error, last error
   from drivers is read.
   fct - driver's function
   ret - could be TD_RETURN or TD_NO_RETURN. If it is set to TD_RETURN
         then return is called in case of error */
#define TD_DRV_FUNC_ASSERT(expr, pPhone, fct, dev, ch, ret)                    \
        if (expr != IFX_SUCCESS)                                               \
        {                                                                      \
           TRACE(TAPIDEMO, DBG_LEVEL_HIGH,                                     \
                ("Error, Phone No %d: "#fct" failed. "                         \
                 "(%s, dev %d, ch %d) "                                        \
                 "(File: %s, line: %d)\n",                                     \
                 pPhone->nPhoneNumber, pPhone->pBoard->pszBoardName,           \
                 dev, ch, __FILE__, __LINE__));                                \
           Common_ErrorHandle (pPhone->pBoard->nDevCtrl_FD, pPhone->pBoard,    \
              dev);                                                            \
           if (ret)                                                            \
           {                                                                   \
              return IFX_ERROR;                                                \
           }                                                                   \
        }

#ifndef TD_IOCTL_TRACE

/** macro is used to call ioctl() so it can be easyly changed
   to function that for example prints ioctl name or makes error handling,
   it should be used only for TAPI drivers (not sockets) */
#define TD_IOCTL(nFd, nIoctl, pArg) \
              TD_OS_DeviceControl(nFd, nIoctl, (IFX_ulong_t) pArg)

/**
   Call ioctl with error handling

   \param nRet - ioctl return value
   \param nFd  - file descriptor on which ioctl will be used on
   \param nFct - ioctl number
   \param pArg - pointer to structure (ioctl parameter)
   \param nPhoneNo - phone number (NO_PHONE_NUMBER if not applicable)
   \param pBoard   - pointer to board
   \param ch   - channel number
   \param dev  - device number (only tapi v4, in v3 use 0)
   \param bReturn  - if set macro should end function

   */
#define TD_DBG_IOCTL(nRet, nFd, nFct, pArg, nPhoneNo, pBoard, ch, dev, bReturn)  \
        do \
        { \
           nRet = TD_OS_DeviceControl(nFd, nFct, (IFX_ulong_t) pArg); \
           if (IFX_SUCCESS > nRet) \
           { \
              TRACE(TAPIDEMO, DBG_LEVEL_HIGH, \
                    ("Err, ioctl(%d, "#nFct", %d) failed, ch %d, dev %d\n", \
                     nFd, (IFX_int32_t) pArg, ch, dev)); \
              if (NO_PHONE_NUMBER != nPhoneNo && IFX_NULL != pBoard) \
              { \
                 TRACE(TAPIDEMO, DBG_LEVEL_HIGH, \
                       ("     Phone No %d, %s,\n", \
                        nPhoneNo, pBoard->pszBoardName)); \
              } \
              TRACE(TAPIDEMO, DBG_LEVEL_HIGH, \
                    ("     File: %s, line: %d.\n", \
                     __FILE__, __LINE__)); \
              if (IFX_NULL != pBoard) \
              { \
                 Common_ErrorHandle(pBoard->nDevCtrl_FD, pBoard, dev); \
              } \
              if (TD_RETURN == bReturn) \
              { \
                 return IFX_ERROR; \
              } \
           } \
        } while (0);

/**
   Call single pin configuration. 
   
   \param nFd - GPIO port file descriptor.
   \param nFct - IOCTL command.
   \param nPort - GPIO port.
   \param nPin - GPIO pin.
   \param nValue - GPIO value.
   \param nModule - GPIO module ID.
   \param nRet - Return value: IFX_SUCCESS on ok, otherwise IFX_ERROR
*/
#ifdef OLD_BSP
#define TD_GPIO_IOCTL_PARM(nFd, nFct, nPort, nPin, nValue, nRet)\
{ \
   struct danube_port_ioctl_parm param = {0}; \
   memset(&param, 0, sizeof (param)); \
   param.port = nPort;\
   param.pin = nPin;\
   param.value = nValue;\
   nRet = TD_OS_DeviceControl(nFd, nFct, (IFX_ulong_t) &param);\
   if (IFX_SUCCESS != nRet)\
   { \
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH, \
            ("Error: ioctl: "#nFct", (%d, %d), value: %d failed.\n",\
             nPort, nPin, nValue));\
   }\
};
#else
#define TD_GPIO_IOCTL_PARM(nFd, nFct, nPort, nPin, nValue, nModule ,nRet)\
{ \
   struct ifx_gpio_ioctl_parm param = {0};\
   memset(&param, 0, sizeof (param)); \
   param.port = nPort;\
   param.pin = nPin;\
   param.value = nValue;\
   param.module = nModule;\
   nRet = TD_OS_DeviceControl(nFd, nFct, (IFX_ulong_t) &param);\
   if (IFX_SUCCESS != nRet)\
   { \
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH, \
            ("Error: ioctl: "#nFct", (%d, %d), value: %d failed.\n",\
             nPort, nPin, nValue));\
   }\
};
#endif /* OLD_BSP */

#endif /* TD_IOCTL_TRACE */

/** Check if function returned error.
   fct - function
   ret - could be TD_RETURN or TD_NO_RETURN. If it is set to TD_RETURN
         then return is called in case of error */
#define TD_FUNC_ASSERT(expr, pPhone, fct, dev, ch, ret)                        \
        if (expr != IFX_SUCCESS)                                               \
        {                                                                      \
           TRACE(TAPIDEMO, DBG_LEVEL_HIGH,                                     \
                ("Error, Phone No %d: "#fct" failed. "                         \
                 "(%s, dev %d, ch %d) "                                        \
                 "(File: %s, line: %d)\n",                                     \
                 pPhone->nPhoneNumber, pPhone->pBoard->pszBoardName,           \
                 dev, ch, __FILE__, __LINE__));                                \
           if (ret)                                                            \
           {                                                                   \
              return IFX_ERROR;                                                \
           }                                                                   \
        }

/* Play call progres tone, check if successfull and print apropriate traces */
#define TD_CPTG_PLAY_CHECK(ret, pCtrl, pPhone, nTone, pszToneName) \
   ret = Common_TonePlay(pCtrl, pPhone, nTone); \
   if (IFX_SUCCESS == ret) \
   { \
      TRACE(TAPIDEMO, DBG_LEVEL_NORMAL, \
            ("Phone No %d: Play %s.\n", pPhone->nPhoneNumber, pszToneName)); \
      COM_MOD_CONFERENCE_BUSY_TRACE(nTone); \
   } \
   else \
   { \
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH, \
            ("Phone No %d: Failed to play %s. (File: %s, line: %d)\n", \
             pPhone->nPhoneNumber, pszToneName, __FILE__, __LINE__)); \
   }

/** Add fd and set new max fd if needed */
#define TD_FD_SET(nFd, pFds, nMaxFd, nType) \
   if (TD_SELECT_DEVICE == nType) \
       TD_OS_DevFdSet(nFd, pFds); \
   else \
       TD_OS_SocFdSet(nFd, pFds); \
   if (nMaxFd < nFd) \
   { \
      nMaxFd = nFd; \
   }

/** Reset timout flag */
#define TD_TIMEOUT_RESET(nUseTimeout)  { nUseTimeout = 0; }
/** Increase timout flag - another resource(phones or FXO) needes
    to use timeout */
#define TD_TIMEOUT_START(nUseTimeout)  { nUseTimeout++; }
/** Decrease timout flag - resource(phones or FXO) doesn't need timeout now */
#define TD_TIMEOUT_STOP(nUseTimeout)   if (nUseTimeout > 0)   \
                          {   nUseTimeout--;  }
/** Check if timeout should be used */
#define TD_TIMEOUT_CHECK(nUseTimeout)  (0 != nUseTimeout)

#ifdef TAPI_VERSION4
#define TD_SET_CH_AND_DEV(oStruct, nChannel, nDevice)   \
    do \
    { \
       oStruct.ch = nChannel; \
       oStruct.dev = nDevice; \
    } while (0);
#endif /* TAPI_VERSION4 */

#ifdef TD_RM_DEBUG
/** Macro used to trace currently allocated resources.
    \param pTmpPhone - pointer to phone structure */
#define TD_RM_TRACE_PHONE_RESOURCES(pTmpPhone) \
   do \
   { \
      if (pTmpPhone->pBoard->pCtrl->pProgramArg->oArgFlags.nRM_Count) \
      { \
         TRACE(TAPIDEMO, DBG_LEVEL_NORMAL, \
               ("Phone No %d: RESOURCES:\n" \
                " ConnId %d\n" \
                " VoipId %d\n" \
                " Gen %d\n" \
                " Det %d\n" \
                " Coder %d\n" \
                " Lec %d\n", \
                pTmpPhone->nPhoneNumber, \
                pTmpPhone->oRM_Debug.nConnIdCnt, \
                pTmpPhone->oRM_Debug.nVoipIdCnt, \
                pTmpPhone->oRM_Debug.nGenCnt, \
                pTmpPhone->oRM_Debug.nDetCnt, \
                pTmpPhone->oRM_Debug.nCodecCnt, \
                pTmpPhone->oRM_Debug.nLecCnt)); \
      } \
   } while (0)
#endif /* TD_RM_DEBUG */

/** returns smaller of two numbers
    \param m_number1 - first number to check
    \param m_number2 - second number to check */
#define TD_GET_MIN(m_number1, m_number2) \
   (((m_number1) > (m_number2)) ? (m_number2) : (m_number1))

/* ============================= */
/* Global Defines                */
/* ============================= */

/* those two enums are different only by name
   so to simplyfy code define is used */
#ifdef TAPI_VERSION4
#define  TD_LINE_MODE_t    IFX_TAPI_LINE_MODE_t
#else /* TAPI_VERSION4 */
#define  TD_LINE_MODE_t    IFX_TAPI_LINE_FEED_t
#endif /* TAPI_VERSION4 */

typedef struct _PHONE_t PHONE_t;
typedef struct _FXO_t FXO_t;
typedef struct _BOARD_t BOARD_t;
typedef struct _CTRL_STATUS_t CTRL_STATUS_t;
typedef struct _PROGRAM_ARG_t PROGRAM_ARG_t;
typedef struct _TAPIDEMO_DEVICE_CPU_t TAPIDEMO_DEVICE_CPU_t;
typedef struct _TAPIDEMO_DEVICE_FXO_t TAPIDEMO_DEVICE_FXO_t;

/** UDP port for voice data (VOICE_UDP_PORT .. (VOICE_UDP_PORT +
 * 2*MAX_SYS_CH_RES))
    According to the standard it is recomended to send RTP packets on even
    ports while odd ones are reserved for RTCP */
#ifdef EASY336
enum { VOICE_UDP_PORT = SVIP_UDP_FROM };
#else /* EASY336 */
enum { VOICE_UDP_PORT = 5002 };
#endif /* EASY336 */

#if (defined(EASY336) || defined(XT16))
/** Wait time for calibration */
enum { SVIP_XT16_WAIT_CALIBRATION_POLLTIME = 300 };
#endif

/** UDP port for making/ending voip/pcm/... calls. */
enum { ADMIN_UDP_PORT = 5000 };

#ifdef USE_FILESYSTEM
enum { MAX_FULL_FILE_LEN = 256 };
#endif /* USE_FILESYSTEM */
/** Phone digit representation */
typedef enum
{
   DIGIT_ONE = 0x01,
   DIGIT_TWO = 0x02,
   DIGIT_THREE = 0x03,
   DIGIT_FOUR = 0x04,
   DIGIT_FIVE = 0x05,
   DIGIT_SIX = 0x06,
   DIGIT_SEVEN = 0x07,
   DIGIT_EIGHT = 0x08,
   /** Used to start PCM calls */
   DIGIT_NINE = 0x09,
   DIGIT_STAR = 0x0A,
   DIGIT_ZERO = 0x0B,
   /** Used to start conference, calling another party */
   DIGIT_HASH = 0x0C
} DIGIT_t;

/** Sign for no parameters for ioctl() call */
enum { NO_PARAM = 0 };

/** Flag for no device FD */
enum { NO_DEVICE_FD = -1 };

/** Flag for no GPIO FD */
enum { NO_GPIO_FD = -1 };

/** Flag for no socket */
enum { NO_SOCKET = -1 };

/** Flag for no pipe */
enum { NO_PIPE = -1 };

/** Flag for no file */
enum { NO_FILE = -1 };

/** Flag for no channel file descriptor */
enum { NO_CHANNEL_FD = -1 };

/** Flag for no channel */
enum { NO_CHANNEL = -1 };

/** Flag for no used channel */
enum { NO_USED_CHANNEL = -1 };

/** Flag for not set value */
enum { TD_NOT_SET = -1 };

/** Flag for no board idx */
enum { NO_BOARD_IDX = -1 };

/** Flag for no phone number */
enum { NO_PHONE_NUMBER = -1 };

/** Flag for PCM call, will be send to external phone */
enum { CALL_FLAG_PCM = 0x80 };

/** Flag for VoIP call, will be send to external phone */
enum { CALL_FLAG_VOIP = 0x00 };

/** Flag for VoIP call, will be send to external phone */
enum { CALL_FLAG_UNKNOWN = 0xFF };

/** Flag indicating start of command message */
enum { COMM_MSG_START_FLAG = 0xFF };

/** Flag indicating end of command message */
enum { COMM_MSG_END_FLAG = 0x77 };

/** Max. length of phone number or number of digits than can be dialed */
enum { MAX_DIALED_NUM = 24 };

/** Phone number lenght for TAPI3, e.g. Danube board */
enum { TAPI3_PHONE_NUM_LEN = 1 };

 /** Phone number lenght for TAPI4, e.g. SVIP board */
enum { TAPI4_PHONE_NUM_LEN = 3 };

/** Min. value of phone number */
#ifdef TAPI_VERSION4
   enum { MIN_PHONE_NUM = 100 };
#else
   enum { MIN_PHONE_NUM = 1 };
#endif

/** Max. length of first 3 number of IP address for different network */
enum { DIFF_NET_IP_LEN = 12 };

/** Use restor default option. */
enum { RESTORE_DEFAULT = -1 };

/** Length for 32 bit IP address in characters "xxx.xxx.xxx.xxx". */
enum { IP_ADDRESS_LEN = 16 };

/** Length for MAC address in characters "xx:xx:xx:xx:xx:xx". */
enum { MAC_ADDRESS_LEN = 18 };

/** Length for port number "nnnnn, 0..65535". */
enum { PORT_NUM_LEN = 5 };

/** Turn off CID support */
enum { TD_TURN_OFF_CID = 10 };

/** call progress tones names */
typedef enum _CPT_NAME_t
{
   NO_TONE = -1,
   DIAL_TONE,
   SECOND_DIAL_TONE,
   BUSY_TONE,
   RINGBACK_TONE,
   CONGESTION_TONE,
   HOWLER_TONE,
   STOP_TONE,
   MAX_TONE
}CPT_NAME_t;

/** Call progress tones indexes for more check drv_tapi_tone.c
    or Predefined Tones Table in tapi documentation. */
typedef enum _TONE_IDX_t
{
   DIALTONE_IDX = 25,
   RINGBACK_TONE_IDX = 26,
   BUSY_TONE_IDX = 27
} TONE_IDX_t;

#ifdef EASY336
/** resource type */
typedef enum _RESOURCE_TYPE_t
{
   RES_NONE = 0x00,
   RES_VOIP_ID = 0x01,
   RES_CONN_ID = 0x02,
   RES_DET = 0x04,
   RES_GEN = 0x08,
   RES_LEC = 0x10,
   RES_CODEC = 0x20,
   /** all resources voip_id, conn_id, sig_det, sig_gen, lec and codec */
   RES_ALL = 0xFF
} RESOURCE_TYPE_t;
#endif /* EASY336 */

/** number of state machines */
enum { MAX_STATE_MACHINE = 2 };

#ifndef ETH_ALEN
/** Octets in one ethernet address */
#define ETH_ALEN 6
#endif

/** mask that can be used to get lowest IP byte */
#define TD_IP_LOWEST_BYTE_MASK            0x000000FF

/** mask to get state info part */
#define TD_STATE_MASK            0xFFFF0000
/** if this flag is set then during this state packets are transmited between
    phones - used by tapidemo packet handling functions */
#define TD_STATE_DATA_TRANSMIT   0x00010000
/** if this flag is set then this state is used for fax, modem or t.38 */
#define TD_STATE_FAX_MODEM       0x00020000
/** if this flag is set then this state is used for conferences */
#define TD_STATE_CONFERENCE      0x00040000
/** if this flag is set then during this state
    tone is played on phone channel */
#define TD_STATE_TONE_PLAY       0x00080000
/** if this flag is set then this state is used for BACUS_TEST */
#define TD_STATE_ITM             0x01000000



/** max size of message that will be received - should not be changed
    in future versions of TAPIDEMO */
#define TD_MAX_COMM_MSG_SIZE              512
/** size of COMM_MSG_DATA_1_t structure */
#define TD_MAX_COMM_MSG_SIZE_DATA_1       32

/** max size of peer name that is send in message through
    communication socket */
#define TD_MAX_PEER_NAME_LEN        20

/** message version
 * change history:
 * 2010-05-24  - creation of message versioning V=01 */
#define TD_COMM_MSG_CURRENT_VERSION   0x01

/** returns non zero value if nInfo flag is set
      \param nState - phone state
      \param nInfo  - state flag to check
 */
#define TD_GET_STATE_INFO(nState, nInfo) (((nState) & TD_STATE_MASK) & (nInfo))

/** States of state machine. Enums represent names of states.
    TD_STATE_... flags are set to notify about state aditional features,
    e.g. S_OUT_DIALTONE = 0x0001 | TD_STATE_TONE_PLAY - during this state tone
    is played. This can help to turn off tone when state is changed e.g.
    TD_GET_STATE_INFO(nState, TD_STATE_TONE_PLAY) will return non zero value
    if tone is played during this state. */
typedef enum
{
   /** Phone is on-hook. */
   S_READY           = 0x0000,
   /** Phone is off-hook, and tone is played(dialtone),
       waiting for dialed digits. */
   S_OUT_DIALTONE    = 0x0001 | TD_STATE_TONE_PLAY,
   /** Phone is off-hook, one or more digits were dialed,
       waiting for more digits to complete number that can be called. */
   S_OUT_DIALING     = 0x0002,
   /** FXO call in progress. */
   S_OUT_FXO_CALL    = 0x0003,
   /** Phone number was dialed waiting for peer confirmation. */
   S_OUT_CALLING     = 0x0004,
   /** Waiting for peer to make hook-off, tone is played(ringback). */
   S_OUT_RINGBACK    = 0x0005 | TD_STATE_TONE_PLAY,
   /** Connection in progress, data should be transmitted. */
   S_ACTIVE          = 0x0006 | TD_STATE_DATA_TRANSMIT,
   /** Connection failed or peer disconected, tone is played(busy tone). */
   S_BUSYTONE        = 0x0007 | TD_STATE_TONE_PLAY,
   /** Phone is ringing, waiting for hook-off to establish connection. */
   S_IN_RINGING      = 0x0008,
   /** Phone is waiting for dialed digits to establish new connection,
       phone is in conference, tone is played(dialtone). */
   S_CF_DIALTONE     = 0x0009 | TD_STATE_CONFERENCE | TD_STATE_TONE_PLAY,
   /** One or more digits were dialed, waiting for more digits to complete
       number that can be called, phone is in conference. */
   S_CF_DIALING      = 0x000A | TD_STATE_CONFERENCE,
   /** Phone number was dialed waiting for peer confirmation, phone is in
       conference. */
   S_CF_CALLING      = 0x000B | TD_STATE_CONFERENCE,
   /** Waiting for peer to make hook-off, phone is in conference,
      tone is played(ringback). */
   S_CF_RINGBACK     = 0x000C | TD_STATE_CONFERENCE | TD_STATE_TONE_PLAY,
   /** Conference connections in progress, data should be transmitted,
       phone is in conference. */
   S_CF_ACTIVE       = 0x000D | TD_STATE_DATA_TRANSMIT | TD_STATE_CONFERENCE,
   /** Connection failed , phone is in conference, tone is played(busy tone). */
   S_CF_BUSYTONE     = 0x000E | TD_STATE_CONFERENCE | TD_STATE_TONE_PLAY,
   /** Tone playing is tested, ITM specific state. */
   S_TEST_TONE       = 0x000F | TD_STATE_ITM,
   /** fax/modem transmission in progress, data should be transmitted,
       fax/modem specific state. */
   S_FAX_MODEM       = 0x0010 | TD_STATE_DATA_TRANSMIT | TD_STATE_FAX_MODEM,
   /** t.38 transmission in progress, data should be transmitted,
       fax/modem specific state. */
   S_FAXT38          = 0x0011 | TD_STATE_DATA_TRANSMIT | TD_STATE_FAX_MODEM,
   /** t.38 transmission request recived, fax/modem specific state. */
   S_FAXT38_REQ      = 0x0012 | TD_STATE_FAX_MODEM,
   /** t.38 transmission end recived, fax/modem specific state. */
   S_FAXT38_END      = 0x0013 | TD_STATE_FAX_MODEM,
   /** dect handset is not registered. */
   S_DECT_NOT_REGISTERED   = 0x0014,
   /** Last state - used to mark end of state tables */
   S_END             = 0xFFFF
} STATE_MACHINE_STATES_t;

/** Internal events */
typedef enum
{
   IE_NONE = 0,
   IE_READY,
   IE_HOOKOFF,
   IE_HOOKON,
   IE_FLASH_HOOK,
   IE_DIALING,
   IE_RINGING,
   IE_RINGBACK,
   IE_ACTIVE,
   IE_CONFERENCE,
   IE_END_CONNECTION,
   IE_BUSY,
   IE_CNG_FAX,
   IE_CNG_MOD,
   IE_AM,
   IE_PR,
   IE_CED,
   IE_CED_END,
   IE_DIS,
   IE_V8BIS,
   IE_V21L,
   IE_V18A,
   IE_V27,
   IE_BELL,
   IE_V22,
   IE_V22ORBELL,
   IE_V32AC,
   IE_CAS_BELL,
   IE_V21H,
   IE_END_DATA_TRANSMISSION,
   IE_CONFIG,
   IE_CONFIG_DIALING,
   IE_CONFIG_HOOKON,
   IE_CONFIG_PEER,
   IE_EXT_CALL_SETUP,
   IE_EXT_CALL_PROCEEDING,
   IE_EXT_CALL_ESTABLISHED,
   IE_EXT_CALL_WRONG_NUM,
   IE_EXT_CALL_NO_ANSWER,
   IE_EXT_CALL_WORKAROUND_EASY508xx,
   IE_T38_ERROR,
   IE_T38_REQ,
   IE_T38_ACK,
   IE_T38_END,
   IE_DECT_REGISTER,
   IE_DECT_UNREGISTER,
   IE_TEST_TONE,
#if defined(XT16) || defined(WITH_VXT)
   IE_FAULT_LINE_GK_LOW,
#endif /* XT16 || WITH_VXT */
   /** this event is never generated,
      it is only used to mark end of EVENT_ACTIONS_t table */
   IE_END
} INTERNAL_EVENTS_t;

/** structure with event and pointer to action that should be taken
    when this event coccures */
typedef struct _EVENT_ACTIONS_t
{
   INTERNAL_EVENTS_t nEvent;
   IFX_return_t (*pAction)();
} EVENT_ACTIONS_t;

/** structure with state and pointer to table of EVENT_ACTIONS_t
    with all handled events */
typedef struct _STATE_COMBINATION_t
{
   STATE_MACHINE_STATES_t nState;
   EVENT_ACTIONS_t* pEventActions;
} STATE_COMBINATION_t;

/** structure with current state
    and pointer to table of STATE_COMBINATION_t for all states */
typedef struct _STATE_MACHINES_t
{
   STATE_MACHINE_STATES_t nState;
   STATE_COMBINATION_t* pStateCombination;
} STATE_MACHINES_t;

/** types of state machines */
typedef enum _STATE_MACHINE_TYPE_t
{
   /** configuration state machine */
   CONFIG_SM = 0,
   /** FXS state machine */
   FXS_SM = 1
} STATE_MACHINE_TYPE_t;

/** call direction */
typedef enum _CALL_DIRECTION_TYPE_t
{
   /** no call direction - no call */
   CALL_DIRECTION_NONE = 0,
   /** transmit direction - set for caller phone */
   CALL_DIRECTION_TX,
   /** receive direction - set for called phone */
   CALL_DIRECTION_RX
} CALL_DIRECTION_TYPE_t;


/** max enum ID */
#define TD_MAX_ENUM_ID  0xffffffff
/** max length of name array */
#define TD_MAX_NAME     64

/** Max path length to download files. */
enum { MAX_PATH_LEN = 256 };

/** Max line's length in .tapidemorc. */
enum { MAX_LINE_LEN = 512 };


/** structure to assigne enum name to its value (id) */
typedef struct _TD_ENUM_2_NAME_t
{
   IFX_int32_t nID;
   IFX_char_t* rgName;
} TD_ENUM_2_NAME_t;

/** error code for Common_Enum2Name() function. This enums are used as
    index for TD_rgEnum2NameErrorCode[] table. */
typedef enum _TD_ENUM_2_NAME_ERR_CODE_t
{
   /** enum value not found in the table */
   ENUM_VALUE_NOT_FOUND = 0,
   /** NULL pointer */
   ENUM_NULL_POINTER
} TD_ENUM_2_NAME_ERR_CODE_t;

/** enum used by macros to decide if call return. */
typedef enum _TD_RETURN_CODE_t
{
   TD_NO_RETURN = 0,
   TD_RETURN = 1
} TD_RETURN_CODE_t;
/** enum used as an argument to mapping functions */
typedef enum _TD_MAP_ADD_REMOVE_t
{
   /** map channels */
   TD_MAP_ADD = 0,
   /** unmap channels */
   TD_MAP_REMOVE = 1
} TD_MAP_ADD_REMOVE_t;
/* Wide/Narrow-Band settings */
typedef enum _TD_BAND_WIDE_NARROW_t
{
   /* NarrowBand */
   TD_BAND_NARROW = 0,
   /* WideBand */
   TD_BAND_WIDE = 1
} TD_BAND_WIDE_NARROW_t;

/** number of star digits that must be used to dial phone number
    with full IP address */
#define NUM_OF_STARS_IN_IP 5

#if (!defined(EASY336) && !defined(XT16))
/** number of digits of phone number x */
#define TD_PHONE_NUM_LEN      1
#else /* (!defined(EASY336) && !defined(XT16)) */
/** number of digits of phone number xxx */
#define TD_PHONE_NUM_LEN      3
#endif /* (!defined(EASY336) && !defined(XT16)) */

#define TD_PHONE_STRING       "Phone No : "

/** length of string holding representation of two uint32 with string end */
#define TD_TWO_UINT32_STRING_LEN     21

/** JB(jitter buffer) data format */
#ifdef TAPI_VERSION4
#define TD_JB_DATA_FORMAT        "%u"
#else /* TAPI_VERSION4 */
#define TD_JB_DATA_FORMAT        "%u"
#endif /* TAPI_VERSION4 */

/** RTCP data format */
#ifdef TAPI_VERSION4
#define TD_RTCP_DATA_FORMAT        "%u"
#else /* TAPI_VERSION4 */
#define TD_RTCP_DATA_FORMAT        "%u"
#endif /* TAPI_VERSION4 */

#ifdef LINUX
#ifdef HAVE_DRV_VMMC_HEADERS
/** Default path to download files. */
#define DEF_DOWNLOAD_PATH        "/lib/firmware/"
/** alternate download path for backward compability */
#define TD_ALTERNATE_DOWNLOAD_PATH     "/opt/ifx/downloads/"
#else /* HAVE_DRV_VMMC_HEADERS */
/** Default path to download files. */
#define DEF_DOWNLOAD_PATH        "/opt/ifx/downloads/"
#endif /* HAVE_DRV_VMMC_HEADERS */
#endif /* LINUX */
#ifdef VXWORKS
/** Default path to download files. */
#define DEF_DOWNLOAD_PATH        "/romfs/"
#endif /* VXWORKS */

/** path to device files directory */
#define TD_DEV_PATH              "/dev/"

/** name of file with path to downloads (FW and BBD files) */
#define DOWNLOAD_FILE_NAME       ".tapidemorc"
/** name of variable with path to downloads in DOWNLOAD_FILE_NAME */
#define DOWNLOAD_VARIABLE       "DOWNLOADS"


/** max device name length with number e.g. /dev/vmmc10 */
#define MAX_DEV_NAME_LEN_WITH_NUMBER      50

/** string is invalid or some other problem occured */
#define TD_INVALID_STRING     "invalid string"
/* --------------------------------------------------------- */
/*                    CONFERENCE START                       */
/* --------------------------------------------------------- */

/** representing local or remote call */
typedef enum _CALL_TYPE_t
{
   UNKNOWN_CALL_TYPE = 0,
   EXTERN_VOIP_CALL = 1,
   LOCAL_CALL = 2,
   PCM_CALL = 3,
   LOCAL_BOARD_CALL = 5,
   FXO_CALL = 6,
   LOCAL_PCM_CALL = 7,
   LOCAL_BOARD_PCM_CALL = 8,
   DTMF_COMMAND = 9
} CALL_TYPE_t;

/** Conference status */
/** Also used for flag if not in conference */
enum { NO_CONFERENCE = 0 };

#ifndef EASY336
/** Max peers in conference */
enum { MAX_PEERS_IN_CONF = 4 };
#else /* EASY336 */
/** Max peers in conference */
enum { MAX_PEERS_IN_CONF = 16 };
#endif /* EASY336 */

/** How many conferences can we have. Cannot be bigger than number of phones. */
#ifndef EASY336
enum { MAX_CONFERENCES = 2 };
#else /* EASY336 */
enum { MAX_CONFERENCES = 16 };
#endif /* EASY336 */

/** Flag for no free data channel */
enum { NO_FREE_DATA_CH = -1 };

/** Status no external peer in conference */
enum { NO_EXTERNAL_PEER = 0 };

/** Status no PCM peer in conference */
enum { NO_PCM_PEER = 0 };

/** Status no local peer in conference */
enum { NO_LOCAL_PEER = 0 };

/** No value is set */
enum { TD_NO_VALUE = -1 };

/** No device is specified */
enum { NO_DEV_SPECIFIED = 0 };

/** used to mark if function should return device or channel data */
enum {
   /** device data is used */
   TYPE_DEVICE = 1,
   /** channel data is used */
   TYPE_CHANNEL
};

/** for pipe() fd[0] is readable and fd[1] is writeable,
    enum used get readable and writeable FDs */
typedef enum _TD_PIPE_IN_OUT_t
{
   /* readable pipe */
   TD_PIPE_OUT = 0,
   /* writeable pipe */
   TD_PIPE_IN = 1
}TD_PIPE_IN_OUT_t;

#ifdef TD_DECT
/** holds call handle and handset ID/number for one channel */
typedef struct _TD_DECT_CHANNEL_TO_HANDSET_t
{
   /** call handle - pointer */
   IFX_int32_t nCallHdl;
   /** handset ID/number */
   IFX_int32_t nHandsetId;
}TD_CHANNEL_TO_HANDSET_t;

/** control structure for DECT timeout for DIALTONE for single channel */
typedef struct _TD_DECT_DIALTONE_TIMEOUT_CTRL_t
{
   /** start time */
   IFX_time_t timeDialtoneStart;
   /** set if timeout is used */
   IFX_boolean_t bTimeoutActive;
}TD_DECT_DIALTONE_TIMEOUT_CTRL_t;

/** structure used to hold dect stack specific data */
typedef struct _TD_DECT_STACK_CTRL_t
{
   /** pipe file descriptor used to write data */
   IFX_int32_t nDectPipeFdIn;
   /** pipe file pointer used to write data */
   TD_OS_Pipe_t* pDectPipeFpIn; 
   /** number of DECT channels */
   IFX_int32_t nDectChNum;
   /** DECT channel data (handset ID and call handle) */
   TD_CHANNEL_TO_HANDSET_t* pChToHandset;
   /** DECT initialized flag */
   IFX_boolean_t nDectInitialized;
   /** Pointer to path to FW files to download */
   IFX_char_t* psPathToDwnldFiles;
   /** dialtone timeout data */
   TD_DECT_DIALTONE_TIMEOUT_CTRL_t* pDialtoneTimeoutCtrl;
   /** Communication socket to send data For Production Test Tool */
   IFX_int32_t nToCliFd;
   /** used for communication with Production Test Tool  */
   struct sockaddr_un oToCliFILE_name;
   /* size of oToCliFILE_name */
   IFX_int32_t nToCliSize;
}TD_DECT_STACK_CTRL_t;

/** DECT events sent to tapidemo currently not all are used */
typedef enum _TD_DECT_EVENT_t
{
   TD_DECT_EVENT_NONE = 0,
   TD_DECT_EVENT_HOOK_OFF,
   TD_DECT_EVENT_HOOK_ON,
   TD_DECT_EVENT_FLASH,
   TD_DECT_EVENT_DIGIT,
   TD_DECT_EVENT_REGISTER,
   TD_DECT_EVENT_WIDEBAND_ENABLE,
   TD_DECT_EVENT_WIDEBAND_DISABLE,
   TD_DECT_EVENT_DECT_WORKING,
   TD_DECT_EVENT_END
} TD_DECT_EVENT_t;

/** dect stack data send in DECT message */
typedef struct _TD_DECT_STACK_DATA_t
{
   /** received diggits buffer */
   IFX_char_t acRecvDigits[IFX_DECT_MAX_DIGITS];
}TD_DECT_STACK_DATA_t;

/** message that is send from callback functions */
typedef struct _TD_DECT_MSG_t
{
   /** DECT channel number */
   IFX_int32_t nDectCh;
   /** event number */
   TD_DECT_EVENT_t nDectEvent;
   /** additional datal */
   TD_DECT_STACK_DATA_t oData;
}TD_DECT_MSG_t;

/** Phone Type */
typedef enum _PHONE_TYPE_t
{
   /** invalid phone type */
   PHONE_TYPE_INVALID,
   /** analog phone type */
   PHONE_TYPE_ANALOG,
   /** dect phone type */
   PHONE_TYPE_DECT
} PHONE_TYPE_t;
#endif /* TD_DECT */

#ifdef TD_FAX_MODEM
/**
   Mode of FAX transmission.
 */
typedef enum _TD_FAX_MODE_t
{
   /** Only transparetn fax transmission can be used */
   TD_FAX_MODE_TRANSPARENT = 0,
   /** Call is started as T.38 transmission (Fax Data Pump) */
   TD_FAX_MODE_FDP,
   /** Fax transmission is configured according to detected signals */
   TD_FAX_MODE_DEAFAULT
} TD_FAX_MODE_t;
#endif /* TD_FAX_MODEM */

#if (defined(TD_FAX_MODEM) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW))
/**
   FAX T.38 configuration data.
 */
 typedef struct _TD_T38_CONFIG_t
 {
#ifdef TD_T38_FAX_TEST
    /** fax stack session configuration */
    IFX_TAPI_T38_FAX_CFG_t oT38FaxCfg;
    /** fax data pump(fdp) configuration */
    IFX_TAPI_T38_FDP_CFG_t oT38FdpCfg;
#endif /* TD_T38_FAX_TEST */
    /** session configuration */
    IFX_TAPI_T38_SESS_CFG_t oT38SessCfg;
 } TD_T38_CONFIG_t;

#ifdef TD_T38_FAX_TEST
 /**
   Structure with FAX T.38 TEST data
 */
 typedef struct _TD_T38_FAX_TEST_CTRL_t
 {
    /** socket fd */
    TD_OS_socket_t nComSocket;
    /** flag to check if fax test should be use,
        set to false if fax test initialization fails */
    IFX_boolean_t bUseFaxTest;
 } TD_T38_FAX_TEST_CTRL_t;
#endif /* TD_T38_FAX_TEST */
#endif /* (defined(TD_FAX_MODEM) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW)) */

/** Conference support */
/* no conferencing support in static RM */
typedef struct _CONFERENCE_t
{
   /** Conference status, IFX_TRUE this conference is active, IFX_FALSE
       is free for use. */
   IFX_boolean_t fActive;

   /** Number of external peers in conference */
   IFX_int32_t nExternalPeersCnt;

   /** Number of PCM peers (channels) in conference */
   IFX_int32_t nPCM_PeersCnt;

   /** Number of local peers in conference */
   IFX_int32_t nLocalPeersCnt;
} CONFERENCE_t;


/* --------------------------------------------------------- */
/*                    CONFERENCE END                         */
/* --------------------------------------------------------- */


/** Represents called peer */
typedef struct _PEER_t
{
   /** Called phone number */
   IFX_int32_t nPhoneNum;

   /** Structure for external phone */
   struct {
      /** Data channel number of this peer. */
      IFX_int32_t nCh;
      /** Addr we send data to */
      TD_OS_sockAddr_t oToAddr;
      /** Device number of this peer. */
      IFX_uint16_t nDev;
      /** MAC address of this peer. */
      IFX_uint8_t MAC[ETH_ALEN];
      /* Keeps dialed number, except network part -
         used in external connection */
      IFX_char_t rgCollectNumber[4];
   } oRemote;
   /** Structure for PCM phone */
   struct {
      /** Addr we send data to */
      TD_OS_sockAddr_t oToAddr;
   } oPCM;
   /** Pointer to connection structure - other local phone */
   struct {
      /** Pointer to other phone */
      struct _PHONE_t* pPhone;
      /** Flag status used for retrieving right timeslot index pair.
          Called phone represents slave, caller phone represents master.
          IFX_TRUE if we are slave, otherwise IFX_FALSE for master */
      IFX_boolean_t fSlave;
      /** if set wideband is used for DECT<->DECT call */
      IFX_boolean_t fDectWideband;
   } oLocal;
} PEER_t;


/** Represents connection */
typedef struct _CONNECTION_t
{
   /** Connection status, IFX_TRUE - active, IFX_FALSE - not active
       Set only for caller when changing state from ringback to active
       and ringing to active. */
   IFX_boolean_t fActive;

   /** Connection status, IFX_TRUE - active, IFX_FALSE - not active */
   /** workaround for new SM use fActive flag */
   IFX_boolean_t fPCM_Active;

   /* ---------------------------------------------------------------
          Information of channels, sockets used to call the phone
    */

   /** 2 - local, 1 - remote, 3 - pcm, ... */
   CALL_TYPE_t nType;

   /** For external actions (IP + port) */
   TD_OS_sockAddr_t oUsedAddr;

   /** For external actions (MAC) */
   IFX_uint8_t oUsedMAC[ETH_ALEN];

   /** Socket on which call to peer was made (only VoIP or PCM connections). */
   IFX_int32_t nUsedSocket;

   /** Channel on which call to peer was made (can be data, pcm).
       It depends of the flag fType. */
   IFX_int32_t nUsedCh;

   /** Channel file descriptor */
   IFX_int32_t nUsedCh_FD;

   /** External dialing status, IFX_TRUE this is external dialing */
   IFX_boolean_t bExtDialingIn;

#if (defined(TD_FAX_MODEM) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW))
   /** Informs if peer finished T.38 transmission. IFX_TRUE - peer finished
       T.38 transmission. */
   IFX_boolean_t bT38_PeerEndTransm;
#endif /* (defined(TD_FAX_MODEM) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW)) */

   /** Time when phone started waiting on second peer in external call */
   IFX_time_t timeStartExtCalling; 

#ifdef EASY336
   /** pointer to used voipID */
   IFX_int32_t voipID;
#endif /* EASY336 */

   /** Feature number to use/configure */
   IFX_int32_t nFeatureID;

   /** PCM call specific data */
   struct {
      /** Timeslot RX index */
      IFX_int32_t nTimeslot_RX;
      /** Timeslot TX index */
      IFX_int32_t nTimeslot_TX;
   } oPCM;


   /* -------------------------------------------
               Information of called phone
    */
   /** Called peer */
   PEER_t oConnPeer;

#ifdef QOS_SUPPORT
    struct {
        /** QOS_SUPPORT: 1 - session is started, 0 - session stopped */
        IFX_int32_t nInSession;
        /** Session pair */
        QOS_INIT_SESSION oPair;
    } oSession;
#endif

} CONNECTION_t;

#ifdef EASY336
/** structure used to count LineIDs asigned to ConnID */
typedef struct _TD_CONN_ID_LINE_ID_CNT_t
{
   /** number of ConnID */
   IFX_int32_t nConnID;
   /** number of LineIDs */
   IFX_int32_t nLineID_Cnt;
} TD_CONN_ID_LINE_ID_CNT_t;
#endif /* EASY336 */

#ifdef TD_RM_DEBUG
/** structure used to count allocated resources
    created for debug purposes */
typedef struct _TD_RM_DEBUG_t
{
   /** number of voip IDs */
   IFX_int32_t nVoipIdCnt;
   /** number of connection IDs */
   IFX_int32_t nConnIdCnt;
   /** number of signal detectors */
   IFX_int32_t nDetCnt;
   /** number of signal generators */
   IFX_int32_t nGenCnt;
   /** number of LEC resources */
   IFX_int32_t nLecCnt;
   /** number of codec resources */
   IFX_int32_t nCodecCnt;
} TD_RM_DEBUG_t;
#endif /* TD_RM_DEBUG */

/** holds array with ascii representation of dialed digits with
    dialed digits count */
typedef struct _TD_CID_DTMF_DIGITS_t
{
   /** array of dialed digits in ascii */
   IFX_char_t     acDialed[MAX_DIALED_NUM];
   /** number of dialed digits */
   IFX_uint8_t    nCnt;
   /** if IFX_TRUE then DTMF detecion
       on data channel connected to FXO is enabled */
   IFX_boolean_t    nDTMF_DetectionEnabled;
}TD_CID_DTMF_DIGITS_t;

/** Connection structure, represents abstraction object. Basically represents
    phone with phone channel and device on which phone events are recorded. */
struct _PHONE_t
{
#if (defined EASY336) || (defined XT16)
   /** connID */
   IFX_int32_t connID;
   /** lineID */
   IFX_int32_t lineID;
   /** array of voipIDs */
   IFX_int32_t *pVoipIDs;
   /** number of voipIDs in the array */
   IFX_int32_t nVoipIDs;
#endif /* (defined EASY336) || (defined XT16) */
#ifdef TD_RM_DEBUG
   /** number of allocated resources
      created for debug purposes */
   TD_RM_DEBUG_t oRM_Debug;
#endif /* TD_RM_DEBUG */
   /** Device number */
   IFX_uint16_t nDev;
   /** Phone number */
   IFX_int32_t        nPhoneNumber;
   /** Phone channel number */
   IFX_int32_t        nPhoneCh;
   /** Data channel number, phone channel is mapped to this data channel at
       start. Used for DTMF and other stuff. */
   IFX_int32_t        nDataCh;
   /** PCM channel number */
   IFX_int32_t        nPCM_Ch;
   /** Socket used for voice, data streaming */
   IFX_int32_t        nSocket;
   /** Device file descriptor this structure is connected to (events coming
       from phone channel. Basically first device is connected to first
       structure, second device to second structure, etc. */
   IFX_int32_t        nPhoneCh_FD;
   /** Device file descriptor this structure is connected to (events coming
       from data channel. */
   IFX_int32_t        nDataCh_FD;
#ifdef TD_DECT
   /** number of dect channel (used for DECT phones). */
   IFX_int32_t       nDectCh;
   /** type of phone (analog or dect) */
   PHONE_TYPE_t       nType;
   /** DECT channel activated flag */
   IFX_int32_t        bDectChActivated;
   /** Count of digits dialed on DECT handset to be handled. */
   IFX_int32_t        nDectDialNrCnt;
   /** Buffer with digits dialed on DECT handset to be handled. */
   IFX_char_t         pDectDialedNum[MAX_DIALED_NUM];
#endif /* TD_DECT */
   /** CID: description and number of source phone */
   IFX_TAPI_CID_MSG_t* pCID_Msg;

   /** Count of dialed numbers. */
   IFX_int32_t        nDialNrCnt;
   /** Array of dialed numbers. */
   IFX_char_t         pDialedNum[MAX_DIALED_NUM];

   /** Array of connections with other phones */
   CONNECTION_t rgoConn[MAX_PEERS_IN_CONF];

   /** Count of connections. This number - 1 represents number of
       active connections. Index to last one in array represents empty
       connection. */
   IFX_int32_t nConnCnt;

/* STATIC RM doesn't support conferencing */
   /** CONFERENCE, conference index which it belongs if greater than zero,
      otherwise does not belong to conference.
      When using to get conference structure
      use the nConfIdx value decrased by 1 */
   IFX_uint32_t nConfIdx;

   /** CONFERENCE, this phone started conference:
       IFX_FALSE - no,
       IFX_TRUE - yes */
   IFX_boolean_t nConfStarter;

   /** We called local peer?
       IFX_FALSE - no,
       IFX_TRUE - yes */
   IFX_boolean_t fLocalPeerCalled;

   /** We called external peer?
       IFX_FALSE - no,
       IFX_TRUE - yes */
   IFX_boolean_t fExtPeerCalled;

   /** We called PCM peer?
       IFX_FALSE - no,
       IFX_TRUE - yes */
   IFX_boolean_t fPCM_PeerCalled;

   /** If local call then this phone channel send us some action. Used when
    * removing local phone from conference. */
   CONNECTION_t* pConnFromLocal;

   /* To which board belongs this phone */
   BOARD_t* pBoard;

   /** FXO, this phone started FXO call:
       IFX_FALSE - no,
       IFX_TRUE - yes */
   IFX_boolean_t fFXO_Call;

   /** To which FXO belongs this phone */
   FXO_t* pFXO;

   /** True if we are in connection and pressed digit 'STAR' to make some
       adjustments. Otherwise false. */
   IFX_boolean_t fConfTAPI_InConn;

   /** For analog phones: holding true if phone is in wideband, otherwise false.
       This flag is also used when we end phone connection or exit from tapidemo
       to set back the default values (NarrowBand).
       For DECT phones: true if handset can use WideBand (can use
       IFX_DECTNG_CODEC_G722_64 codec) */
   IFX_boolean_t fWideBand_Cfg;

   /** Holding true if phone is in wideband for PCM calls, otherwise false.
       This flag is also used when we end phone connection to set back the
       default values (NarrowBand). */
   IFX_boolean_t fWideBand_PCM_Cfg;

   /** Holding true if phone AGC is configurd, otherwise false. This flag
       is also used when we end phone connection or exit from tapidemo to
       set back the default values (no AGC). */
   IFX_boolean_t fAGC_Cfg;

#ifdef EASY336
   /** true if CID detection is enabled on phone */
   IFX_boolean_t fCID_Enabled;
#endif /* EASY336 */

#if defined(XT16) || defined(WITH_VXT)
   /** Used. When Ground Start support is enabled.
      If Ground Start support is enabled then the IFX_TAPI_EVENT_FAULT_LINE_GK_LOW
      must be received before off-hook. This variable is set to true when
      IFX_TAPI_EVENT_FAULT_LINE_GK_LOW is received. */
   IFX_boolean_t fHookOff_Allowed;
#endif /* XT16 || WITH_VXT */

   /** Current state of phone. */
   STATE_MACHINES_t rgStateMachine[MAX_STATE_MACHINE];

   /** New internal FXS event to handle */
   INTERNAL_EVENTS_t nIntEvent_FXS;

   /** New internal config event to handle */
   INTERNAL_EVENTS_t nIntEvent_Config;

   /** Caller or Called - transmit and receive direction */
   CALL_DIRECTION_TYPE_t nCallDirection;

   /** Signals enabled for phone */
   IFX_TAPI_SIG_DETECTION_t stSignal;

   /** external peer name to use with CID */
   IFX_char_t szPeerName[TD_MAX_PEER_NAME_LEN];
};


/** Connection structure, represents abstraction object. Basically represents
    FXO with channel fd. */
struct _FXO_t
{
   /** FXO number (internal phone number or timeslot index) which follows
       the last phone number */
   IFX_int32_t        nFXO_Number;
   /** pointer to FXO device data */
   TAPIDEMO_DEVICE_FXO_t* pFxoDevice;

   /** FXO channel */
   IFX_int32_t        nFxoCh;

   /** PCM channel number which is used for FXO connection on main board. */
   IFX_int32_t        nPCM_Ch;

   /** Data channel on main board used for CID RX between first and second ring.
       Afterthat this channel is released. */
   IFX_int32_t        nDataCh;

   /** Device file descriptor this structure is connected to (events coming
       from data channel. */
   IFX_int32_t        nFxoCh_FD;

   /** Current state of FXO. */
   STATE_MACHINE_STATES_t nState;

   /** information about detected dtmf digits - ETSI DTMF CID standard uses DTMF
       digits to send caller number */
   TD_CID_DTMF_DIGITS_t oCID_DTMF;

   /** CID message  */
   IFX_TAPI_CID_MSG_t oCID_Msg;

   /** Only one phone can be connected to this FXO. */
   CONNECTION_t oConn;

   /** Time when ringing event has occured. */
   IFX_time_t timeRingEventTimeout;

   /** Indication that first ring stop event has occured. Its use for
       CID RX FSK support. */
   IFX_boolean_t fFirstStopRingEventOccured;

   /** Indication that CID FSK is started */
   IFX_boolean_t fCID_FSK_Started;

   /* FXO direction marker IFX_TRUE when incoming call, when not IFX_FALSE
      used to distinguish with connection structure use */
   IFX_boolean_t fIncomingFXO;
};


/** represents startup mapping table */
/* mapping is done by RM */
typedef struct _STARTUP_MAP_TABLE_t
{
   /** Phone channel, index */
   IFX_int32_t nPhoneCh;

   /** Connected data channel, just index, if -1 no connection to data
       channel */
   IFX_int32_t nDataCh;

   /** Connected PCM channel, just index, if -1 no connection to PCM
       channel.
       NOTICE: PCM channel is not mapped at start, but when PCM call
       is started.  */
   IFX_int32_t nPCM_Ch;

} STARTUP_MAP_TABLE_t;


/** Structure holding status for data channel */
typedef struct _VOIP_DATA_CH_t
{
   /* mapping managed by RM */
   /** IFX_FALSE if its free, not mapped. Otherwise IFX_TRUE if mapped. */
   IFX_boolean_t fMapped;

   /** Status of encoder/decoder. IFX_TRUE - started, IFX_FALSE - stopped */
   IFX_boolean_t fCodecStarted;

   /** IFX_FALSE - don't start codec, IFX_TRUE - start codec */
   IFX_boolean_t fStartCodec;

   /** IFX_TRUE - default VAD configuration,
      IFX_FALSE - VAD configuration changed */
   IFX_boolean_t fDefaultVad;

   /** Which type of coder to use. */
   IFX_TAPI_ENC_TYPE_t nCodecType;

   /** Length of frame in miliseconds OR packetisation time. */
   IFX_int32_t nFrameLen;

   /** ITU-T I366.2 Bit Alignment for G.726 codecs (IETF RFC3550),
       Used only by G.726. */
   IFX_TAPI_COD_AAL2_BITPACK_t nBitPack;

   /** Which type of coder was used before fax/modem connection*/
   IFX_TAPI_ENC_TYPE_t nPrevCodecType;

   /** What length of frame was used before fax/modem connection*/
   IFX_int32_t nPrevFrameLen;

   /** Settings of LEC before Fax/modem connection */
   IFX_TAPI_WLEC_CFG_t oPrevLECcfg;

   /** IFX_FALSE if no need to restore channel after fax/modem transmission */
   IFX_boolean_t fRestoreChannel;

   /**Setting of Jitter buffor before Fax/modem connection */
   IFX_TAPI_JB_CFG_t tapi_jb_PrevConf;

#ifdef TAPI_VERSION4
   /** Device number */
   IFX_uint16_t nDev;

   /** Channel number */
   IFX_uint16_t nCh;

   /** Port number */
   IFX_uint16_t nUDP_Port;
#endif /* TAPI_VERSION4 */
} VOIP_DATA_CH_t;


/** Structure holding status for socket */
typedef struct _SOCKET_STAT_t
{
   /** IFX_FALSE if its free, not used. Otherwise IFX_TRUE if used. */
   IFX_boolean_t fUsed;

   /** Handle to socket */
   IFX_int32_t nSocket;

   /** Port used */
   IFX_int32_t nPort;
} SOCKET_STAT_t;


/* --------------------------------------------------------- */
/*                 MULTIPLE BOARDS START                     */
/* --------------------------------------------------------- */

/** Max supporting boards */
enum { MAX_BOARDS = 4 };

/** No extension board is detected */
#define NO_EXT_BOARD_DETECT -1

/** Max supporting phones on all boards together */
#ifndef EASY336
enum { MAX_ALL_PHONES = 16 };
#else /* EASY336 */
enum { MAX_ALL_PHONES = 96 };
#endif /* EASY336 */

/** Types of board */
typedef enum
{
   TYPE_VINETIC = 0,
   TYPE_DANUBE = 1,
   TYPE_DUSLIC_XT = 2,
   TYPE_DUSLIC = 3,
   TYPE_VINAX = 4,
   TYPE_AR9 = 5,
   TYPE_SVIP = 6,
   TYPE_XT16 = 7,
   TYPE_TERIDIAN = 8,
   TYPE_VR9 = 9,
   TYPE_NONE = 10
} BOARD_TYPE_t;

/** Types of FXO devices */
typedef enum
{
   FXO_TYPE_DUSLIC = 0,
   FXO_TYPE_TERIDIAN = 1,
   FXO_TYPE_SLIC121 = 2,
   FXO_TYPE_NONE = 3
} FXO_TYPE_t;

/** devices to which access is done by GPIO */
typedef enum
{
   /** Vinetic on the EASY50510 */
   GPIO_VINETIC_EASY50510 = 1,
   /** DuSLIC on the EASY50712 (Danube) */
   GPIO_DUSLIC_EASY50712 = 2,
   /** Teridian on the EASY50712 (Danube) */
   GPIO_TERIDIAN_EASY50712 = 3,
   /** DuSLIC on the EASY50510 */
   GPIO_DUSLIC_EASY50510 = 4,
   /** Teridian on the EASY508XX (AR9) board */
   GPIO_TERIDIAN_EASY508XX = 5,
   /** Teridian on the EASY80800 (VINAX) board */
   GPIO_TERIDIAN_EASY80800 = 6,
   /** Teridian on the extension board */
   GPIO_TERIDIAN_EXT = 7,
   /** Teridian on the EASY80800 (VINAX) board */
   GPIO_TERIDIAN_EASY80910 = 8,
   /** Unknown device and board type */
   GPIO_NONE = 9
} GPIO_ACCESS_DEV_t;

/** Types of devices */
typedef enum
{
   TAPIDEMO_DEV_CPU = 1,
   TAPIDEMO_DEV_FXO = 2,
   TAPIDEMO_DEV_NONE = 3
} TAPIDEMO_DEVICE_TYPE_t;


/** Max. len of device path (/dev/vin...) */
enum { DEV_NAME_LEN = 32 };

/** Structure representing FXO device. */
struct _TAPIDEMO_DEVICE_FXO_t
{
   /** FXO device. Can be duslic or teridian */
   FXO_TYPE_t nFxoType;

   /** pointer to FXO name string */
   IFX_char_t* pFxoTypeName;

   /** FXO device number of given type counted from 1 */
   IFX_int32_t nDevNum;

   /** number of FXO channels available for this device */
   IFX_int32_t nNumOfCh;

   /** FXO device FD */
   IFX_int32_t nDevFD;

   /** Country selection. */
   IFX_uint8_t nCountry;

   /** pointer to table of FXO channel structures, allocated during
       initializtion */
   FXO_t* rgoFXO;

   /** last event on FXO device */
   IFX_TAPI_EVENT_t oTapiEventOnFxo;

   /** string used to adjust indention in traces */
   IFX_char_t* pIndent;

   /* To which board belongs this FXO */
   BOARD_t* pBoard;
};

/** Structure representing CPU device. */
struct _TAPIDEMO_DEVICE_CPU_t
{
   /** FW files data */
   IFX_char_t* pszDRAM_FW_File;

   /** FW files code */
   IFX_char_t* pszPRAM_FW_File[2];

   /** Coefficient file (BBD, CRAM) */
   IFX_char_t* pszBBD_CRAM_File[2];

   /** Pointer to device dependent structure */
   IFX_void_t* pDevDependStuff;

   /** Access mode for device. */
   IFX_int32_t AccessMode;

   /** Device physical base address. */
   unsigned long nBaseAddress;

   /** Device irq number. */
   IFX_int32_t nIrqNum;

   /** Pointer to function which initializes device */
   IFX_return_t (*Init)       (BOARD_t* pBoard, IFX_char_t* pPath);

   /** Pointer to function which read verion */
   IFX_return_t (*Version)    (BOARD_t* pBoard);

   /** Pointer to function which Handles the DRIVER error codes and stack */
   IFX_void_t   (*DecodeErrno)(IFX_int32_t nCode);

   /** Pointer to function which returns number of last driver error */
   IFX_return_t   (*GetErrno) (IFX_int32_t nFd);

   /** Pointer to function which fills in firmware pointers */
   IFX_return_t (*FillinFwPtr) (IFX_void_t* pDrvInit,
                                TAPIDEMO_DEVICE_CPU_t* pCpuDevice,
                                IFX_char_t* pPath);

   /** Pointer to function which releases firmware pointers */
   IFX_void_t  (*ReleaseFwPtr) (IFX_void_t* pDrvInit);
};

/** Structure representing device. */
typedef struct _TAPIDEMO_DEVICE_t
{
   /** device type (CPU or FXO) */
   TAPIDEMO_DEVICE_TYPE_t nType;

   /** Pointer to next device structure. */
   struct _TAPIDEMO_DEVICE_t *pNext;

   union
   {
      TAPIDEMO_DEVICE_CPU_t   stCpu;
      TAPIDEMO_DEVICE_FXO_t   stFxo;
   } uDevice;
} TAPIDEMO_DEVICE_t;

#ifdef EVENT_COUNTER
   /* debugging feature checking if events have been lost between
      TAPI driver and tapidemo. */
typedef struct EventSeqNo
{
   /** event sequence number of the next device specific event. */
   IFX_uint32_t nEventDevSeqNo;
   /** event sequence number of the next channel specific event. */
   IFX_uint32_t pEventChSeqNo[32];
} EventSeqNo_t;
#endif

/** Structure representing board and info about it. */
struct _BOARD_t
{
   /** Board idx */
   IFX_int32_t nBoard_IDX;

   /** Board id, used to open right device */
   IFX_int32_t nID;

   /** Board type */
   BOARD_TYPE_t nType;

   /** Board name */
   const char* pszBoardName;

   /** Prefix used to adjust indention in traces */
   IFX_char_t* pIndentBoard;

   /** Prefix used to adjust indention in traces */
   IFX_char_t* pIndentPhone;

   /** Number of analog ch on board. */
   IFX_int32_t nMaxAnalogCh;

   /** Max number of phones (analog + DECT) */
   IFX_int32_t nMaxPhones;

   /** Number of analog ch on board. */
   IFX_int32_t nMaxDectCh;

   /** Number of coder, signalling (data) ch on board. */
   IFX_int32_t nMaxCoderCh;

   /** Number of T.38 channels. */
   IFX_int32_t nMaxT38;

   /** Number of FXO channels supported by FW (currently only in SLIC121) */
   IFX_int32_t nMaxFxo;

   /** Number of used channels for open/close, events, streaming, ...
       Used, because some systems have only analog channels, coder channels are
       not present. In SVIP this field is preserved, but not used so often
       as before, nMaxAnalogCh or nMaxCoderCh is used instead */
   IFX_int32_t nUsedChannels;

   /** Number of pcm ch on board */
   IFX_int32_t nMaxPCM_Ch;

   /** System file descriptor */
   IFX_int32_t nSystem_FD;

   /** Usage of Sys FD, 0 - not used */
   IFX_boolean_t nUseSys_FD;

   /** Device control file descriptor */
   IFX_int32_t nDevCtrl_FD;

   /** Channel file descriptors */
   IFX_int32_t* nCh_FD;

   /** if IFX_TRUE, T.38 is supported */
   IFX_boolean_t nT38_Support;

#if (defined(TD_FAX_MODEM) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW))
   /** Capabilities of the T.38 stack implementation. */
   IFX_TAPI_T38_CAP_t stT38Cap;

   /** T.38 configuration */
   TD_T38_CONFIG_t oT38_Config;
#endif /* (defined(TD_FAX_MODEM) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW)) */

   /** Device name without numbers, like /dev/vmmc, /dev/vin, ... */
   IFX_char_t pszChDevName[DEV_NAME_LEN];

   /** Device control name numbers, like /dev/vmmc[board id]0, ... */
   IFX_char_t pszCtrlDevName[DEV_NAME_LEN];

   /** System device name, like /dev/easy3332/[board id], ... */
   IFX_char_t pszSystemDevName[DEV_NAME_LEN];

   /** Number of chips on board */
   IFX_int32_t nChipCnt;

   /** Startup mapping table */
   const STARTUP_MAP_TABLE_t* pChStartupMapping;

   /** Custom mapping table */
   STARTUP_MAP_TABLE_t* pChCustomMapping;

   /** Idication that Coefficient file (BBD, CRAM) was changed */
   IFX_boolean_t fBBD_Changed;

   /** Status array of data channels */
   VOIP_DATA_CH_t* pDataChStat;

   /** Default RTP and RTCP settings */
   IFX_TAPI_PKT_RTP_CFG_t rtpConf;

   /** Status of used pcm channels, IFX_FALSE - used, IFX_TRUE - not used */
   IFX_boolean_t* fPCM_ChFree;

   /** Socket on which we expect incoming data (events and voice)
       Also used to send data to external peer. There is linear
       transformation  between sockets and data channel. Socket
       with index 1 belongs to data channel 1, socket with index
       2 to data channel 2, etc. */
   IFX_int32_t* rgnSockets;

   /** Hold array of phones */
   PHONE_t* rgoPhones;

   /** Pointer to control structure */
   CTRL_STATUS_t *pCtrl;

   /** Pointer to device info structure */
   TAPIDEMO_DEVICE_t *pDevice;

   /** used to store board event data */
   IFX_TAPI_EVENT_t tapiEvent;

#ifdef TAPI_VERSION4
   /** Holding true if there is a single file descriptor */
   IFX_boolean_t fSingleFD;

   /** Holding true if sockets are used for voice streaming through codecs */
   IFX_boolean_t fUseSockets;
#endif /* TAPI_VERSION4 */

   /** Pointer to function which initializes board */
   IFX_return_t (*Init)       (BOARD_t* pBoard, PROGRAM_ARG_t* pProgramArg);

   /** Pointer to function which release board structure */
   IFX_void_t   (*RemoveBoard) (BOARD_t* pBoard);
#ifdef EVENT_COUNTER
   EventSeqNo_t evDbg[3];
#endif
};

/** Max combinations */
enum { MAX_COMBINATIONS = 12 };

/** Max boards in one combination */
enum { MAX_BOARD_IN_COMBINATION = MAX_BOARDS };

/** structure for different board combinations */
typedef struct _BOARD_COMBINATION_ELEM_t
{
   /** board ID */
   IFX_int32_t nBoard_ID;
   /** board type */
   BOARD_TYPE_t nBoardType;
} BOARD_COMBINATION_ELEM_t;



/** Structure holding combination */
typedef struct _BOARD_COMBINATION_t
{
   /** identifier */
   IFX_int32_t nIdx;
   /** number of boards */
   IFX_int32_t nBoardCnt;
   /** array with boards parameters */
   BOARD_COMBINATION_ELEM_t nBoards[MAX_BOARD_IN_COMBINATION];
} BOARD_COMBINATION_t;

/* --------------------------------------------------------- */
/*                 MULTIPLE BOARDS END                       */
/* --------------------------------------------------------- */


/** Flags for command line arguments */
typedef struct _ARG_FLAGS_t
{
   /** If 1 then CID is send when calling other phone, but only local */
   IFX_uint32_t nCID : 1;
   /** If 1 then conference is used */
   IFX_uint32_t nConference : 1;
   /** If 1 then help is displayed */
   IFX_uint32_t nHelp : 1;
   /** If 1 then version is displayed */
   IFX_uint32_t nVersion : 1;
#ifdef LINUX
   /** If 1 then configuration is displayed */
   IFX_uint32_t nConfigure : 1;
#endif /* LINUX */
   /** If 1 then wait after each state machine step */
   IFX_uint32_t nWait : 1;
   /** If 1 then download CRAM file */
   /*IFX_uint32_t nCRAM : 1;*/
   /** 1 if quality of service is used, otherwise 0 */
   IFX_uint32_t nQos : 1;
   /** 1 if PCM is used and we are master, otherwise 0 */
   IFX_uint32_t nPCM_Master : 1;
   /** 1 if PCM is used and we are slave, otherwise 0  */
   IFX_uint32_t nPCM_Slave : 1;
   /** 1 for new encoder type, otherwise 0  */
   IFX_uint32_t nEncTypeDef : 1;
   /** 1 for new packetisation time, otherwise 0  */
   IFX_uint32_t nFrameLen : 1;
   /** 1 for calls to different network, otherwise 0  */
   IFX_uint32_t nDiffNet : 1;
   /** 1 if FXO is used, otherwise 0 */
   IFX_uint32_t nFXO : 1;
   /** 1 if local UDP will be tested, otherwise 0 - external UDP is used */
   IFX_uint32_t nQOS_Local : 1;
   /** 1 if local loop PCM will be used, otherwise 0 */
   IFX_uint32_t nPCM_Loop : 1;
   /** 1 to use coders for local connection, 0 to use phone mapping */
   IFX_uint32_t nUseCodersForLocal : 1;
   /** 1 to use PCM wideband, 0 to use PCM narrowband */
   IFX_uint32_t nUsePCM_WB : 1;
   /** 1 to run Tapidemo as daemon */
   IFX_uint32_t nDaemon : 1;
   /** path to download has been changed */
   IFX_uint32_t nUseCustomDownloadsPath : 1;
   /** do not use LEC - obsolete */
   IFX_uint32_t nNoLEC : 1;
   /** 1 if Ground Start is supported */
   IFX_uint32_t nGroundStart : 1;
   /** if not 0 then DECT is not initialized */
   IFX_uint32_t nNoDect : 1;
   /** if not 0 flag -b is not used */
   IFX_uint32_t nUseCombinationBoard : 1;
#ifdef TD_RM_DEBUG
   /** if not 0 resource counting results are printed
       created for debug purposes */
   IFX_uint32_t nRM_Count : 1;
#endif /* TD_RM_DEBUG */
} ARG_FLAGS_t;


/** type of phone port */
typedef enum _TAPIDEMO_PORT_TYPE_t
{
   /** FXO port */
   PORT_FXO,
   /** FXS port */
   PORT_FXS
} TAPIDEMO_PORT_TYPE_t;

/** port data structure */
typedef struct _TAPIDEMO_PORT_DATA_t
{
   /** port type(fxo/fxs) */
   TAPIDEMO_PORT_TYPE_t nType;
   /** port data */
   union
   {
      /** FXS data */
      PHONE_t* pPhopne;
      /** FXO data */
      FXO_t*   pFXO;
   } uPort;
} TAPIDEMO_PORT_DATA_t;

/** Test types performed by test script on Control PC*/
typedef enum _COM_TEST_TYPE_t
{
   COM_NO_TEST = -1,
   COM_MODULAR_TEST = 0,
   COM_BULK_CALL_TEST = 1 /* default */
} COM_TEST_TYPE_t;

/** Modular subtest types performed by test script on Control PC*/
typedef enum _COM_SUB_TEST_TYPE_t
{
   /** default */
   COM_SUBTEST_NONE              = 0x0000,
   COM_SUBTEST_HOOKSTATE         = 0x0001,
   COM_SUBTEST_HOOKFLASH         = 0x0002,
   COM_SUBTEST_PULSEDIGIT        = 0x0004,
   COM_SUBTEST_DTMFDIGIT         = 0x0008,
   COM_SUBTEST_CONFERENCE        = 0x0010,
   COM_SUBTEST_FXO               = 0x0020,
   COM_SUBTEST_FAX               = 0x0040,
   COM_SUBTEST_MODEM             = 0x0080,
   COM_SUBTEST_FAX_T_38          = 0x0100,
   COM_SUBTEST_SYSINIT           = 0x0200,
   COM_SUBTEST_CID               = 0x0400
} COM_SUB_TEST_TYPE_t;

/** holds received from test script CID data */
typedef struct _TD_CID_TEST_DATA_t
{
   /** Caller ID structure pointer */
   IFX_TAPI_CID_MSG_t* pCID_Msg;
   /** number of phone on which CID will be played */
   IFX_int32_t nPhoneNumber;
   /** IFX_TRUE if number from test script is set */
   IFX_boolean_t bNumberIsSet;
   /** IFX_TRUE if name from test script is set */
   IFX_boolean_t bNameIsSet;
} TD_CID_TEST_DATA_t;

/** List of supported FXO devices. */
typedef enum _TD_FXO_DEV_t
{
   /** teridian */
   TD_FXO_TER             = 0x01,
   /** duslic, Clare */
   TD_FXO_DUS             = 0x02,
   /** SLIC 121 */
   TD_FXO_SLIC121         = 0x04  
} TD_FXO_DEV_t;

/** Arguments used for '-x' option. */
#define TD_CMD_ARG_FXO_TERIDIAN   "t"
#define TD_CMD_ARG_FXO_DUSLIC     "d"
#define TD_CMD_ARG_FXO_SLIC121    "s"

/** Pipe names */
#define LOG_THREAD_SYNCH_PIPE "td_log_synch_pipe"
#define SOCKET_THREAD_SYNCH_PIPE "td_socket_synch_pipe"
#define REDIRECT_LOG_PIPE "td_redirect_log_pipe"
#define DECT_PIPE "td_dect_pipe"

/** Structure holding pipe */
typedef struct _TD_PIPE_t
{
   /** Pipe names */
   IFX_char_t rgPipeName[30];
   /** Array with file pointers to pipe */ 
   TD_OS_Pipe_t* rgPipeFp[2];
   /** Array with file descriptors to pipe */ 
   IFX_int32_t rgPipeFd[2];
} TD_PIPE_t;

/** Demo structure holding command line arguments as flags */
struct _PROGRAM_ARG_t
{
   /* #warning karol -workaround for teridian extension board */
   IFX_boolean_t hUseDuslicFxo;
   /* end workaround */

    /** Set parameters */
    ARG_FLAGS_t oArgFlags;
    /** Contains IP address for connection in network format */
    TD_OS_sockAddr_t oMy_IP_Addr;
    /** Contains first three numbers of IP address when calling different
        network. Syntax: xxx.xxx.xxx{.yyy - added by us} */
    struct in_addr oDiffNetIP;
    /** Path to FW, BBD filenames to download */
    IFX_char_t sPathToDwnldFiles[MAX_PATH_LEN];
    /** Board combination */
    IFX_int32_t nBoardComb;
    /** Debug, trace level */
    DEBUG_LEVEL_t nDbgLevel;
#ifdef TD_DECT
    /** DECT debug trace level of DECT process */
    DEBUG_LEVEL_t nDectDbgLevel;
#endif /* TD_DECT */
    /** CID standard - this variable should be only used to get command line
        arguement and set default value, to check current CID configuration use
        function CID_GetStandard() */
    IFX_int32_t nCIDStandard;
    /** Encoder type */
    IFX_int32_t nEnCoderType;
    /** Packetisation time */
    IFX_int32_t nPacketisationTime;
    /** Voice Activity Detection (VAD) settings */
    IFX_TAPI_ENC_VAD_t nVadCfg;
    /** LEC type */
    IFX_TAPI_WLEC_TYPE_t  nLecCfg;
    /** Trace redirection */
    TRACE_REDIRECTION_t nTraceRedirection;
    /** FXO devices which should be used. */
    IFX_uint32_t nFxoMask;
};

/** Multithreading control structure */
typedef struct _TD_MULTITHREAD_CTRL_t
{
   struct
   {
      /** Mutex used to lock state machine */
      TD_OS_mutex_t mutexStateMachine;
   } oMutex;
   struct 
   {
      /** Lock used for synchronization socket thread with the main thread */
      TD_OS_lock_t oSocketThreadStopLock;
      /** Lock used for synchronization log thread with the main thread */
      TD_OS_lock_t oLogThreadStopLock;
   } oLock;
   struct
   {
      /** Socket thread control stucture */
      TD_OS_ThreadCtrl_t oHandleSocketThreadCtrl;
      /** Log thread control stucture */
      TD_OS_ThreadCtrl_t oHandleLogThreadCtrl;
   } oThread;
   struct
   {   
      /** Pipe used for synchronization socket thread with the main thread */
      TD_PIPE_t oSocketSynchPipe;
      /** Pipe used for synchronization log thread with the main thread */
      TD_PIPE_t oLogSynchPipe;
      /** Pipe used to redirect log */
      TD_PIPE_t oRedirectLogPipe;
      /** Pipe used for communication from DECT stack */
      TD_PIPE_t oDectPipe;
   } oPipe;
} TD_MULTITHREAD_CTRL_t;

/** Types of msg used for synch threads */
typedef enum
{
   /** Update timeout */
   TD_SYNCH_THREAD_MSG_SET_TIMEOUT = 1,
   /** Stop thread */
   TD_SYNCH_THREAD_MSG_STOP = 2
} TD_SYNCH_THREAD_MSG_t;

/** Specific which select() is used */
typedef enum
{
   /** Use for device select */
   TD_SELECT_DEVICE,
   /** Use for socket select */
   TD_SELECT_SOCKET
} TD_SELECT_TYPE_t;

/** Internal Test Module control structure */
typedef struct _TD_ITM_CONTROL_t
{
   /** Is Internal Test Module enabled flag. */
   IFX_boolean_t fITM_Enabled;
   /** Socket used for communication with control PC (incoming) */
   TD_OS_socket_t nComInSocket;
   /** Socket used for communication with control PC (outgoing) */
   TD_OS_socket_t nComOutSocket;
   /** Did the SUT send its ip address to control PC*/
   IFX_boolean_t fComIdentSent;
   /** Change address IP (request to send IP was send) */
   IFX_boolean_t bChangeIPAddress;
   /** Verify System Init modular test data */
   struct
   {
      /** Expected command IDs from Control PC */
      IFX_int32_t nExpectedID;
      /** Command IDs received from Control PC */
      IFX_int32_t nReceivedID;
      /** Used to hold test data */
      IFX_char_t aParametrs[TD_TWO_UINT32_STRING_LEN];
      /** Is verify system init enabled flag. */
      IFX_boolean_t fEnabled;
   } oVerifySytemInit;
   /** Remote (send to control PC) debug, trace level */
   DEBUG_LEVEL_t nComDbgLevel;
   /** Test type beeing performed (bulk call or modular) */
   COM_TEST_TYPE_t nTestType;
   /** Modular sub test type beeing performed */
   IFX_uint32_t nSubTestType;
   /** if IFX_TRUE use custom CPTs */
   IFX_boolean_t nUseCustomCpt;
   /** array with tapidemo version */
   IFX_char_t rgoTapidemoVersion[TD_MAX_NAME];
   /** array with ITM version */
   IFX_char_t rgoITM_Version[TD_MAX_NAME];
   /** array with firmware version */
   IFX_char_t rgoFW_Version[TD_MAX_NAME];
   /** Array with program options (ARGV). */
   IFX_char_t** rgsArgv;
   /** Number of program arguments (ARGC). */
   IFX_int32_t nArgc;
   /** Default program arguments. */
   PROGRAM_ARG_t oDefaultProgramArg;
   /** Caller Id data used for test. */
   TD_CID_TEST_DATA_t oCID_Test;
} TD_ITM_CONTROL_t;

/** Control structure for all channels, holding status of connections and
    also program arguments */
struct _CTRL_STATUS_t
{
   /* #warning karol -workaround for teridian extension board */
   IFX_boolean_t hUseDuslicFxo;
   /* end workaround */

   /** Holds array of boards */
   BOARD_t rgoBoards[MAX_BOARDS];
   /** Holds array of conferences */
   /* conferencing not supported by static RM */
   CONFERENCE_t rgoConferences[MAX_CONFERENCES];
   /** Conference index, if 0 no conference yet */
   IFX_uint32_t nConfCnt;
   /** Program arguments */
   PROGRAM_ARG_t* pProgramArg;
   /** Holds control device file descriptor, like for /dev/vin10, but
       highest file descriptor, used for FD_ISSET */
   IFX_int32_t nMaxFdDevice;
   /** Holds control device file descriptor, like for /dev/vin10, but
       highest file descriptor, used for FD_ISSET */
   IFX_int32_t nMaxFdSocket;
   /** Contains IP address for connection but as number */
   IFX_uint32_t nMy_IP_Addr;
   /** Contains our ip address */
   TD_OS_sockAddr_t oTapidemo_IP_Addr;
   /** Contains our MAC address */
   IFX_uint8_t oTapidemo_MAC_Addr[ETH_ALEN];
#if (defined EASY336 || defined XT16)
   /** Contains veth0 ip address */
   TD_OS_sockAddr_t oVeth0_IP_Addr;
   /** Contains veth0 MAC address */
   IFX_uint8_t oVeth0_MAC_Addr[ETH_ALEN];
#endif /* (defined EASY336 || defined XT16) */
   /** Boards in combination */
   IFX_int32_t nBoardCnt;
   /** Sum of all data channels on all boards. */
   IFX_int32_t nSumCoderCh;
   /** Sum of all phones on all boards (analog + DECT). */
   IFX_int32_t nSumPhones;
   /** Sum of all pcm channels on all boards. */
   IFX_int32_t nSumPCM_Ch;
   /** Max number of FXOs */
   IFX_int32_t nMaxFxoNumber;

   /** Socket on which we expect incoming data (events and voice)
       Also used to send data to external peer. */
   SOCKET_STAT_t* rgnSockets;

   /** Socket used for administration of calls (make/end/...) */
   TD_OS_socket_t nAdminSocket;
#ifdef TD_FAX_MODEM
#if (defined(TD_T38_FAX_TEST) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW))
   /** fax test control structure */
   TD_T38_FAX_TEST_CTRL_t oFaxTestCtrl;
#endif /* (defined(TD_T38_FAX_TEST) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW)) */
   /* used in tapidemo fax transmission mode */
   TD_FAX_MODE_t nFaxMode;
#endif /* TD_FAX_MODEM */
   /* socket used for syslog */
   IFX_int32_t nSyslogSocket;
   /** socket used for syslog */
   IFX_int32_t nSyslogChildPid;
   /** set if internal event generated from local phone or fxo,
       this flag is set together with connection pointer on destination phone
       e.g. p_dst_phone->pConnFromLocal = p_conn */
   IFX_boolean_t bInternalEventGenerated;
   /** flag with number of resources(phones and FXO channels)
       that need timeout check */
   IFX_int32_t nUseTimeoutDevice;
   /** flag with number of resources(phones and FXO channels)
       that need timeout check */
   IFX_int32_t nUseTimeoutSocket;
   /** Multithreading control structure */
   TD_MULTITHREAD_CTRL_t oMultithreadCtrl;
#ifdef EASY336
   /** pointer to array with LineID count for ConnIDs */
   TD_CONN_ID_LINE_ID_CNT_t* pConnID_LinID_Cnt;
#endif /* EASY336 */
#ifdef TD_DECT
   /** FD of pipe to read DECT messages */
   IFX_int32_t nDectPipeFdOut;
   /** DECT stack control data */
   TD_DECT_STACK_CTRL_t oDectStackCtrl;
   /** timer FD */
   IFX_int32_t nTimerFd;
   /** paging key FD */
   IFX_int32_t viPagingKeyFd;
   /** Communication socket to receive data For Production Test Tool */
   IFX_int32_t viFromCliFd;
#endif /* TD_DECT */
   /** Internal Test Module control data */
   TD_ITM_CONTROL_t oITM_Ctrl;
};

/** types of data in COMM_MSG_DATA_1_t */
typedef enum _TD_COMM_MSG_DATA_1_TYPE_t
{
   /** data isn't set */
   TD_COMM_MSG_DATA_1_NONE = 0,
   /** CID data */
   TD_COMM_MSG_DATA_1_CID = 1
}TD_COMM_MSG_DATA_1_TYPE_t;

/** holds peer name for CID */
typedef struct _TD_COMM_MSG_DATA_1_CID_t
{
   /** type of data - must be TD_COMM_MSG_DATA_1_CID */
   TD_COMM_MSG_DATA_1_TYPE_t nType; /* 4 (4) */
   IFX_char_t aPeerName[TD_MAX_PEER_NAME_LEN]; /* 20 (24) */
}TD_COMM_MSG_DATA_1_CID_t;

/** data structures for message no 1, each new structure must contain
    TD_COMM_MSG_DATA_1_TYPE_t as first data member and new structure must be
    smaller than aReserved field (size of aReserved cannot be changed),
    check TD_COMM_MSG_t description for more info */
typedef union _TD_COMM_MSG_DATA_1_t
{
   /** array defined to set maximal size of union */
   IFX_char_t aReserved[TD_MAX_COMM_MSG_SIZE_DATA_1]; /* 32 */
   /** CID data */
   TD_COMM_MSG_DATA_1_CID_t oCID;
}TD_COMM_MSG_DATA_1_t;


/** Message used to communicate between boards, structure can be changed only
    by adding elements right before nMarkEnd or by adding structures to unions,
    Union size cannot change due to those changes. When changin structure
    TD_COMM_MSG_CURRENT_VERSION must also be changed */
typedef struct _TD_COMM_MSG_t
{
   /** Mark start of message - must match, otherwise wrong message */
   IFX_uint32_t nMarkStart; /* 4 - field size (4 - overall size) */

   /** message version  */
   IFX_uint32_t nMsgVersion; /* 4 (8) */

   /** message version  */
   IFX_uint32_t nMsgLength; /* 4 (12) */

   /** Phone number of message sender */
   IFX_uint32_t nSenderPhoneNum; /* 4 (16) */

   /** Phone number of message receiver */
   IFX_uint32_t nReceiverPhoneNum; /* 4 (20) */

   /** Port number of message sender for VOICE streaming */
   IFX_int32_t nSenderPort; /* 4 (24) */

   /** message sender MAC */
   IFX_uint8_t MAC[ETH_ALEN];  /* 6 (30) */

   /** Action (OFF_HOOK, ...) */
   IFX_uint8_t nAction;  /* 1 (31) */

   /** 0x80 - PCM or 0x00 - VoIP call */
   IFX_uint8_t fPCM;  /* 1 (32) */

   /** Timeslot RX index, used only in PCM connection */
   IFX_int32_t nTimeslot_RX;  /* 4 (36) */
   /** Timeslot TX index, used only in PCM connection */
   IFX_int32_t nTimeslot_TX;  /* 4 (40) */

   /** message sender board idx */
   IFX_int32_t nBoard_IDX;  /* 4 (44) */


   /** Configure phone with some features (number defines which one)
       also nAction must be set to IE_CONFIG_PEER,
       mainly used to set feature(e.g. codec) for both peers during call,
       feature is usually set with DTMF codes */
   IFX_int32_t nFeatureID;  /* 4 (48) */

   /** holds various data that can be send */
   TD_COMM_MSG_DATA_1_t oData1; /* 32 (80) */
   /* NEW DATA CAN BE ADDED ONLY RIGHT BEFORE nMarkEnd
      check structure description for more info */
   /** Mark end of message - must match, otherwise wrong message */
   IFX_uint32_t nMarkEnd;  /* 4 (84) */
} TD_COMM_MSG_t;

/** Type of mapping */
typedef enum _MAPPING_TYPE_t { NO_MAPPING = -1 } MAPPING_TYPE_t;

/** Structure holding status for trace redirection */
typedef struct _TRACE_REDIRECTION_CTRL_t
{
   /** Contains IP address for connection to syslog */
   TD_OS_sockAddr_t oSyslog_IP_Addr;
   /** Path to log file*/
   IFX_char_t sPathToLogFiles[MAX_PATH_LEN];
   /** Socket used for communication with remote syslog */
   TD_OS_socket_t nSyslogSocket;
   /* Holding file descriptor for log file */
   TD_OS_File_t* pFileLogFD;
   /* Start redirecting log */
   IFX_boolean_t fStartRedirectLogs;
} TRACE_REDIRECTION_CTRL_t;

#define SEPARATE \
TAPIDEMO_PRINTF(("############################################\n"));

/* ============================= */
/* Global Variables              */
/* ============================= */

/** Frequencies strings */
extern const char * TAPIDEMO_PCM_FREQ_STR[];

/* ============================= */
/* Global Structures             */
/* ============================= */

 #ifdef EASY336
/** Structure for requesting allocation/deallocation of resources */
typedef struct _RESOURCES_t
{
   /* resources to get/release */
   IFX_int32_t nType;
   /* connID number to use */
   IFX_int32_t nConnID_Nr;
   /* voipID number to release */
   IFX_int32_t nVoipID_Nr;
} RESOURCES_t;
#endif

/* ============================= */
/* Global function declaration   */
/* ============================= */

/* fd operations */
extern IFX_return_t Common_Open(IFX_char_t* pszName);
extern IFX_return_t Common_Set_FDs(BOARD_t* pBoard);
extern IFX_void_t Common_Close_FDs(BOARD_t* pBoard);
extern IFX_int32_t Common_GetFD_OfCh(BOARD_t* pBoard, IFX_int32_t nChNum);

/* FXO fd operations */
extern IFX_int32_t Common_GetNextFxoDevCtrlFd(CTRL_STATUS_t* pCtrl,
                                              TAPIDEMO_DEVICE_FXO_t** ppFxoDev,
                                              IFX_boolean_t fOmitSlic121Fxo);

/* board initialization */
extern IFX_return_t Common_RegisterBoard(BOARD_t* pBoard);
extern IFX_return_t Common_GetVersions(BOARD_t* pBoard);

extern IFX_return_t Common_SetPhonesAndUsedChannlesNumber(BOARD_t* pBoard);
extern IFX_return_t Common_GetCapabilities(BOARD_t* pBoard);
extern IFX_char_t* Common_GetTraceIndention(IFX_int32_t nSize);
#ifdef USE_FILESYSTEM
IFX_return_t Common_CheckFileExists(IFX_char_t* pszFullFilename);
IFX_return_t Common_CreateFullFilename(IFX_char_t* pszFilename,
                                     IFX_char_t* pszPath,
                                     IFX_char_t* pszFullFilename);
IFX_boolean_t Common_FindPRAM_FW(TAPIDEMO_DEVICE_CPU_t* pCpuDevice,
                                   IFX_char_t* pszPath,
                                   IFX_char_t* pszFullFilename);
IFX_boolean_t Common_FindBBD_CRAM(TAPIDEMO_DEVICE_CPU_t* pCpuDevice,
                                  IFX_char_t* pszPath,
                                  IFX_char_t* pszFullFilename);
#ifndef VXWORKS
extern IFX_return_t Common_CheckDownloadPath(IFX_char_t* psPath,
                                      TAPIDEMO_DEVICE_CPU_t* pCpuDevice,
                                      IFX_boolean_t bPrintTrace);
extern IFX_return_t Common_SetDownloadPath(IFX_char_t* sPathToDwnldFiles,
                                             BOARD_t* pBoard,
                                             TAPIDEMO_DEVICE_CPU_t* pCpuDevice);
extern IFX_return_t Common_CheckDownloadPath(IFX_char_t* psPath,
   TAPIDEMO_DEVICE_CPU_t* pCpuDevice,
   IFX_boolean_t bPrintTrace);
#endif /* VXWORKS */
#endif  /* USE_FILESYSTEM */

extern IFX_return_t Common_ConfigureTAPI(CTRL_STATUS_t* pCtrl,
                                         BOARD_t* pBoard,
                                         PHONE_t* pPhone,
                                         IFX_int32_t nAction,
                                         IFX_int32_t nDataCh_FD,
                                         IFX_boolean_t fConfigureAll,
                                         IFX_boolean_t fConfigureConn);

extern IFX_return_t Common_FillAddrByNumber(CTRL_STATUS_t* pCtrl,
                                            PHONE_t* pPhone,
                                            CONNECTION_t* pConn,
                                            IFX_uint32_t nIP_Addr,
                                            IFX_boolean_t fIsPCM_Num);
extern IFX_int32_t Common_DigitsToNum(IFX_char_t* prgnDialNum,
                                      IFX_int32_t nFirstDigit,
                                      IFX_int32_t nDialNrCnt);

extern CALL_TYPE_t Common_CheckDialedNumForFull_IP(IFX_char_t* prgnDialNum,
                                                   IFX_int32_t nDialNrCnt,
                                                   IFX_uint32_t* nIpAddr);
extern CALL_TYPE_t Common_CheckDialedNum(PHONE_t* pPhone,
                                         IFX_int32_t* nDialedNum,
                                         IFX_uint32_t* nIpAddr,
                                         CTRL_STATUS_t* pCtrl);

extern IFX_void_t Common_PrintCallProgress(STATE_MACHINE_STATES_t current_state,
                                           INTERNAL_EVENTS_t new_event,
                                           CALL_TYPE_t nOldCallType,
                                           PHONE_t* pPhone, CONNECTION_t* pConn,
                                           IFX_int32_t nUseCodersForLocal);
extern IFX_void_t Common_PrintEncoderType(CTRL_STATUS_t* pCtrl, PHONE_t* pPhone,
                                          CONNECTION_t* pConn);
extern IFX_return_t Common_TonePlay(CTRL_STATUS_t* pCtrl, PHONE_t* pPhone,
                                    IFX_int32_t nTone);
extern IFX_return_t Common_ToneStop(CTRL_STATUS_t* pCtrl, PHONE_t* pPhone);
extern IFX_return_t Common_JBStatistic(IFX_int32_t nDataCh,
                                       IFX_int32_t nDev,
                                       IFX_boolean_t fReset,
                                       IFX_boolean_t fShow,
                                       BOARD_t* pBoard);

extern IFX_return_t Common_RTCPStatistic(IFX_int32_t fd,
                                         IFX_boolean_t fReset,
                                         IFX_boolean_t fShow);

extern IFX_void_t Common_ErrorHandle(IFX_int32_t nFD, BOARD_t* pBoard,
   IFX_uint16_t nDev);
#ifdef TAPI_VERSION4
#if (defined(EASY336) || defined(XT16))
extern IFX_return_t Common_PCM_ConfigSet(BOARD_t* pBoard,
                                         RM_SVIP_PCM_CFG_t* pcm_cfg);
#endif

#ifdef EASY336
extern IFX_return_t Common_ResourcesGet(PHONE_t* pPhone,
                                        const RESOURCES_t* pResources);
extern IFX_return_t Common_ResourcesRelease(PHONE_t* pPhone,
                                            const RESOURCES_t* pResources);
extern VOIP_DATA_CH_t* Common_GetCodec(CTRL_STATUS_t* pCtrl,
                                       IFX_int32_t voipID);
extern VOIP_DATA_CH_t* Common_GetFirstCodecOfPhone(PHONE_t* pPhone);
#endif
extern IFX_return_t Common_DTMF_Enable(CTRL_STATUS_t* pCtrl, PHONE_t* pPhone);
extern IFX_return_t Common_DTMF_Disable(CTRL_STATUS_t* pCtrl, PHONE_t* pPhone);
#ifdef EASY336
extern IFX_int32_t Common_ConnID_PhonesGet(CTRL_STATUS_t* pCtrl,
                                           IFX_int32_t connID,
                                           PHONE_t** pPhones[]);
#endif /* EASY336 */

extern IFX_return_t Common_PulseEnable(CTRL_STATUS_t* pCtrl, PHONE_t* pPhone);
extern IFX_return_t Common_PulseDisable(CTRL_STATUS_t* pCtrl, PHONE_t* pPhone);

extern IFX_return_t Common_CID_Enable(PHONE_t* pPhone, IFX_boolean_t bCID_AS);
extern IFX_return_t Common_CID_Disable(PHONE_t* pPhone);
extern IFX_return_t Common_SetChipCount(BOARD_t* pBoard);
#ifdef LINUX
extern IFX_return_t Common_WriteByte(IFX_uint32_t nAddr, IFX_uint8_t nValue);
#endif
#else /* TAPI_VERSION4 */
extern IFX_return_t Common_DTMF_SIG_Enable(CTRL_STATUS_t* pCtrl,
   PHONE_t* pPhone);
extern IFX_return_t Common_DTMF_SIG_Disable(CTRL_STATUS_t* pCtrl,
   PHONE_t* pPhone);
#endif /* TAPI_VERSION4 */
IFX_return_t Common_CommMsgSet(TD_COMM_MSG_t* pMsg, IFX_char_t* pBuff,
                               IFX_int32_t nBuffLen);
extern IFX_return_t Common_LineFeedSet(PHONE_t* pPhone, TD_LINE_MODE_t mode);

extern IFX_return_t Common_RingingStart(PHONE_t* pPhone);
extern IFX_return_t Common_RingingStop(PHONE_t* pPhone);

extern IFX_return_t Common_LEC_Enable(CTRL_STATUS_t* pCtrl, PHONE_t* pPhone);
extern IFX_return_t Common_LEC_Disable(CTRL_STATUS_t* pCtrl, PHONE_t* pPhone);
extern IFX_return_t Common_LEC_On_PCM_Ch(IFX_int32_t nPCM_Ch,
   IFX_enDis_t nEnable, BOARD_t* pBoard);
extern IFX_return_t Common_SetVad(PHONE_t* pPhone, BOARD_t* pBoard,
                                   IFX_int32_t nVadCfg);
extern IFX_void_t Common_StripPath (IFX_char_t** sPath);

extern TAPIDEMO_DEVICE_t * Common_GetDevice_CPU(TAPIDEMO_DEVICE_t* pDevice);
extern IFX_return_t Common_AddDevice(BOARD_t* pBoard,
                                      TAPIDEMO_DEVICE_t* pDevice);
extern IFX_return_t Common_RemoveDeviceList(BOARD_t* pBoard);
extern IFX_char_t * Common_Enum2Name(IFX_int32_t nEnum,
                                    TD_ENUM_2_NAME_t *pEnumName);
extern IFX_int32_t Common_Name2Enum(IFX_char_t* pString,
                                    TD_ENUM_2_NAME_t *pEnumName,
                                    IFX_boolean_t bPrintError);
IFX_return_t Common_MapLocalPhones(PHONE_t* pPhone, PHONE_t* pDstPhone,
                                   TD_MAP_ADD_REMOVE_t nMapFlag);
extern IFX_boolean_t Common_IsNumber(IFX_char_t* pValue);
extern IFX_return_t Common_StartThread(TD_OS_ThreadCtrl_t* pThreadCtrl,
                                    IFX_int32_t (*pFun)(TD_OS_ThreadParams_t*),
                                    IFX_char_t* pThreadName,
                                    TD_PIPE_t* pSynchPipe,
                                    TD_OS_lock_t* pLockThreadStop);
extern IFX_void_t Common_StopThread(TD_OS_ThreadCtrl_t* pThreadCtrl,
                               TD_PIPE_t* pSynchPipe,
                               TD_OS_lock_t* pLockThreadStop);
extern IFX_return_t Common_PreparePipe(TD_PIPE_t* pPipe);
#ifdef LINUX
IFX_void_t Common_CheckTraceLen(IFX_char_t* pBuffer);
#endif /* LINUX */
/* GPIO */
extern IFX_int32_t Common_GPIO_OpenPort(IFX_char_t* pGpioPortName);
extern IFX_return_t Common_GPIO_ClosePort(IFX_char_t* pGpioPortName, 
                                          IFX_int32_t nGPIO_PortFd);
#if (defined(DANUBE) || defined(AR9) || defined(VR9))
extern IFX_return_t Common_GPIO_ReservePin(IFX_int32_t nFd, IFX_int32_t nPort, 
                                           IFX_int32_t nPin, IFX_int32_t nModule);
extern IFX_return_t Common_GPIO_FreePin(IFX_int32_t nFd, IFX_int32_t nPort, 
                                        IFX_int32_t nPin, IFX_int32_t nModule);
#endif /* (defined(DANUBE) || defined(AR9) || defined(VR9)) */
#ifdef EVENT_COUNTER
extern IFX_void_t EventSeqNoCheck(BOARD_t *pBoard, IFX_TAPI_EVENT_t *pEvent);
extern IFX_char_t *Common_Enum2Name(IFX_int32_t nEnum, TD_ENUM_2_NAME_t *pEnumName);
#endif /* EVENT_COUNTER */
extern TD_ENUM_2_NAME_t TD_rgTapiEventName[];
#endif /* _COMMON_H */

