

/******************************************************************************

                              Copyright (c) 2009
                            Lantiq Deutschland GmbH
                     Am Campeon 3; 85579 Neubiberg, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

  ****************************************************************************/

/**
   \file device_vmmc.c
   \date 2006-10-01
   \brief Interface implementation for VMMC device.

   Application needs it to initialize device.
*/

/* ============================= */
/* Includes                      */
/* ============================= */
#include "tapidemo.h"
#include "common.h"

#ifdef HAVE_DRV_VMMC_HEADERS
    #include "drv_vmmc_errno.h"
    #include "drv_vmmc_strerrno.h"
#else
    #error "drv_vmmc is missing"
#endif

#include "device_vmmc.h"

/* ============================= */
/* Defines                       */
/* ============================= */

/* ============================= */
/* Local variable definition     */
/* ============================= */


/* ============================= */
/* Global variable definition    */
/* ============================= */


#ifdef USE_FILESYSTEM
#ifdef DANUBE
   /** Prepare file names for DANUBE */
   IFX_char_t* sPRAMFile_VMMC = "voice_danube_firmware.bin";
   IFX_char_t* sPRAMFile_VMMC_Old = "danube_firmware.bin";
   IFX_char_t* sDRAMFile_VMMC = "";
   /** File holding coefficients. */
   IFX_char_t* sBBD_CRAM_File_VMMC = "danube_bbd.bin";
   IFX_char_t* sBBD_CRAM_File_VMMC_Old = "danube_bbd_fxs.bin";
#elif AR9
   /** Prepare file names for AR9 */
   IFX_char_t* sPRAMFile_VMMC = "voice_ar9_firmware.bin";
   IFX_char_t* sPRAMFile_VMMC_Old = "ar9_firmware.bin";
   IFX_char_t* sDRAMFile_VMMC = "";
   /** File holding coefficients. */
   IFX_char_t* sBBD_CRAM_File_VMMC = "ar9_bbd.bin";
   IFX_char_t* sBBD_CRAM_File_VMMC_Old = "ar9_bbd_fxs.bin";
#elif VINAX
   /** Prepare file names for VINAX */
   IFX_char_t* sPRAMFile_VMMC = "voice_vinax_firmware.bin";
   IFX_char_t* sPRAMFile_VMMC_Old = "firmware.bin";
   IFX_char_t* sDRAMFile_VMMC = "";
   /** File holding coefficients. */
   IFX_char_t* sBBD_CRAM_File_VMMC = "bbd.bin";
   IFX_char_t* sBBD_CRAM_File_VMMC_Old = "";
#elif VR9
   /** Prepare file names for VR9 */
   IFX_char_t* sPRAMFile_VMMC = "voice_vr9_firmware.bin";
   IFX_char_t* sPRAMFile_VMMC_Old = "vr9_firmware.bin";
   IFX_char_t* sDRAMFile_VMMC = "";
   /** File holding coefficients. */
   IFX_char_t* sBBD_CRAM_File_VMMC = "vr9_bbd.bin";
   IFX_char_t* sBBD_CRAM_File_VMMC_Old = "vr9_bbd_fxs.bin";
#else
#endif
#endif /* USE_FILESYSTEM */

/** Device names */
IFX_char_t* sCH_DEV_NAME_VMMC = "/dev/vmmc";
IFX_char_t* sCTRL_DEV_NAME_VMMC = "/dev/vmmc";
IFX_char_t* sSYS_DEV_NAME_VMMC = "";

/** Holding device dependent data */
VMMC_IO_INIT VmmcDevInitStruct = {0};

/* ============================= */
/* Local function declaration    */
/* ============================= */
IFX_return_t DEVICE_Vmmc_Fillin_FW_Ptr(IFX_void_t* pDrvInit,
                                       TAPIDEMO_DEVICE_CPU_t* pCpuDevice,
                                       IFX_char_t* pPath);
IFX_void_t DEVICE_Vmmc_Release_FW_Ptr(IFX_void_t* pDrvInit);
IFX_return_t DEVICE_Vmmc_GetVersions(BOARD_t* pBoard);
IFX_return_t DEVICE_Vmmc_Init(BOARD_t* pBoard, IFX_char_t* pPath);
IFX_void_t DEVICE_Vmmc_DecodeErrnoForDrv(IFX_int32_t nCode);
IFX_return_t DEVICE_Vmmc_setupChannel(BOARD_t* pBoard, IFX_char_t* pPath);

#ifdef TD_USE_CHANNEL_INIT
IFX_return_t DEVICE_Vmmc_SetupChannelUseChInit(BOARD_t* pBoard, IFX_char_t* pPath);
#else /* TD_USE_CHANNEL_INIT */
IFX_return_t DEVICE_Vmmc_SetupChannelUseDevStart(BOARD_t* pBoard, IFX_char_t* pPath);
#endif /* TD_USE_CHANNEL_INIT */


/* ============================= */
/* Local function definition     */
/* ============================= */
/**
   Fills in firmware pointers.

   \param pDrvInit - handle to VMMC_IO_INIT structure, should not be null
   \param pCpuDevice - pointer to CPU device
   \param pPath - path to FW files for download

   \return IFX_SUCCES on ok, otherwise IFX_ERROR

   \remarks
*/
IFX_return_t DEVICE_Vmmc_Fillin_FW_Ptr(IFX_void_t* pDrvInit,
                                       TAPIDEMO_DEVICE_CPU_t* pCpuDevice,
                                       IFX_char_t* pPath)
{
   VMMC_IO_INIT* vinit = (VMMC_IO_INIT *) pDrvInit;
#ifdef USE_FILESYSTEM
   IFX_return_t ret;
   IFX_uint8_t *pPram = IFX_NULL;
   IFX_uint8_t *pBBD = IFX_NULL;
   IFX_char_t pszFullFilename[MAX_PATH_LEN] = {0};
   IFX_boolean_t bFoundBinFile = IFX_FALSE;
#endif /* USE_FILESYSTEM */

   /* check input arguments */
   if ((IFX_NULL == vinit) || (pCpuDevice == IFX_NULL))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if ((vinit->pBBDbuf != IFX_NULL) && (vinit->pPRAMfw != IFX_NULL))
   {
      /* fw and bbd files already downloaded. If there is need to
         download new fw and bbd files, first current buffers
         must be released - DEVICE_Vmmc_Release_FW_Ptr()*/
      TRACE(TAPIDEMO, DBG_LEVEL_LOW,
           ("FW and BBD files already downloaded.\n"));
      return IFX_SUCCESS;
   }

#ifdef USE_FILESYSTEM
   /* Reading PRAM_FW file */
   /* Find PRAM_FW file with default names */
   /* First check for new name, next for old one */
   bFoundBinFile = Common_FindPRAM_FW(pCpuDevice, pPath, pszFullFilename);
   if(IFX_TRUE == bFoundBinFile)
   {
      /* Fill in PRAM_FW buffers from binary */
      ret = TD_OS_FileLoad(pszFullFilename, &pPram, 
                           (IFX_size_t *) &vinit->pram_size);
      /* Release PRAM_FW memory if applicable */
      if ((IFX_ERROR == ret) || (IFX_NULL == pPram))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Error reading firmware file %s.\n(File: %s, line: %d)\n",
               pszFullFilename, __FILE__, __LINE__));
   
         if (pPram != IFX_NULL)
         {
            TD_OS_MemFree(pPram);
            pPram = IFX_NULL;
         }
         return IFX_ERROR;
      }
      else
      {
         vinit->pPRAMfw = pPram;
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Filesystem firmware binary used (%s).\n",
               pszFullFilename));
      } /* if ((IFX_ERROR == ret_fwram) ... */
   } /* if(IFX_TRUE == bFileFound) */
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Error, firmware file %s is not found.\n(File: %s, line: %d)\n",
            pCpuDevice->pszPRAM_FW_File[0], __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* Reading BBD_CRAM file */
   /* Find BBD_CRAM file with default name */
   /* First check for new name, next for old one */
   bFoundBinFile = Common_FindBBD_CRAM(pCpuDevice, pPath, pszFullFilename);
         if (IFX_SUCCESS == ret) 
         {
         /* Fill in BBD_CRAM buffers from binary */
         ret = TD_OS_FileLoad(pszFullFilename, &pBBD, 
                              (IFX_size_t *) &vinit->bbd_size);
         /* Release BBD_CRAM memory if applicable */
         if ((IFX_ERROR == ret) || (IFX_NULL == pBBD)) 
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                 ("Error reading BBD file %s.\n(File: %s, line: %d)\n",
                pszFullFilename, __FILE__, __LINE__));
      
            if (pBBD != IFX_NULL)
            {
               TD_OS_MemFree(pBBD);
               pBBD = IFX_NULL;
            }
            return IFX_ERROR;
         }
         else
         {
            TRACE(TAPIDEMO, DBG_LEVEL_NORMAL, ("Filesystem BBD binary used (%s).\n",
                                                pszFullFilename));
            vinit->pBBDbuf = pBBD;
            bFoundBinFile = IFX_TRUE;
         }
      }
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error, BBD file %s is not found.\n(File: %s, line: %d)\n",
             pCpuDevice->pszBBD_CRAM_File[0], __FILE__, __LINE__));
      return IFX_ERROR;
   }

#endif /* USE_FILESYSTEM */

  return IFX_SUCCESS;
} /* DEVICE_Vmmc_Fillin_FW_Ptr */


/**
   Release firmware pointers.

   \param pDrvInit - handle to VMMC_IO_INIT structure

   \return none

   \remarks
*/
IFX_void_t DEVICE_Vmmc_Release_FW_Ptr(IFX_void_t* pDrvInit)
{
   VMMC_IO_INIT* vinit = (VMMC_IO_INIT *) pDrvInit;

   /* check input arguments */
   if (vinit == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return;
   }

   /* pram file was read from file, as default isn't set: free memory */
   if (vinit->pPRAMfw != IFX_NULL)
   {
      TD_OS_MemFree(vinit->pPRAMfw);
      vinit->pPRAMfw = IFX_NULL;
   }

   if (vinit->pBBDbuf != IFX_NULL)
   {
      TD_OS_MemFree(vinit->pBBDbuf);
      vinit->pBBDbuf = IFX_NULL;
   }

} /* DEVICE_Vmmc_Release_FW_Ptr() */

/* ============================= */
/* Global function definition    */
/* ============================= */

/**
   Read version after succesfull initialization and display it.

   \param pBoard - pointer to board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark
*/
IFX_return_t DEVICE_Vmmc_GetVersions(BOARD_t* pBoard)
{
   IFX_return_t ret;
   VMMC_IO_VERSION vinVers;

   memset(&vinVers, 0, sizeof(VMMC_IO_VERSION));

   ret = TD_IOCTL(pBoard->nDevCtrl_FD, FIO_GET_VERS, (IFX_int32_t) &vinVers);
   if (IFX_SUCCESS == ret)
   {
      TAPIDEMO_PRINTF(("VMMC [version 0x%2X, type 0x%2X, channels %d] ready!\n"
                       "FW Version %d.%d\n"
                       "VMMC Driver version %d.%d.%d.%d\n",
                       vinVers.nChip, vinVers.nType, vinVers.nChannel,
                       vinVers.nEdspVers, vinVers.nEdspIntern,
                       (int) (vinVers.nDrvVers >> 24) & 0xFF,
                       (int) (vinVers.nDrvVers >> 16) & 0xFF,
                       (int) (vinVers.nDrvVers >> 8)  & 0xFF,
                       (int) (vinVers.nDrvVers)       & 0xFF));

      COM_GET_FW_VERSION(vinVers);

   }
   else
   {
      TAPIDEMO_PRINTF(("VMMC version could not be read.\n"));
   }

   return ret;
} /* DEVICE_Vmmc_GetVersions() */

/**
   Init device.

   \param pBoard - pointer to board
   \param pPath - path to FW files for download

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark
*/
IFX_return_t DEVICE_Vmmc_Init(BOARD_t* pBoard, IFX_char_t* pPath)
{
   IFX_TAPI_CH_INIT_t Init = {0};
   IFX_TAPI_TONE_t tone;
   IFX_int32_t i = 0;
   TAPIDEMO_DEVICE_t      *pDevice = IFX_NULL;
   TAPIDEMO_DEVICE_CPU_t  *pCpuDevice = IFX_NULL;

   /* check input arguments */
   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if ((pDevice = Common_GetDevice_CPU(pBoard->pDevice)) == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err, CPU device not found. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   pCpuDevice = &pDevice->uDevice.stCpu;

#ifndef TD_USE_CHANNEL_INIT
   if(g_pITM_Ctrl->oVerifySytemInit.fEnabled)
#endif /* TD_USE_CHANNEL_INIT */
   {
      DEVICE_Vmmc_Fillin_FW_Ptr(pCpuDevice->pDevDependStuff,
                                pCpuDevice, pPath);
   
      /* Set tapi init structure */
      memset(&Init, 0, sizeof(IFX_TAPI_CH_INIT_t));
      Init.nMode = IFX_TAPI_INIT_MODE_VOICE_CODER;
      Init.pProc = (IFX_void_t*) pCpuDevice->pDevDependStuff;
   }

   if(g_pITM_Ctrl->oVerifySytemInit.fEnabled)
   {
      /* Read version */
      DEVICE_Vmmc_GetVersions(pBoard);
      COM_VerifySystemInit(pBoard, &Init);
   }
   else
   {
      for (i = 0; i <= pBoard->nMaxCoderCh; i++)
      {

         if (pBoard->nMaxCoderCh == i)
         {
            /* Read version */
            DEVICE_Vmmc_GetVersions(pBoard);
            break;
         }
#ifdef TD_USE_CHANNEL_INIT
            /* Initialize all system channels */
#ifdef TAPI_VERSION4
         if (!pBoard->fSingleFD)
#endif
         {
            if (IFX_SUCCESS != TD_IOCTL(pBoard->nCh_FD[i], IFX_TAPI_CH_INIT,
                                     (IFX_int32_t) &Init))
            {
               TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                     ("ERR on ioctl cmd IFX_TAPI_CH_INIT, ch %d "
                      "(File: %s, line: %d)\n", i, __FILE__, __LINE__));
               break;
            }
         } /* (!pBoard->fSingleFD */
#endif /* TD_USE_CHANNEL_INIT */

         /* Set appropriate feeding on all (analog) line channels */
         if (i < pBoard->nMaxAnalogCh)
         {
            /* Set line in standby */
#ifdef TAPI_VERSION4
            if (!pBoard->fSingleFD)
#endif /* TAPI_VERSION4 */
            {
               if (IFX_SUCCESS != TD_IOCTL(pBoard->nCh_FD[i],
                                           IFX_TAPI_LINE_FEED_SET,
                                           IFX_TAPI_LINE_FEED_STANDBY))
               {
                  TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                        ("ERR on ioctl cmd IFX_TAPI_LINE_FEED_SET, ch %d  "
                         "(File: %s, line: %d)\n", i, __FILE__, __LINE__));
                  break;
               }
            } /* if (!pBoard->fSingleFD) */
         } /* if (i < pBoard->nMaxAnalogCh) */
      } /* for */
   }

   DEVICE_Vmmc_Release_FW_Ptr(pCpuDevice->pDevDependStuff);

   if(!g_pITM_Ctrl->oVerifySytemInit.fEnabled)
   {
      if (pBoard->nMaxCoderCh != i)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Init failed for ch %d, (File: %s, line: %d)\n",
                (int) i, __FILE__, __LINE__));
         return IFX_ERROR;
      }
   }

   memset (&tone, 0, sizeof(tone));

   tone.simple.format = IFX_TAPI_TONE_TYPE_SIMPLE;
   tone.simple.index  = 50;
   tone.simple.levelA = -1;
   tone.simple.levelB = -1;
   tone.simple.levelC = -1;
   tone.simple.levelD = -1;
   tone.simple.loop   = 1;
   tone.simple.freqA  = 396;
   tone.simple.freqB  = 440;
   tone.simple.freqC  = 495;
   tone.simple.freqD  = 528;
   tone.simple.pause = 10;
   tone.simple.cadence[0]     = 100;
   tone.simple.frequencies[0] = 1;
   tone.simple.modulation[0]  = 0;
   tone.simple.cadence[1]     = 100;
   tone.simple.frequencies[1] = 2;
   tone.simple.modulation[1]  = 0;
   tone.simple.cadence[2]     = 100;
   tone.simple.frequencies[2] = 4;
   tone.simple.modulation[2]  = 0;
   tone.simple.cadence[3]     = 100;
   tone.simple.frequencies[3] = 8;
   tone.simple.modulation[3]  = 0;

   TD_IOCTL(pBoard->nDevCtrl_FD, IFX_TAPI_TONE_TABLE_CFG_SET, &tone);

   tone.simple.index  = 51;
   tone.simple.frequencies[0] = 8;
   tone.simple.frequencies[1] = 4;
   tone.simple.frequencies[2] = 2;
   tone.simple.frequencies[3] = 1;

   TD_IOCTL(pBoard->nDevCtrl_FD, IFX_TAPI_TONE_TABLE_CFG_SET, &tone);

   return IFX_SUCCESS;
} /* DEVICE_Vmmc_Init() */

/**
   Handles the DRIVER error codes and stack.

   \param nCode - error code

   \remarks
*/
IFX_void_t DEVICE_Vmmc_DecodeErrnoForDrv(IFX_int32_t nCode)
{
   IFX_int32_t i = 0;

   for (i = 0; i < VMMC_ERRNO_CNT; ++i)
   {
      if (VMMC_drvErrnos[i] == (nCode & 0xFFFF))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("LL Err: %s\n", VMMC_drvErrStrings[i]));
         break;
      }
   }
}

/**
   Return last error number for vmmc driver.

   \param nFd  -  channel file descriptor

   \return number of error if successful, otherwise IFX_ERROR.
   \remarks
*/
IFX_return_t DEVICE_Vmmc_GetErrnoNum(IFX_int32_t nFd)
{
   IFX_int32_t nLastErr = 0, ret;

   ret = TD_IOCTL(nFd, FIO_LASTERR, (IFX_int32_t) &nLastErr);

   if (IFX_ERROR == ret)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Failed to get error number for fd %d, (File: %s, line: %d)\n",
             nFd, __FILE__, __LINE__));
      return IFX_ERROR;
   }
   return nLastErr;
}

/**
   Downloads fw and bbd files to TAPI and starts TAPI.

   \param pBoard - pointer to board
   \param pPath - path to FW files for download

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.
*/
IFX_return_t DEVICE_Vmmc_setupChannel(BOARD_t* pBoard, IFX_char_t* pPath)
{
#ifdef TD_USE_CHANNEL_INIT 
   return DEVICE_Vmmc_SetupChannelUseChInit(pBoard, pPath);
#else
   return DEVICE_Vmmc_SetupChannelUseDevStart(pBoard, pPath);
#endif /* TD_USE_CHANNEL_INIT */
}

#ifdef TD_USE_CHANNEL_INIT
/**
   Downloads fw and bbd files and calls IFX_TAPI_CH_INIT for first channel.

   \param pBoard - pointer to board
   \param pPath - path to FW files for download

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remarks It is necessary to call this function to have working the
    IFX_TAPI_CAP_LIST.
*/
IFX_return_t DEVICE_Vmmc_SetupChannelUseChInit(BOARD_t* pBoard, IFX_char_t* pPath)
{
   IFX_TAPI_CH_INIT_t Init;
   IFX_return_t ret = IFX_SUCCESS;
   IFX_char_t sdev_name[MAX_DEV_NAME_LEN_WITH_NUMBER];
   IFX_int32_t nCh_FD = -1;
   TAPIDEMO_DEVICE_t *pDevice = IFX_NULL;
   TAPIDEMO_DEVICE_CPU_t *pCpuDevice = IFX_NULL;

   /* check input arguments */
   TD_PTR_CHECK(pBoard, IFX_ERROR);
   TD_PTR_CHECK(pPath, IFX_ERROR);

   if ((pDevice = Common_GetDevice_CPU(pBoard->pDevice)) == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err, CPU device not found. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   pCpuDevice = &pDevice->uDevice.stCpu;

   ret = DEVICE_Vmmc_Fillin_FW_Ptr(pCpuDevice->pDevDependStuff,
                                   pCpuDevice, pPath);
   if (ret == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Error, required binaries files are not downloaded.\n"
            "(File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* open channel 0, 10 max len of int32 as string */
   if(MAX_DEV_NAME_LEN_WITH_NUMBER > (strlen(pBoard->pszChDevName) + 2 * 10))
   {
      sprintf(sdev_name, "%s%d%d", (char *) pBoard->pszChDevName,
              (int) pBoard->nID, 1);
   }
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Buffer overflow, array index of 'sdev_name' "
            "may be outside the bounds. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   nCh_FD = Common_Open(sdev_name);
   if (0 > nCh_FD)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid channel (ch 0) file descriptor."
            "(File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* Set tapi init structure */
   memset(&Init, 0, sizeof(IFX_TAPI_CH_INIT_t));
   Init.nMode = IFX_TAPI_INIT_MODE_VOICE_CODER;
   Init.pProc = (IFX_void_t*) pCpuDevice->pDevDependStuff;

   ret = TD_IOCTL(nCh_FD, IFX_TAPI_CH_INIT, (IFX_int32_t) &Init);
   if (IFX_SUCCESS != ret)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("ERR on ioctl cmd IFX_TAPI_CH_INIT, ch 0 "
             "(File: %s, line: %d)\n", __FILE__, __LINE__));
      TD_OS_DeviceClose(nCh_FD);
      return IFX_ERROR;
   }

   TD_OS_DeviceClose(nCh_FD);

   return IFX_SUCCESS;
}

#else /* TD_USE_CHANNEL_INIT */

/**
   Downloads fw and bbd files to TAPI and starts TAPI.

   \param pBoard - pointer to board
   \param pPath - path to FW files for download

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.
*/
IFX_return_t DEVICE_Vmmc_SetupChannelUseDevStart(BOARD_t* pBoard, IFX_char_t* pPath)
{
   VMMC_IO_INIT oVmmcInit;
   IFX_return_t ret = IFX_SUCCESS;
   TAPIDEMO_DEVICE_t *pDevice = IFX_NULL;
   TAPIDEMO_DEVICE_CPU_t *pCpuDevice = IFX_NULL;
   IFX_int32_t nDevFd;
   VMMC_DWLD_t oBBD_Data;
   IFX_TAPI_DEV_START_CFG_t oStartConfig = {0};

   /* check input arguments */
   TD_PTR_CHECK(pBoard, IFX_ERROR);
   TD_PTR_CHECK(pPath, IFX_ERROR);

   if ((pDevice = Common_GetDevice_CPU(pBoard->pDevice)) == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err, CPU device not found. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   pCpuDevice = &pDevice->uDevice.stCpu;

   ret = DEVICE_Vmmc_Fillin_FW_Ptr(pCpuDevice->pDevDependStuff,
                                   pCpuDevice, pPath);
   if (ret == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Error, required binaries files are not downloaded.\n"
            "(File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   /* open device file descriptor */
   nDevFd = Common_Open(pBoard->pszCtrlDevName);
   if (0 > nDevFd)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err, Failed to open device %s fd. (File: %s, line: %d)\n",
             pBoard->pszCtrlDevName, __FILE__, __LINE__));
      return IFX_ERROR;
   }
   /* stop TAPI - all resources allocated by TAPI for this specific device
      are freed */
   if (IFX_SUCCESS != TD_IOCTL(nDevFd, IFX_TAPI_DEV_STOP, NO_PARAM))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err, Failed to stop. (File: %s, line: %d)\n",
             __FILE__, __LINE__));
      TD_OS_DeviceClose(nDevFd);
      return IFX_ERROR;
   }
   /* Set tapi init structure */
   memset(&oVmmcInit, 0, sizeof(VMMC_IO_INIT));
   oVmmcInit.pPRAMfw = ((VMMC_IO_INIT*)pCpuDevice->pDevDependStuff)->pPRAMfw;
   oVmmcInit.pram_size = ((VMMC_IO_INIT*)pCpuDevice->pDevDependStuff)->pram_size;
   if (IFX_SUCCESS != TD_IOCTL(nDevFd, FIO_FW_DOWNLOAD, &oVmmcInit))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err, Failed download FW. (File: %s, line: %d)\n",
             __FILE__, __LINE__));
      TD_OS_DeviceClose(nDevFd);
      return IFX_ERROR;
   }
   /* start TAPI - reserve all resources needed by TAPI
      for this specific device. */
   oStartConfig.nMode = IFX_TAPI_INIT_MODE_VOICE_CODER;
   if (IFX_SUCCESS != TD_IOCTL(nDevFd, IFX_TAPI_DEV_START, &oStartConfig))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err, Failed to start device. (File: %s, line: %d)\n",
             __FILE__, __LINE__));
      TD_OS_DeviceClose(nDevFd);
      return IFX_ERROR;
   }
   /* Set tapi init structure */
   memset(&oBBD_Data, 0, sizeof(VMMC_DWLD_t));
   oBBD_Data.buf = ((VMMC_IO_INIT*)pCpuDevice->pDevDependStuff)->pBBDbuf;
   oBBD_Data.size = ((VMMC_IO_INIT*)pCpuDevice->pDevDependStuff)->bbd_size;
   if (IFX_SUCCESS != TD_IOCTL(nDevFd, FIO_BBD_DOWNLOAD, &oBBD_Data))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err, Failed download BBD. (File: %s, line: %d)\n",
             __FILE__, __LINE__));
      TD_OS_DeviceClose(nDevFd);
      return IFX_ERROR;
   }
   TD_OS_DeviceClose(nDevFd);
   return IFX_SUCCESS;
}
#endif /* TD_USE_CHANNEL_INIT */

/**
   Register device to the board.

   \param pCpuDevice - pointer to device

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remarks
*/
IFX_return_t DEVICE_Vmmc_Register(TAPIDEMO_DEVICE_CPU_t* pCpuDevice)
{
   /* check input arguments */
   if (pCpuDevice == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   pCpuDevice->Version = DEVICE_Vmmc_GetVersions;
   pCpuDevice->DecodeErrno = DEVICE_Vmmc_DecodeErrnoForDrv;
   pCpuDevice->GetErrno = DEVICE_Vmmc_GetErrnoNum;
   pCpuDevice->FillinFwPtr = DEVICE_Vmmc_Fillin_FW_Ptr;
   pCpuDevice->ReleaseFwPtr = DEVICE_Vmmc_Release_FW_Ptr;
   return IFX_SUCCESS;
}


