#ifndef _COM_CLIENT_H
#define _COM_CLIENT_H
/******************************************************************************

                              Copyright (c) 2009
                            Lantiq Deutschland GmbH
                     Am Campeon 3; 85579 Neubiberg, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

  ****************************************************************************
   Module      : com_client.h
   Date        : 2007-05-10
   Description : This file contains definitions of communication functions.
 ****************************************************************************
   \file com_client.h

   \remarks

   \note Changes:
*******************************************************************************/

/* ============================= */
/* Includes                      */
/* ============================= */

#include "tapidemo.h"
#include "control_pc.h"
#include "itm_version.h"

/* ============================= */
/* Global Defines                */
/* ============================= */

#define COM_COMMAND_PORT    1234
/*#define COM_SENDING_PORT    9999*/
#define COM_SENDING_PORT    514

#define COM_DIRECTION_OUT   IFX_TRUE
#define COM_DIRECTION_IN    IFX_FALSE

/** max length of buffer used to sending traces to control PC */
#define TD_COM_BUF_SIZE     512

/** default debug level for ITM */
#define TD_ITM_DBG_LEVEL_DEFAULT             DBG_LEVEL_OFF

/**
   Stores the trace string in temporal variable.

   \param ...     - variable number of arguments

   \return

   \remark
      Macro is used to remove parenthesis from TRACE macro's message argument
      Used only for sending Trace messages through communication socket.
*/
#ifdef VXWORKS
#define COM_GENERATE_STRING(ARGS...) \
   snprintf(g_buf, TD_COM_BUF_SIZE, ## ARGS)
#else /* VXWORKS */
#define COM_GENERATE_STRING(...) \
   snprintf(g_buf, TD_COM_BUF_SIZE, __VA_ARGS__)
#endif /* VXWORKS */

/**
   Prints a trace message without sending it through communication socket.

   \param name     - Name of the trace group
   \param level    - level of this message
   \param message  - a printf compatible formated string + opt. arguments

   \return

   \remark
      The string will be redirected via printf if level is higher or equal to
      the actual level for this trace group ( please see SetTraceLevel ).
*/
#define TRACE_LOCAL(name,level,message) \
      do \
      { \
          TD_OS_PRN_USR_DBG_NL(name, level, message); \
      } while(0)

/* ============================= */
/* Local Macros & Definitions    */
/* ============================= */

/* ============================= */
/* Global Structures and enums   */
/* ============================= */

/* ============================= */
/* Global Variable declaration   */
/* ============================= */

/** global pointer to ITM control structure */
extern TD_ITM_CONTROL_t* g_pITM_Ctrl;

/** global buffer for sending ITM messages */
extern IFX_char_t g_buf[TD_COM_BUF_SIZE];

/* ============================= */
/* Global function declaration   */
/* ============================= */

extern TD_OS_socket_t COM_InitCommunicationSocket(IFX_uint32_t,
                                               IFX_uint16_t, IFX_boolean_t);
extern IFX_return_t COM_SendMessage(IFX_int32_t, IFX_char_t*);
extern IFX_void_t COM_CleanUpComSockets(IFX_void_t);
extern IFX_return_t COM_ResetCommunicationSockets(CTRL_STATUS_t* pCtrl,
                                                 fd_set* pRFds);
/*extern IFX_return_t COM_SendTraceMessage(IFX_int32_t, IFX_char_t*);*/

#endif /* _COM_CLIENT_H */
