#ifndef _DEV_HELP_H
#define _DEV_HELP_H
/******************************************************************************

                              Copyright (c) 2009
                            Lantiq Deutschland GmbH
                     Am Campeon 3; 85579 Neubiberg, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

  ****************************************************************************
   Module      : debug_help.h
   Date        : 2008-05-09
   Description : This file contains developer helping functions.
 ****************************************************************************
   \file dev_help.h

   \remarks

   \note Changes:
*******************************************************************************/

/* ============================= */
/* Includes                      */
/* ============================= */

#include "tapidemo.h"

/* ============================= */
/* Global Defines                */
/* ============================= */

#ifdef TD_IOCTL_TRACE
/** macro is used to call ioctl() so it can be easyly changed
   to function that for example prints ioctl name or makes error handling,
   it should be used only for TAPI drivers (not sockets) */
#define TD_IOCTL(nFd, nIoctl, pArg) \
   TD_DEV_Ioctl(nFd, nIoctl, (IFX_int32_t) pArg , #nIoctl, __FILE__, __LINE__)

/**
   Call ioctl with error handling

   \param nRet - ioctl return value
   \param nFd  - file descriptor on which ioctl will be used on
   \param nFct - ioctl number
   \param pArg - pointer to structure (ioctl parameter)
   \param nPhoneNo - phone number (NO_PHONE_NUMBER if not applicable)
   \param pBoard   - pointer to board
   \param ch   - channel number
   \param dev  - device number (only tapi v4, in v3 use 0)
   \param bReturn  - if set macro should end function

   */
#define TD_DBG_IOCTL(nRet, nFd, nFct, pArg, nPhoneNo, pBoard, ch, dev, bReturn)  \
        do \
        { \
           nRet = TD_DEV_Ioctl(nFd, nFct, (IFX_int32_t) pArg , \
                               #nFct, __FILE__, __LINE__); \
           if (IFX_SUCCESS > nRet) \
           { \
              TRACE(TAPIDEMO, DBG_LEVEL_HIGH, \
                    ("Err, ioctl(%d, "#nFct", %d) failed, ch %d, dev %d\n", \
                     nFd, (IFX_int32_t) pArg, ch, dev)); \
              if (NO_PHONE_NUMBER != nPhoneNo && IFX_NULL != pBoard) \
              { \
                 TRACE(TAPIDEMO, DBG_LEVEL_HIGH, \
                       ("     Phone No %d, %s,\n", \
                        nPhoneNo, pBoard->pszBoardName)); \
              } \
              TRACE(TAPIDEMO, DBG_LEVEL_HIGH, \
                    ("     File: %s, line: %d.\n", \
                     __FILE__, __LINE__)); \
              if (IFX_NULL != pBoard) \
              { \
                 Common_ErrorHandle(pBoard->nDevCtrl_FD, pBoard, dev); \
              } \
              if (TD_RETURN == bReturn) \
              { \
                 return IFX_ERROR; \
              } \
           } \
        } while (0);

/**
   Call single pin configuration. 
   
   \param nFd - GPIO port file descriptor.
   \param nFct - IOCTL command.
   \param nPort - GPIO port.
   \param nPin - GPIO pin.
   \param nValue - GPIO value.
   \param nModule - GPIO module ID.
   \param nRet - Return value: IFX_SUCCESS on ok, otherwise IFX_ERROR
*/
#define TD_GPIO_IOCTL_PARM(nFd, nFct, nPort, nPin, nValue, nModule ,nRet)\
{ \
   struct ifx_gpio_ioctl_parm param = {0};\
   memset(&param, 0, sizeof (param)); \
   param.port = nPort;\
   param.pin = nPin;\
   param.value = nValue;\
   param.module = nModule;\
   nRet = TD_DEV_Ioctl(nFd, nFct, (IFX_int32_t) param , \
                       #nFct, __FILE__, __LINE__); \
   if (IFX_SUCCESS != nRet)\
   { \
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH, \
            ("Error: ioctl: "#nFct", (%d, %d), value: %d failed.\n",\
             nPort, nPin, nValue));\
   }\
};

#endif /* TD_IOCTL_TRACE */

/* ============================= */
/* Local Macros & Definitions    */
/* ============================= */

/* ============================= */
/* Global Structures and enums   */
/* ============================= */

/* ============================= */
/* Global Variable definition    */
/* ============================= */

/* ============================= */
/* Global function declaration   */
/* ============================= */

extern IFX_return_t TD_DEV_PrintMappedDataCh(BOARD_t* pBoard);
extern IFX_return_t TD_DEV_PrintConnections(BOARD_t* pBoard);

#ifdef TD_IOCTL_TRACE
IFX_return_t TD_DEV_IoctlSetNameOfFd(IFX_int32_t nFd, IFX_char_t* pszFdName);
IFX_return_t TD_DEV_IoctlCleanUp(IFX_void_t);
IFX_int32_t TD_DEV_Ioctl(IFX_int32_t nFd, IFX_int32_t nIoctl, IFX_int32_t pArg,
                         IFX_char_t* pszIoctl,
                         IFX_char_t* pszFile, IFX_int32_t nLine);
#endif /* TD_IOCTL_TRACE */

#endif /* _DEV_HELP_H */

