#ifndef _PCM_H
#define _PCM_H
/******************************************************************************

                              Copyright (c) 2009
                            Lantiq Deutschland GmbH
                     Am Campeon 3; 85579 Neubiberg, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

  ****************************************************************************
   Module      : pcm.h
   Date        : 2006-03-29
   Description : This file enchance tapi demo with PCM support.
 ****************************************************************************
   \file pcm.h

   \remarks

   \note Changes:
*******************************************************************************/

/* ============================= */
/* Includes                      */
/* ============================= */

/* ============================= */
/* Global Defines                */
/* ============================= */

/* ============================= */
/* Local Macros & Definitions    */
/* ============================= */

/* ============================= */
/* Global Structures and enums   */
/* ============================= */

/** PCM specific data */
typedef struct _TAPIDEMO_PCM_DATA_t
{
   /** Sample rate, minimum value is 512 kHz,
      should be set during PCM configuration */
   IFX_TAPI_PCM_IF_DCLFREQ_t nPCM_Rate;
   /** MAX number of timeslots */
   IFX_int32_t nMaxTimeslotNum;
} TAPIDEMO_PCM_DATA_t;

/** timeslots direction */
typedef enum _TS_DIRECTION_t
{
   /** receiving timeslot */
   TS_RX = 1,
   /** transmiting timeslot */
   TS_TX = 2
} TS_DIRECTION_t;

/** Flag for no pcm channel */
enum { NO_FREE_PCM_CH = -1 };

/** Flag for no timeslot */
enum { NO_FREE_TIMESLOT = -1 };

/** Flag for timeslot is not set */
enum { TIMESLOT_NOT_SET = -1 };

/* ============================= */
/* Global Variable declaration   */
/* ============================= */

extern TAPIDEMO_PCM_DATA_t oPCM_Data;

/* ============================= */
/* Global function declaration   */
/* ============================= */
IFX_int32_t PCM_GetTimeslots(TS_DIRECTION_t nTS_direction,
                             IFX_boolean_t fWideBand);
IFX_return_t PCM_FreeTimeslots(IFX_int32_t nTimeslot);
IFX_return_t PCM_Init(PROGRAM_ARG_t* pProgramArg);
IFX_return_t PCM_SetActivation(TAPIDEMO_PORT_DATA_t* pPortData,
                               IFX_int32_t nDataCh_FD,
                               IFX_boolean_t fActivate,
                               IFX_int32_t nTimeslot_RX, 
                               IFX_int32_t nTimeslot_TX,
                               IFX_boolean_t fWideBand);
IFX_boolean_t PCM_CheckPhoneNum(IFX_char_t* prgnDialNum,
                                IFX_int32_t nDialNrCnt);
IFX_boolean_t PCM_StartConnection(TAPIDEMO_PORT_DATA_t* pPortData,
                                  PROGRAM_ARG_t* pProgramArg,
                                  BOARD_t* pBoard,
                                  CONNECTION_t* pConn);
IFX_boolean_t PCM_EndConnection(TAPIDEMO_PORT_DATA_t* pPortData,
                                PROGRAM_ARG_t* pProgramArg,
                                BOARD_t* pBoard,
                                CONNECTION_t* pConn);
IFX_return_t PCM_MapToPhone(IFX_int32_t nDstCh,
                            IFX_int32_t nAddCh,
                            IFX_boolean_t fDoMapping,
                            BOARD_t* pBoard);
IFX_return_t PCM_MapToPCM(IFX_int32_t nDstCh,
                          IFX_int32_t nAddCh,
                          IFX_boolean_t fDoMapping,
                          BOARD_t* pBoard);
IFX_return_t PCM_MapToData(IFX_int32_t nDstCh,
                           IFX_int32_t nAddCh,
                           IFX_boolean_t fDoMapping,
                           BOARD_t* pBoard);
IFX_return_t PCM_FreeCh(IFX_int32_t nPCM_Ch, BOARD_t* pBoard);
IFX_return_t PCM_ReserveCh(IFX_int32_t nPCM_Ch, BOARD_t* pBoard);
IFX_int32_t PCM_GetFreeCh(BOARD_t* pBoard);
IFX_int32_t PCM_GetFD_OfCh(IFX_int32_t nPCM_Ch, BOARD_t* pBoard);
IFX_return_t PCM_CfgInterface(BOARD_t* pBoard);
IFX_return_t PCM_Release(PROGRAM_ARG_t* pProgramArg, BOARD_t* pBoard);
IFX_void_t PCM_ReleaseSocket(CTRL_STATUS_t* pCtrl);
#endif /* _PCM_H */

