
/******************************************************************************

                              Copyright (c) 2009
                            Lantiq Deutschland GmbH
                     Am Campeon 3; 85579 Neubiberg, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

  ****************************************************************************
   Module      : qos.c
   Date        : 2005-11-29
   Description : This file contains the implementation of the functions for
                 the tapi demo working with quality of service
   \file

   \remarks QOS now only works for first called peer, not an array of peers.

   \note Changes: Only support in LINUX at the moment.
                  Look drv_qos.h for further info.
*******************************************************************************/

/* ============================= */
/* Includes                      */
/* ============================= */

#include "tapidemo.h"
#include "abstract.h"
#include "voip.h"
#include "qos.h"
#ifdef LINUX
#include "tapidemo_config.h"
#endif

/* ============================= */
/* Global Structures             */
/* ============================= */

/* ============================= */
/* Global function declaration   */
/* ============================= */

/* ============================= */
/* Local variable definition     */
/* ============================= */

/* ============================= */
/* Local function declaration    */
/* ============================= */

/* ============================= */
/* Local function definition     */
/* ============================= */

/* ============================= */
/* Global function definition    */
/* ============================= */


#ifdef QOS_SUPPORT

/**
   Turn QoS Service on

   \param pProgramArg - pointer to program arguments
*/
IFX_void_t QOS_TurnServiceOn(PROGRAM_ARG_t *pProgramArg)
{
   if (IFX_NULL == pProgramArg)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return;
   }
} /* QOS_TurnServiceOn() */


/**
   Initialize qos pair, only used with external calls.

   \param pCtrl        - pointer to status control structure
   \param pConn        - pointer to phone connection
   \param nDevIdx      - device index
*/
IFX_void_t QOS_InitializePairStruct(CTRL_STATUS_t* pCtrl,
                                    CONNECTION_t* pConn,
                                    IFX_int32_t nDevIdx)
{

/*   IFX_int32_t qos_port = -1; */


   if ((IFX_NULL == pConn) || (IFX_NULL == pCtrl))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return;
   }

    /* Initialize pair struct */
   if ((pCtrl->pProgramArg->oArgFlags.nQos)
       || (0 == pConn->oSession.nInSession))
   {

      pConn->oSession.oPair.srcPort = htons(VOICE_UDP_QOS_PORT + 
                                            pConn->nUsedCh * 2 + nDevIdx);

      pConn->oSession.oPair.srcAddr =
         pCtrl->pProgramArg->oMy_IP_Addr.sin_addr.s_addr;
      /* pConn->oSession.oPair.destPort = htons(qos_port); */
      pConn->oSession.oPair.destAddr =
         pConn->oConnPeer.oRemote.oToAddr.sin_addr.s_addr;
   }

} /* QOS_InitializePairStruct() */


/**
   Cleans QoS session

   \param pCtrl       - pointer to status control structure
   \param pProgramArg - pointer to program arguments
*/
IFX_void_t QOS_CleanUpSession(CTRL_STATUS_t* pCtrl, PROGRAM_ARG_t *pProgramArg)
{
   IFX_int32_t i = 0, j, nDataFd;
   IFX_return_t ret;
   BOARD_t* pBoard;

   if ((IFX_NULL == pProgramArg) || (IFX_NULL == pCtrl))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return;
   }

   /* Clean up qos sessions */
   if (pProgramArg->oArgFlags.nQos)
   {
      /* for all boards */
      for (j=0; j<pCtrl->nBoardCnt; j++)
      {
         /* get next board */
         pBoard = &pCtrl->rgoBoards[j];
         /* for all data channels */
         for (i = 0; i < pBoard->nMaxCoderCh; i++)
         {
            nDataFd = VOIP_GetFD_OfCh(i, pBoard);
            if (NO_DEVICE_FD != nDataFd)
            {
               /* stops the entire qos support, it deactivates and deletes all
                  sessions on all channels, according to description
                  in drv_tapi_qos_io.h */
               ret = TD_IOCTL(nDataFd, FIO_QOS_CLEAN, 0);
               if ( IFX_ERROR == ret )
               {
                  TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                       ("Err: FIO_QOS_CLEAN failed (File: %s, line: %d)\n",
                        __FILE__, __LINE__));
               }
               /* clean up on one channel should be sufficient */
               break;
            }
            else
            {
               TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                    ("Err: FIO_QOS_CLEAN %s failed to get FD of ch %d.\n"
                     "(File: %s, line: %d)\n",
                     pBoard->pszBoardName, i, __FILE__, __LINE__));
            }
         }
      }
      TRACE(TAPIDEMO, DBG_LEVEL_NORMAL, ("QoS: Device cleanuped\n"));
   }

} /* QOS_CleanUpSession() */


/**
   Check if QOS Session handles this connection.

   \param  pCtrl - pointer to status control structure
   \param  pConn - pointer to phone connection

   \return IFX_TRUE if QoS is handling data, otherwise return IFX_FALSE
*/
IFX_boolean_t QOS_HandleService(CTRL_STATUS_t* pCtrl, CONNECTION_t* pConn)
{
   /* check input parameters */
   TD_PTR_CHECK(pCtrl, IFX_ERROR);
   TD_PTR_CHECK(pConn, IFX_ERROR);

   /* In case of QoS support, packets are not to be handled by the application
      once a qos session was initiated. */
   if (pCtrl->pProgramArg->oArgFlags.nQos &&
       1 == pConn->oSession.nInSession)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_LOW,
            ("Qos - UDP redirect will handle data instead of tapidemo.\n"));
      return IFX_TRUE;
   }

   return IFX_FALSE;
} /* QOS_HandleService() */


/**
   Starts qos session.

   \param pCtrl - pointer to status control structure
   \param pConn - pointer to phone connection
   \param nAction - flag to setup port, address of session
*/
IFX_return_t QOS_StartSession(CONNECTION_t* pConn,
                              PHONE_t* pPhone,
                              TD_QOS_ACTION_t nAction,
                              IFX_int32_t nDevIdx)
{
   IFX_return_t ret = IFX_SUCCESS;
   CTRL_STATUS_t* pCtrl = IFX_NULL;
   IFX_int32_t nDataCh = -1;

   TD_PTR_CHECK(pConn, IFX_ERROR);
   TD_PTR_CHECK(pPhone, IFX_ERROR);

   pCtrl = pPhone->pBoard->pCtrl;
/*    nDataCh = ABSTRACT_GetDataChannelOfFD(pConn->nUsedCh_FD, pPhone->pBoard);       */
/*    TD_PARAMETER_CHECK(((nDataCh >= 0) && (nDataCh < pPhone->pBoard->nMaxCoderCh)), */
/*                         nDataCh);                                                  */
   nDataCh = pConn->nUsedCh;

   /* Remote call : start qos session */
   /* Also use QoS or udp redirect on local calls */
/*   if ((EXTERN_VOIP_CALL == pConn->nType) */
   if ((PCM_CALL != pConn->nType) &&
       (pCtrl->pProgramArg->oArgFlags.nQos) &&
       (0 == pConn->oSession.nInSession))
   {
      if (EXTERN_VOIP_CALL == pConn->nType)
      {
         /* Start qos session */
         if (TD_SET_ADDRESS_PORT == nAction)
         {
            /* For caller */
            pConn->oSession.oPair.srcPort = htons( VOICE_UDP_QOS_PORT + 
                                                   pConn->nUsedCh * 2 + 
                                                   nDevIdx);
            pConn->oSession.oPair.srcAddr =
               pCtrl->pProgramArg->oMy_IP_Addr.sin_addr.s_addr;
            pConn->oSession.oPair.destPort = htons(pConn->oConnPeer.oRemote.oToAddr.sin_port);
            pConn->oSession.oPair.destAddr =
             pConn->oConnPeer.oRemote.oToAddr.sin_addr.s_addr;
         }
         else if (TD_SET_ONLY_PORT == nAction)
         {
            /* For called */
            pConn->oSession.oPair.destPort = htons(pConn->oConnPeer.oRemote.oToAddr.sin_port);
         }
      }
      else /* Local call */
      {
         /* set port numbers according to phone device and channel. */
         pConn->oSession.oPair.srcPort =
            htons(pConn->nUsedCh * 2 + nDevIdx * 100 + 1);
         pConn->oSession.oPair.destPort =
            htons(pConn->oConnPeer.oLocal.pPhone->nDataCh * 2
               + pConn->oConnPeer.oLocal.pPhone->pBoard->nBoard_IDX * 100 + 1);
         pConn->oSession.oPair.destAddr =
            pCtrl->pProgramArg->oMy_IP_Addr.sin_addr.s_addr;
         pConn->oSession.oPair.srcAddr =
            pCtrl->pProgramArg->oMy_IP_Addr.sin_addr.s_addr;
      }

      TRACE(TAPIDEMO, DBG_LEVEL_LOW,
            ("Phone No %d: FIO_QOS_START --> src:0x%08X:%d, dst:0x%08X:%d\n",
             pPhone->nPhoneNumber,
            (int) pConn->oSession.oPair.srcAddr,
            (int) pConn->oSession.oPair.srcPort,
            (int) pConn->oSession.oPair.destAddr,
            (int) pConn->oSession.oPair.destPort));

      ret = TD_IOCTL(pConn->nUsedCh_FD, FIO_QOS_START, &pConn->oSession.oPair);
      TD_DRV_FUNC_ASSERT(ret, pPhone, FIO_QOS_START,
                         0, nDataCh, TD_NO_RETURN);
      if ( IFX_SUCCESS == ret )
      {
         TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
           ("Phone No %d: QoS started, data ch %d,"
            "src port: %d, dst port: %d\n",
            pPhone->nPhoneNumber,
            pConn->nUsedCh,
            pConn->oSession.oPair.srcPort, 
            pConn->oSession.oPair.destPort));
      }
      pConn->oSession.nInSession = 1;
   }
   return ret;
} /* QOS_StartSession() */


/**
   Stops qos session.

   \param pCtrl - pointer to status control structure
   \param pConn - pointer to phone connection
*/
IFX_return_t QOS_StopSession(CONNECTION_t* pConn, 
                             PHONE_t* pPhone)
{

   IFX_return_t ret = IFX_SUCCESS;
   CTRL_STATUS_t* pCtrl = IFX_NULL;

   TD_PTR_CHECK(pConn, IFX_ERROR);
   TD_PTR_CHECK(pPhone, IFX_ERROR);
   pCtrl = pPhone->pBoard->pCtrl;

   /* Remote call : stop qos session */
/* Also use QoS or udp redirect on local calls */
   if ((PCM_CALL != pConn->nType)
        && (pCtrl->pProgramArg->oArgFlags.nQos)
        && (1 == pConn->oSession.nInSession))
   {
      /* Stop qos session */
      TRACE(TAPIDEMO, DBG_LEVEL_LOW,
       ("Phone No %d: FIO_QOS_STOP: src:0x%08X:%d, dst:0x%08X:%d\n",
        pPhone->nPhoneNumber,
        (int) pConn->oSession.oPair.srcAddr,
        (int) pConn->oSession.oPair.srcPort,
        (int) pConn->oSession.oPair.destAddr,
        (int) pConn->oSession.oPair.destPort));

      ret = TD_IOCTL(pConn->nUsedCh_FD, FIO_QOS_STOP, 0);
      TD_DRV_FUNC_ASSERT(ret, pPhone, FIO_QOS_STOP,
                         0, pConn->nUsedCh, TD_RETURN);

      TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
           ("Phone No %d: QoS stopped, data ch %d\n",
            pPhone->nPhoneNumber,
            pConn->nUsedCh));

      pConn->oSession.nInSession = 0;
   }
   return ret;
} /* QOS_StopSession() */

/* ---------------------------------------------------------------------- */
#else /* QOS_SUPPORT */
/* ---------------------------------------------------------------------- */

IFX_void_t QOS_TurnServiceOn(PROGRAM_ARG_t *pProgramArg)
{
   if (IFX_NULL == pProgramArg)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return;
   }

   TAPIDEMO_PRINTF(("QoS support not available\n"));

   pProgramArg->oArgFlags.nQos = 0;
}


IFX_void_t QOS_InitializePairStruct(CTRL_STATUS_t* pCtrl,
                                    CONNECTION_t* pConn,
                                    IFX_int32_t nDevIdx)
{
   /* empty function */
}


IFX_void_t QOS_CleanUpSession(CTRL_STATUS_t* pCtrl, PROGRAM_ARG_t *pProgramArg)
{
   /* empty function */
}


IFX_boolean_t QOS_HandleService(CTRL_STATUS_t* pCtrl, CONNECTION_t* pConn)
{
   /* Don't have QoS support so application can handle the data. */
   return IFX_FALSE;
}


IFX_return_t QOS_StartSession(CONNECTION_t* pConn,
                              PHONE_t* pPhone,
                              TD_QOS_ACTION_t nAction,
                              IFX_int32_t nDevIdx)
{
   /* empty function */
   return IFX_SUCCESS;
}


IFX_return_t QOS_StopSession(CONNECTION_t* pConn, 
                             PHONE_t* pPhone)
{
   /* empty function */
   return IFX_SUCCESS;
}

#endif /* QOS_SUPPORT */

