#ifndef _TD_DECT_STACK_H_
#define _TD_DECT_STACK_H_
/******************************************************************************

                              Copyright (c) 2009
                            Lantiq Deutschland GmbH
                     Am Campeon 3; 85579 Neubiberg, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

  ****************************************************************************/

/**
    \file  td_dect_stack.h
    \date  2010-01-29
    \brief DECT stack handling implementation for tapidemo application.

   This file includes methods which initialize and handle DECT stack module.
*/


#ifdef LINUX
#include "tapidemo_config.h"
#endif
#include "ifx_types.h"

#ifdef TD_DECT

#include <stdio.h>
#include <string.h>
/* According to POSIX 1003.1-2001 */
#include <sys/select.h>
/* According to earlier standards */
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stddef.h>
#include <sys/socket.h>         
#include <netinet/in.h>
#include <netdb.h>
#include <sys/un.h>

/* Workaround - IFX_TRUE and IFX_FALSE are defined twice time.
   In file ifx_types.h and ifx_common_defs.h */ 
#define IFX_TRUE 
#define IFX_FALSE 
#include "ifx_common_defs.h"
#undef IFX_TRUE
#undef IFX_FALSE

#include "IFX_DECT_Stack.h"
#include "IFX_DECT_MU.h"
#include "IFX_DECT_CSU.h"


#include "common.h"

extern IFX_int32_t g_nDectDebugLevelInternal;
extern TD_ENUM_2_NAME_t TD_rgDectEventName[];

/** event message for all channels is send */
#define TD_DECT_STACK_ALL_CH           0x0F0F

#define TD_DECT_STACK_MODE             IFX_DECT_ASYNC_MODE

#define TD_DECT_PAGEKEY_DRV              "/dev/pb"
#define IFX_DECT_REG_KEY_PRESS_DUR     5000  /* in ms*/
#define IFX_DECT_REG_DURATION          60000 /* in ms*/
#ifdef TD_DECT_DEBUG_ENABLE

/** used for debug TRACES that can be removed in near future,
    they were used for development */
#define TD_DECT_DEBUG(name, level, message) \
   do \
   { \
      if (g_nDectDebugLevelInternal != DBG_LEVEL_OFF) \
      { \
         TRACE(name,level,message); \
      } \
   } \
   while (0)
#else /* TD_DECT_DEBUG */
#define TD_DECT_DEBUG(name, level, message)
#endif /* TD_DECT_DEBUG */

IFX_return_t TD_DECT_STACK_SetCtrlPointer(TD_DECT_STACK_CTRL_t* pDectStackCtrl);
IFX_return_t TD_DECT_STACK_CliSocketOpen(CTRL_STATUS_t* pCtrl);
/* Declarations of functions that were copied from dect agent START */

/*! \brief Maximum number of timers supported by timer module */
# define IFX_MAX_TIMERS         40

/*! 
   \brief   This is the timer callback function.  This function is implemented
   by the module, requesting for services from the Timer module.  This function
   pointer needs to be passed to the Timer module when requesting for a timer 
   to be started.  This function will be called back when the timer expires.
   \param[in] uiTimerId The timer Id of the Timer which is expired.  
   \param[in] pvPrivateData Pointer to Data given by 
               the module which triggered the timer
   \return  void No Value
*/
typedef void (*pfn_IFX_TIM_TimerCallBack)(
                    IN uint32 uiTimerId,
                    IN void* pvPrivateData );

typedef struct
{
        uint16 unTimerId;               /*The Timer Id returned by the TLIB timer library*/
        void * pvPrivateData;   /*The private data*/
        uint16 unPrivateDataLen;/*Length of private data*/
        uint32 uiTimerId;               /*The Timer Id returned by timer agent*/
        pfn_IFX_TIM_TimerCallBack pfnTimerCallBack;
}x_IFX_TimerInfo;

/*!
   \brief  This function indicates the availability of a message in the Timer FIFO.
   This function must be called when there is data available for reading in 
           the fifo descriptor passed during initialization. This functions reads the fifo
           data and invokes the appropriate timer callback function.
   \param[in] iTimFd Timer Fifo Fd that is used for reading the 
           timer message.
   \return If returned IFX_FAILURE, then something wrong with timer interface module
           and caller should reinitialize this module.
*/
e_IFX_Return IFX_TIM_TimerMsgReceived( IN int32 iTimFd);

/*!
   \brief      This function initializes the Timer Interface module.
   This function is invoked when the timer module is initialized.
   \param[in]  szFifoName Name of fifo to be used by timer interface module          
   \param[out] puiFifoFd Fifo descriptor is returned in this parameter. Caller 
               of this function should wait for this fd and when ready for reading,
               call IFX_TIM_TimerMsgReceived to process and invoke timer callback.
   \return     If timer initialization is success IFX_SUCCESS is returned, else
               IFX_FAILURE is returned.
*/
e_IFX_Return IFX_TIM_Init(
                IN char8* szFifoName,
                OUT uint32* puiFifoFd );


/*!
   \brief      This function shuts down the timer interface module. 
   \return     void No Value
*/
void IFX_TIM_Shut();

/*! 
    \brief       This function starts a timer.  This function is called  by
    modules to start a timer. The modules can start more than one timer. 
    \param[in]   uiTimeOut     Time out value in milliseconds.
    \param[in]   pvPrivateData Private data of the caller. Timer module does not 
                 modify this data.
    \param[in]   uiPrivateDataLen Length of the private data. If length is zero
                 no data will be copied, only pointer (pvPrivateData) value will
                 be passed into timer callback function.
    \param[in]   pfnTimerCallBack Pointer to Timer call back function
    \param[out]  puiTimerId On success, this contains the Timer Id. 
      This Id is unique and should be used to stop timer and also when timer fires.
    \return      On success returns IFX_SUCCESS otherwise IFX_FAILURE.
*/
e_IFX_Return IFX_TIM_TimerStart(
           IN uint32 uiTimeOut,
           IN void* pvPrivateData,
           IN uint16 uiPrivateDataLen,
           IN pfn_IFX_TIM_TimerCallBack pfnTimerCallBack,
           OUT uint32* puiTimerId
          );

/*! 
    \brief  This function stops a previously started timer.  
    The modules invoke this function to stop the timer that was started 
    by invoking IFX_TIM_StartTimer.  If timer had fired already or if the 
    Timer Id is wrong, this function returns failure.
    \param[in] uiTimerId  Timer Id of the timer to be stopped.
    \return On success returns IFX_SUCCESS otherwise IFX_FAILURE.
*/
e_IFX_Return IFX_TIM_TimerStop(
          IN uint32 uiTimerId
          );

IFX_return_t IFX_DECT_ProcessPagekeyMessage( 
                       uint32 uiPagingKeyPressedDur //in ms
                                 );
/* Declarations of functions that were copied from dect agent STOP */
IFX_boolean_t TD_DECT_STACK_IsPagingActive(IFX_void_t);

IFX_return_t TD_DECT_STACK_PipeOpen(CTRL_STATUS_t* pCtrl);

IFX_return_t TD_DECT_STACK_SendToTD(TD_DECT_STACK_CTRL_t* pDectStackCtrl,
                                    TD_DECT_EVENT_t nEvent,
                                    IFX_int32_t nHandset,
                                    TD_DECT_STACK_DATA_t* pData);

/* callback functions MU */
IFX_int32_t TD_DECT_STACK_MU_Notify(x_IFX_DECT_MU_NotifyInfo *pNotifyInfo);
IFX_int32_t TD_DECT_STACK_MU_ModemReset();

/* callback functions CSU */
e_IFX_Return TD_DECT_STACK_CallInitiate(uint32 uiCallHdl,
                                        uchar8 ucHandsetId,
                                        x_IFX_DECT_CSU_CallParams *pxCallParams,
                                        e_IFX_DECT_ErrReason *peStatus,
                                        uint32 *puiPrivateData);

e_IFX_Return TD_DECT_STACK_CallAccept(uint32 uiCallHdl,
                                      x_IFX_DECT_CSU_CallParams *pxCallParams,
                                      uint32 uiPrivateData);

e_IFX_Return TD_DECT_STACK_CallAnswer(uint32 uiCallHdl,
                                      x_IFX_DECT_CSU_CallParams *pxCallParams,
                                      uint32 uiPrivateData);

e_IFX_Return TD_DECT_STACK_CallRelease(uint32 uiCallHdl,
                                       x_IFX_DECT_CSU_CallParams *pxCallParams,
                                       e_IFX_DECT_RelType eReason,
                                       uint32 uiPrivateData);

e_IFX_Return TD_DECT_STACK_InfoReceived(uint32 uiCallHdl,
                                        x_IFX_DECT_CSU_CallParams *pxCallParams,
                                        uint32 uiPrivateData);

e_IFX_Return TD_DECT_STACK_ServiceChange(uint32 uiCallHdl,
                                         x_IFX_DECT_CSU_CallParams *pxCallParams,
                                         e_IFX_DECT_CSU_CodecChangeType eType,
                                         uint32 uiPrivateData);

e_IFX_Return TD_DECT_STACK_VoiceModify(uint32 uiCallHdl,
                                       x_IFX_DECT_CSU_CallParams *pxCallParams,
                                       e_IFX_DECT_CSU_VoiceEventType eType,
                                       uint32 uiPrivateData);

e_IFX_Return TD_DECT_STACK_CipherCfm(uint32 uiCallHdl,
                                     x_IFX_DECT_CSU_CallParams *pxCallParams,
                                     uint32 uiPrivateData);

e_IFX_Return TD_DECT_STACK_CallToggle(uint32 uiSrcCallHdl,
                                      uint32 uiDstCallHdl,
                                      x_IFX_DECT_CSU_CallParams *pxCallParams,
                                      e_IFX_DECT_ErrReason *peStatus,
                                      uint32 uiPrivateData);

e_IFX_Return TD_DECT_STACK_CallHold(uint32 uiCallHdl,
                                    x_IFX_DECT_CSU_CallParams *pxCallParams,
                                    e_IFX_DECT_ErrReason *peStatus,
                                    uint32 uiPrivateData);

e_IFX_Return TD_DECT_STACK_CallResume(uint32 uiCallHdl,
                                      x_IFX_DECT_CSU_CallParams *pxCallParams,
                                      e_IFX_DECT_ErrReason *peStatus,
                                      uint32 uiPrivateData);

e_IFX_Return TD_DECT_STACK_CallTransfer(uint32 uiSrcCallHdl,
                                        uint32 uiDstCallHdl,
                                        x_IFX_DECT_CSU_CallParams *pxCallParams,
                                        e_IFX_DECT_ErrReason *peStatus,
                                        uint32 uiPrivateData);

e_IFX_Return TD_DECT_STACK_CallConference(uint32 uiSrcCallHdl,
                                          uint32 uiDstCallHdl,
                                          x_IFX_DECT_CSU_CallParams *pxCallParams,
                                          e_IFX_DECT_ErrReason *peStatus,
                                          uint32 uiPrivateData);
/* end of callback functions */


IFX_return_t TD_DECT_STACK_ChannelToHandsetInit(CTRL_STATUS_t* pCtrl,
                                                BOARD_t* pBoard);

IFX_return_t TD_DECT_STACK_AddHandset(TD_DECT_STACK_CTRL_t* pDectStackCtrl,
                                      IFX_int32_t nHandset);
IFX_return_t TD_DECT_STACK_AddCallHdl(TD_DECT_STACK_CTRL_t* pDectStackCtrl,
                                      IFX_int32_t nHandset, IFX_int32_t nCallHdl);

IFX_int32_t TD_DECT_STACK_GetChannelOfHandset(TD_DECT_STACK_CTRL_t* pDectStackCtrl,
                                              IFX_int32_t nHandset);
IFX_int32_t TD_DECT_STACK_GetChannelOfCallHdl(TD_DECT_STACK_CTRL_t* pDectStackCtrl,
                                              IFX_int32_t nCallHdl);

IFX_int32_t TD_DECT_STACK_GetHandsetOfCallHdl(TD_DECT_STACK_CTRL_t* pDectStackCtrl,
                                              IFX_int32_t nCallHdl);
IFX_int32_t TD_DECT_STACK_GetHandsetOfChannel(TD_DECT_STACK_CTRL_t* pDectStackCtrl,
                                              IFX_int32_t nCh);

IFX_int32_t TD_DECT_STACK_GetCallHdlOfHandset(TD_DECT_STACK_CTRL_t* pDectStackCtrl,
                                              IFX_int32_t nHandsetId);
IFX_int32_t TD_DECT_STACK_GetCallHdlOfChannel(TD_DECT_STACK_CTRL_t* pDectStackCtrl,
                                              IFX_int32_t nCh);

IFX_return_t TD_DECT_STACK_BMCRegParamGet(x_IFX_DECT_BMCRegParams *pxBmc);

IFX_return_t TD_DECT_STACK_RcFreqParamGet(IFX_uint8_t *pucFreqTx,
                                          IFX_uint8_t *pucFreqRx,
                                          IFX_uint8_t *pucFreqRange);

IFX_return_t TD_DECT_STACK_OscTrimParamGet(x_IFX_DECT_OscTrimVal *pxOscTrimVal);

IFX_return_t TD_DECT_STACK_GaussianValGet(uint16 *pucGaussianVal);

IFX_return_t TD_DECT_STACK_BS_PinGet(char8 *pcBasePin);
IFX_return_t TD_DECT_STACK_TPCParamGet(x_IFX_DECT_TransmitPowerParam *pxTPCParams);

IFX_return_t TD_DECT_STACK_SetDownloadPath(CTRL_STATUS_t* pCtrl);

IFX_void_t TD_DECT_STACK_InitDownload(IFX_int32_t iStatus);

IFX_return_t TD_DECT_STACK_RFPI_Get(char8 *pcRFPI, CTRL_STATUS_t* pCtrl);

IFX_return_t TD_DECT_STACK_DbgLevelSet(CTRL_STATUS_t* pCtrl,
                                       x_IFX_DECT_StackInitCfg* pxInitCfg);
#endif /* TD_DECT */

#endif /* _TD_DECT_STACK_H_ */

