/******************************************************************************

                              Copyright (c) 2009
                            Lantiq Deutschland GmbH
                     Am Campeon 3; 85579 Neubiberg, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

  ****************************************************************************/

/**
   \file tapidemo.c
   \date 2003-03-24
   \brief Demonstrates a simple PBX with voice over IP on Linux or VxWorks OS.

   \note Changes:
      Date: 28.11.2005 it is working on new board with new drivers, kernel.
                       CID structure changed.
                       Removed EXCHFD_SUPPORT, its obsolete.
                       Extracted QOS, TAPI SIGNALLING into different files.
      Date: 15.12.2005 Added conference support.
      Date: 21.04.2006 Major changes, added PCM, Features support. Also added
                       Event handling (bitfields or messages).
      Date: 07.12.2006 Major changes, support for FXO, DANUBE with CPE
                       extension board, multiple boards, parsing argumens in
                       VxWorks.
      Date: 31.05.2007 Added support for board DuSLIC-XT.
*/


/* ============================= */
/* Includes                      */
/* ============================= */

#ifdef VXWORKS
#include <selectLib.h>
#include <inetLib.h>
#endif

#ifdef LINUX
#include "tapidemo_config.h"
#endif

#include "tapidemo_version.h"
#include "tapidemo.h"
#include "abstract.h"
#include "analog.h"
#include "cid.h"
#include "conference.h"
#include "common.h"
#include "event_handling.h"
#include "feature.h"
#include "parse_cmd_arg.h"
#include "pcm.h"
#include "qos.h"
#include "state_trans.h"
#include "lib_tapi_signal.h"
#include "voip.h"

#ifdef FXO
   #include "common_fxo.h"
#endif /* FXO */

#ifdef EASY50510
   #include "board_easy50510.h"
#endif /* EASY50510 */

#ifdef DXT
#include "board_easy3111.h"
#endif /* DXT */

#ifdef EASY336
#include "lib_svip.h"
#include "lib_svip_rm.h"
#include "board_easy336.h"
#ifdef WITH_VXT
#include "board_xt16.h"
#endif /* WITH_VXT */
#endif /* EASY336 */

#ifdef XT16
/*#include "lib_svip.h"*/
/*#include "board_easy336.h"*/
#include "board_xt16.h"
#endif

#if (defined(TD_T38_FAX_TEST) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW))
#include "./t38_fax_test/td_t38_fax_test.h"
#endif /* (defined(TD_T38_FAX_TEST) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW)) */

#ifdef TD_DECT
#include "td_dect.h"
#endif /* TD_DECT */

/* ============================= */
/* Debug interface               */
/* ============================= */

TD_OS_PRN_USR_MODULE_CREATE(TAPIDEMO, TD_OS_PRN_LEVEL_HIGH);

/* ============================= */
/* Global Structures             */
/* ============================= */

/* ============================= */
/* Global function declaration   */
/* ============================= */

/* ============================= */
/* Local variable definition     */
/* ============================= */

const IFX_char_t TD_EOL = '\n';

/** Holding information about each connection */
static CTRL_STATUS_t oCtrlStatus;
/** Holding program arguments */
static PROGRAM_ARG_t oProgramArg;

/** List of combinations */
const BOARD_COMBINATION_t BOARD_COMBINATIONS[MAX_COMBINATIONS] =
{
   /* comb idx, board cnt, list of boards
                           {board id, type, sub type, init (normal/gpio) } */
   { ONE_CPE, 1, { {1, TYPE_VINETIC},
                   {0}, {0}, {0} } },
   /* reserved value */
   { BOARD_COMB_RESERVED, 1, { {1, TYPE_NONE},
                               {0}, {0}, {0} } },
   { ONE_VMMC, 1, { {1, TYPE_DANUBE},
                    {0}, {0}, {0} } },
   { ONE_VMMC_ONE_CPE, 2, { {1, TYPE_DANUBE},
                            {1, TYPE_VINETIC},
                            {0}, {0} } },
   { ONE_DXT, 1, { {1, TYPE_DUSLIC_XT},
                   {0}, {0}, {0} } },
   { TWO_DXTS, 2, { {1, TYPE_DUSLIC_XT},
                    {2, TYPE_DUSLIC_XT},
                    {0}, {0} } },
   { ONE_VMMC_VINAX, 1, { {1, TYPE_VINAX},
                    {0}, {0}, {0} } },
   { ONE_VMMC_EASY508XX, 1, { {1, TYPE_AR9},
                    {0}, {0}, {0} } },
   { X_SVIP, 1, { {1, TYPE_SVIP},
                  {0}, {0}, {0} } },
   { X_SVIP_X_XT16, 2, { {1, TYPE_SVIP},
                         {1, TYPE_XT16},
                         {0}, {0} } },
   { X_XT16, 1, { {1, TYPE_XT16},
                    {0}, {0}, {0} } },
   { ONE_EASY80910, 1, { {1, TYPE_VR9},
                       {0}, {0}, {0} } },
};

/** Contains board combination names. */
const IFX_char_t* BOARD_COMB_NAMES[MAX_COMBINATIONS] =
{
   "single board",
   "reserved",
   "EASY50712",
   "EASY50712 with EASY50510",
   "EASY3201",
   "EASY3201 with EASY3111",
   "EASY80800",
   "EASY508xx",
   "EASY336",
   "EASY336 and xT-16s",
   "ONE xT-16",
   "EASY80910"
};

#ifdef EASY336
/** PCM configuration */
RM_SVIP_PCM_CFG_t pcm_cfg;
#endif

#ifdef LINUX
/** Max log's length */
enum { MAX_LOG_LEN = 2048 };

/** Max message's length for external syslog: 2048+5
    Look into function TAPIDEMO_SendLogRemote() for more details */
enum { MAX_EXT_SYSLOG_MSG_LEN = 2053 };

#endif

/** Holding trace redirection settings */
TRACE_REDIRECTION_CTRL_t oTraceRedirection;

/* ============================= */
/* Local function declaration    */
/* ============================= */
IFX_return_t TAPIDEMO_BlockSignals(IFX_void_t);
static IFX_return_t TAPIDEMO_Setup(BOARD_t* pBoard,
                                   CTRL_STATUS_t* pCtrl,
                                   IFX_int32_t* nPhoneNum);
TD_OS_socket_t TAPIDEMO_InitAdminSocket(IFX_void_t);
IFX_return_t TAPIDEMO_HandleCallSeq(CTRL_STATUS_t* pCtrl);
#if defined(LINUX)
IFX_int32_t TAPIDEMO_InitSyslogSocket(IFX_void_t);
IFX_return_t TAPIDEMO_SendLogRemote(IFX_char_t* pBuffer, IFX_int32_t nSyslogSocket);
IFX_return_t TAPIDEMO_SendLog(IFX_char_t* pMsg);
IFX_return_t TAPIDEMO_PreapareLogRedirection(IFX_void_t);
IFX_return_t TAPIDEMO_CreateDaemon(CTRL_STATUS_t* pCtrl);
#endif
IFX_return_t TAPIDEMO_SetMainBoard(BOARD_t* pBoard);
IFX_return_t TAPIDEMO_CheckBoardCombination(BOARD_t* pBoard);
IFX_return_t TAPIDEMO_SetExtBoard(BOARD_t* pBoard);
IFX_boolean_t TAPIDEMO_ExtBoardCheckForDuplicate(IFX_int32_t nType, 
                                                 IFX_int32_t nID);
IFX_return_t TAPIDEMO_StartBoard(IFX_int32_t nIndex);
IFX_return_t TAPIDEMO_RebuildBoardArray(IFX_int32_t nIndex);

/* ============================= */
/* Local function definition     */
/* ============================= */

/* SIGNAL HANDLER */
static IFX_return_t TAPIDEMO_RegisterSigHandler(IFX_void_t);
/*static IFX_void_t TAPIDEMO_CleanUp(IFX_int32_t nSignal);*/
static void TAPIDEMO_CleanUp(int nSignal);
static IFX_int32_t TAPIDEMO_HandleDeviceThread(CTRL_STATUS_t* pCtrl);
static IFX_int32_t TAPIDEMO_HandleLogThread(TD_OS_ThreadParams_t *pThread);
static IFX_int32_t TAPIDEMO_HandleSocketThread(TD_OS_ThreadParams_t *pThread);

#ifdef  VXWORKS
/** Holds application task ID. */
static IFX_int32_t nTapiDemo_TID = -1;

/**
   This function is called after task is deleted in clear up everything.

   \param pTcb - pointer to deleted task's WIND_TCB
*/
void deleteHook(WIND_TCB* pTcb)
{
   WIND_TCB* evlog_tcb = IFX_NULL;

   evlog_tcb = taskTcb(nTapiDemo_TID);
   if (evlog_tcb == pTcb)
   {
      /* Only call this when our task is killed */
      TRACE(TAPIDEMO, DBG_LEVEL_LOW, ("deleteHook()\n"));

      TAPIDEMO_CleanUp(0);

      if (taskDeleteHookDelete((FUNCPTR) deleteHook) != IFX_SUCCESS)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Err delete task handler when task is deleted. "
               "(File: %s, line: %d)\n",
               __FILE__, __LINE__));
      }
   }
}
#endif /* VXWORKS */

/**
   Block all signals on specific thread.

   Signal handlers are shared between all threads: when a thread calls sigaction(), 
   it sets how the signal is handled not only for itself, but for all other 
   threads in the program as well. 
   On the other hand, signal masks are per-thread: each thread chooses which 
   signals it blocks independently of others.

   \return IFX_SUCCESS on success, otherwise IFX_ERROR
*/
IFX_return_t TAPIDEMO_BlockSignals(IFX_void_t)
{
#ifdef LINUX
   static sigset_t signal_mask;
   IFX_return_t ret;
   sigemptyset (&signal_mask);
   sigaddset (&signal_mask, SIGINT);
   sigaddset (&signal_mask, SIGTERM);
   sigaddset (&signal_mask, SIGABRT);
   sigaddset (&signal_mask, SIGQUIT);

   ret = pthread_sigmask (SIG_BLOCK, &signal_mask, NULL);
   if (ret != 0) 
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH, ("Error: Block signals for thread.\n"
            "(File: %s, line: %d)\n", __FILE__, __LINE__));
     return IFX_ERROR;
   }
#else
   TRACE(TAPIDEMO, DBG_LEVEL_LOW,
         ("Unsupported function TAPIDEMO_BlockSig()\n"));
#endif
   return IFX_SUCCESS;
} /* TAPIDEMO_BlockSignals */

/**
   Register signal handler which is used when task is deleted from LINUX by
   'kill' or CTRL + C command and from VXWORKS by 'td' command. So task can
   cleanup everything.

   \return IFX_SUCCESS on success, otherwise IFX_ERROR
*/
static IFX_return_t TAPIDEMO_RegisterSigHandler(IFX_void_t)
{
   TRACE(TAPIDEMO, DBG_LEVEL_LOW, ("TAPIDEMO_RegisterSigHandler()\n"));
#ifdef LINUX
   /*sighandler_t sig_handler = TAPIDEMO_CleanUp;*/
   struct sigaction act;

   act.sa_handler = TAPIDEMO_CleanUp;
   act.sa_flags = SA_NODEFER; /* Do not prevent signal from being blocked. */
   if (sigaction(SIGABRT, &act, IFX_NULL) < 0) /* CTRL + C */
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err adding signal handler. "
            "(File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   if (sigaction(SIGQUIT, &act, IFX_NULL) < 0)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err adding signal handler. "
            "(File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   if (sigaction(SIGINT, &act, IFX_NULL) < 0)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err adding signal handler. "
            "(File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   if (sigaction(SIGTERM, &act, IFX_NULL) < 0)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err adding signal handler. "
            "(File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
#elif VXWORKS
   if (taskDeleteHookAdd((FUNCPTR) deleteHook) != IFX_SUCCESS)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err adding delete task handler. "
            "(File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
#endif /* VXWORKS */

   return IFX_SUCCESS;
} /* TAPIDEMO_RegisterSigHandler */


/**
   Clean up memory after task ends, close all open connections, ...

   \param nSignal - in LINUX signal number in VXWORKS not used
*/
static void TAPIDEMO_CleanUp(int nSignal)
{
   IFX_int32_t i = 0;
   IFX_int32_t ch = 0;
   PHONE_t* pPhone = IFX_NULL;
   BOARD_t* pBoard = IFX_NULL;
#ifdef EASY336
   SVIP_RM_Status_t ret;
#endif
   TAPIDEMO_DEVICE_t *pDevice = IFX_NULL;
   TAPIDEMO_DEVICE_CPU_t *pCpuDevice = IFX_NULL;
#ifndef TAPI_VERSION4
   IFX_TAPI_SIG_DETECTION_t dtmfDetection;
#endif

#ifdef LINUX
   if ((nSignal == SIGINT) || (nSignal == SIGTERM) || (nSignal == SIGSTOP)
       || (nSignal == SIGABRT) ||(nSignal == SIGQUIT) ||(nSignal == SIGCHLD))
   {
#endif /* LINUX */

      TRACE(TAPIDEMO, DBG_LEVEL_NORMAL, ("TAPIDEMO_CleanUp()\n"));

      /* Stop thread: handle sockets */
      Common_StopThread(&oCtrlStatus.oMultithreadCtrl.oThread.oHandleSocketThreadCtrl,
                        &oCtrlStatus.oMultithreadCtrl.oPipe.oSocketSynchPipe,
                        &oCtrlStatus.oMultithreadCtrl.oLock.oSocketThreadStopLock);

      if (IFX_SUCCESS !=
          TD_OS_MutexDelete(&oCtrlStatus.oMultithreadCtrl.oMutex.mutexStateMachine))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Error: TD_OS_MutexDelete failed. (File: %s, line: %d)\n",
               __FILE__, __LINE__));
      }

#if (defined(TD_T38_FAX_TEST) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW))
      TD_T38_FAX_TEST_DeInit(&oCtrlStatus);
#endif /* (defined(TD_T38_FAX_TEST) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW)) */

      /* Cleanup stuff. */
      for (i = 0; i < oCtrlStatus.nBoardCnt; i++)
      {
         pBoard = &oCtrlStatus.rgoBoards[i];

         if (pBoard == IFX_NULL)
         {
            continue;
         }

         TRACE(TAPIDEMO, DBG_LEVEL_LOW,
               ("Clean all phones for this board\n"));

         for (ch = 0; ch < pBoard->nMaxAnalogCh; ch++)
         {
            pPhone = &pBoard->rgoPhones[ch];

            if (pPhone == IFX_NULL)
            {
               continue;
            }

            /* End with all connections, ringing, phone calls,
               cleanup mappings. Set phone to state S_READY.*/
            if ((&pPhone->rgoConn[0] != IFX_NULL) &&
                (pPhone->rgStateMachine[FXS_SM].nState != S_READY))
            {
               TRACE(TAPIDEMO, DBG_LEVEL_LOW,
                     ("Remove all connections for this phone\n"));

               pPhone->nIntEvent_FXS = IE_HOOKON;
               while (pPhone->nIntEvent_FXS != IE_NONE)
               {
                  ST_HandleState_FXS(&oCtrlStatus, pPhone,
                                     &pPhone->rgoConn[0]);
               }
               if (IFX_TRUE == oCtrlStatus.bInternalEventGenerated)
               {
                  EVENT_InternalEventHandle(&oCtrlStatus);
               }
            }

            /* Set line feed to disabled */
            Common_LineFeedSet(pPhone, IFX_TAPI_LINE_FEED_DISABLED);

            if (pPhone->pCID_Msg != IFX_NULL)
            {
               if (pPhone->pCID_Msg->message != IFX_NULL)
               {
                  TD_OS_MemFree(pPhone->pCID_Msg->message);
                  pPhone->pCID_Msg->message = IFX_NULL;
               }
               TRACE(TAPIDEMO, DBG_LEVEL_LOW,
                     ("Release CID messages from phone.\n"));
               TD_OS_MemFree(pPhone->pCID_Msg);
               pPhone->pCID_Msg = IFX_NULL;
            }
         } /* for (ch = 0; ... */
         
         /* disable the DTMF detection */
#ifndef TAPI_VERSION4
         memset(&dtmfDetection, 0, sizeof (IFX_TAPI_SIG_DETECTION_t));
         /* Stop detection of DTMF tones from local interface */
         dtmfDetection.sig = IFX_TAPI_SIG_DTMFTX;
         if (0 < pBoard->nMaxCoderCh)
         {
            TRACE(TAPIDEMO, DBG_LEVEL_LOW,
                  ("%s: Disable DTMF detection on all data channels \n",
                   pBoard->pszBoardName));
         }
         for (ch = 0; ch < pBoard->nMaxCoderCh; ch++)
         {
             if (IFX_SUCCESS != TD_IOCTL(pBoard->nCh_FD[ch], IFX_TAPI_SIG_DETECT_DISABLE,
                                        (IFX_int32_t) &dtmfDetection))
            {
               TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                    ("Err: IFX_TAPI_SIG_DETECT_DISABLE failed (File: %s, line: %d)\n",
                     __FILE__, __LINE__));
            }
         }
#endif /* TAPI_VERSION4 */
#ifdef TD_DECT
         /* DECT cleanup only for first board */
         if (0 == i)
         {
            /* close DECT specific FDs and clean up after DECT initialization */
            TD_DECT_Clean(&oCtrlStatus);
         }
#endif /* TD_DECT */
         if (oCtrlStatus.pProgramArg->oArgFlags.nQos)
         {
/* no QoS for SVIP */
#ifndef EASY336
            TRACE(TAPIDEMO, DBG_LEVEL_LOW, ("QOS_CleanUp()\n"));
            /* QoS Support, cleanup for all boards. */
            QOS_CleanUpSession(&oCtrlStatus, oCtrlStatus.pProgramArg);
#endif /* EASY336 */
         }

         VOIP_Release(oCtrlStatus.pProgramArg, pBoard);

         /* \todo Only one should be set to master? */
         if ((oCtrlStatus.pProgramArg->oArgFlags.nPCM_Master)
             || (oCtrlStatus.pProgramArg->oArgFlags.nPCM_Slave))
         {
            PCM_Release(oCtrlStatus.pProgramArg, pBoard);
         }

#ifndef EASY336
         /* Unmapp all mapped channels */
         TRACE(TAPIDEMO, DBG_LEVEL_LOW, ("Unmap default mapped channels.\n"));
         ABSTRACT_UnmapDefaults(&oCtrlStatus, pBoard);
#endif

#ifdef EASY336
         ret = SVIP_RM_DeInit(&pcm_cfg);
         if (ret != SVIP_RM_Success)
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                 ("Resource manager deinit failed with error code %d. "
                  "(File: %s, line: %d)\n",
                  ret, __FILE__, __LINE__));
         }
         TD_OS_MemFree(pcm_cfg.pDevsSVIP);
         TD_OS_MemFree(pcm_cfg.pDevsXT16);
#endif

         TRACE(TAPIDEMO, DBG_LEVEL_LOW, ("Close opened file descriptors.\n"));
         /* Close devices, sockets, ... for all boards. */
         Common_Close_FDs(pBoard);

         if ((pDevice = Common_GetDevice_CPU(pBoard->pDevice)) == IFX_NULL)
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                 ("Err, CPU device not found. (File: %s, line: %d)\n",
                  __FILE__, __LINE__));
         }
         else
         {
            pCpuDevice = &pDevice->uDevice.stCpu;

            /* when changing country settings memory is allocated
               for new BBD file name, it should be freed */
            if ((pCpuDevice->pszBBD_CRAM_File != IFX_NULL) &&
                (IFX_TRUE ==pBoard->fBBD_Changed))
            {
               TD_OS_MemFree(pCpuDevice->pszBBD_CRAM_File);
            }
         }

         TRACE(TAPIDEMO, DBG_LEVEL_LOW,
               ("Remove board %s.\n", pBoard->pszBoardName));
         /* Release boards and free memory. */
         pBoard->RemoveBoard(pBoard);

         pBoard = IFX_NULL;
      } /* for (i = 0; ... */

      TRACE(TAPIDEMO, DBG_LEVEL_LOW, ("Close opened sockets.\n"));

      if (!oCtrlStatus.pProgramArg->oArgFlags.nQos)
      {
         if (oCtrlStatus.nSumCoderCh > 0)
         {
            VOIP_ReleaseSocket(&oCtrlStatus);
         }
      }

      if (oCtrlStatus.rgnSockets != IFX_NULL)
      {
         TD_OS_MemFree(oCtrlStatus.rgnSockets);
         oCtrlStatus.rgnSockets = IFX_NULL;
      }

      /* Close socket used for call administration */
      TD_OS_SocketClose(oCtrlStatus.nAdminSocket);
      oCtrlStatus.nAdminSocket = NO_SOCKET;

      COM_ITM_CLEAN_UP(0);

#ifdef TD_IOCTL_TRACE
      TD_DEV_IoctlCleanUp();
#endif /* TD_IOCTL_TRACE */

#ifdef LINUX
      if (oCtrlStatus.pProgramArg->oArgFlags.nDaemon)
      {
         /* Stop thread: handle log */
         Common_StopThread(&oCtrlStatus.oMultithreadCtrl.oThread.oHandleLogThreadCtrl,
                           &oCtrlStatus.oMultithreadCtrl.oPipe.oLogSynchPipe,
                           &oCtrlStatus.oMultithreadCtrl.oLock.oLogThreadStopLock);
         /* Close pipe for redirect log */
         if (IFX_SUCCESS != 
             TD_OS_PipeClose(oCtrlStatus.oMultithreadCtrl.oPipe.oRedirectLogPipe.rgPipeFp[TD_PIPE_IN]))
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                 ("Error: TD_OS_PipeClose IN failed. (File: %s, line: %d)\n",
                  __FILE__, __LINE__));
         }
         if (IFX_SUCCESS != 
             TD_OS_PipeClose(oCtrlStatus.oMultithreadCtrl.oPipe.oRedirectLogPipe.rgPipeFp[TD_PIPE_OUT]))
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                 ("Error: TD_OS_PipeClose OUT failed. (File: %s, line: %d)\n",
                  __FILE__, __LINE__));
         }
         /* Remove pipe for redirect log */
         remove(oCtrlStatus.oMultithreadCtrl.oPipe.oRedirectLogPipe.rgPipeName);
      } /* Daemon */

      if (oCtrlStatus.pProgramArg->nTraceRedirection ==
          TRACE_REDIRECTION_SYSLOG_LOCAL)
      {
         closelog();
      }
      else if (oCtrlStatus.pProgramArg->nTraceRedirection ==
               TRACE_REDIRECTION_SYSLOG_REMOTE)
      {
         TD_OS_SocketClose(oTraceRedirection.nSyslogSocket);
         oTraceRedirection.nSyslogSocket = NO_SOCKET;
      }
      else if (oCtrlStatus.pProgramArg->nTraceRedirection ==
               TRACE_REDIRECTION_FILE)
      {
         TD_OS_FClose(oTraceRedirection.pFileLogFD);
         oTraceRedirection.pFileLogFD = IFX_NULL;
      }
      /* End with task */
      exit(0);
   }
#endif /* LINUX */
} /* TAPIDEMO_CleanUp */


/**
   Turn on services specified by user.

   \param pCtrl - program control structure
*/
IFX_return_t TAPIDEMO_RunServices(CTRL_STATUS_t* pCtrl)
{
   PROGRAM_ARG_t* pProgramArg;

   TD_PTR_CHECK(pCtrl, IFX_ERROR);
   TD_PTR_CHECK(pCtrl->pProgramArg, IFX_ERROR);

   /* set pointer to program settings structure */
   pProgramArg = pCtrl->pProgramArg;

   TD_OS_PRN_USR_LEVEL_SET(TAPIDEMO, pProgramArg->nDbgLevel);
#ifdef EASY336
   SVIP_RM_SetTraceLevel(pProgramArg->nDbgLevel);
#endif

   if (pProgramArg->oArgFlags.nQos)
   {
      QOS_TurnServiceOn(pProgramArg);
   }

   if ((pProgramArg->oArgFlags.nQos) &&
       (pProgramArg->oArgFlags.nQOS_Local))
   {
      /* With local UDP redirection conference should be disabled. */
      pProgramArg->oArgFlags.nConference = 0;
   }

   if (pProgramArg->oArgFlags.nCID)
   {
      CID_SetStandard(pProgramArg->nCIDStandard);
   }

#ifdef FXO
   if (pProgramArg->oArgFlags.nFXO)
   {
      if (pProgramArg->oArgFlags.nPCM_Slave)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("The PCM i/f can not be configured as a SLAVE when FXO is used.\n "
             "PCM i/f will be set as MASTER\n"));
         pProgramArg->oArgFlags.nPCM_Slave = 0;
      }
      /* General FXO intialization if needed */
      pProgramArg->oArgFlags.nPCM_Master = 1;
   }
#endif /* FXO */
   return IFX_SUCCESS;
}


/**
   Displays calling sequence.

   \param pCtrl - handle to control status structure

   \return always IFX_SUCCESS
*/
static IFX_int32_t TAPIDEMO_Call_Info(CTRL_STATUS_t* pCtrl)
{
   IFX_int32_t nCh = 0;
   IFX_int32_t j = 0;
   BOARD_t* pBoard = IFX_NULL;
   IFX_char_t* sTableSeparate = IFX_NULL;
#ifndef TAPI_VERSION4
#ifdef FXO
   FXO_t* pFXO;
   TAPIDEMO_DEVICE_FXO_t* pFxoDevice = IFX_NULL;
#endif /* FXO */
#endif /* TAPI_VERSION4 */

#ifndef TAPI_VERSION4
   sTableSeparate = "------------------------------------------------";
#else /* TAPI_VERSION4 */
   sTableSeparate = "----------------------------------------------------------";
#endif /*  TAPI_VERSION4 */

#ifndef TAPI_VERSION4
   TAPIDEMO_PRINTF(("Dialing sequences for different call types:\n"));
   TAPIDEMO_PRINTF((" a) Local connection: <c>\n"));
   TAPIDEMO_PRINTF((" b) Extern connection: <0iiic>\n"));
   TAPIDEMO_PRINTF((" c) PCM connection: <09iiic>\n"));
   TAPIDEMO_PRINTF((" d) FXO connection: <05d>\n"));
   TAPIDEMO_PRINTF((" e) FEATURE selection first digit must be 'STAR (*),\n"));
   TAPIDEMO_PRINTF(("    afterthat feature number (4 digits).\n"));
   TAPIDEMO_PRINTF((" f) CONFERENCING first digit must be 'HASH (#)',\n"));
   TAPIDEMO_PRINTF(("    afterthat normal dialing sequence,\n"));
   TAPIDEMO_PRINTF((" g) Extern connection with full ip:\n"
                    "    <0*xxx*xxx*xxx*xxx*dcc> - each part separated with STAR (*),\n"));
   TAPIDEMO_PRINTF(("\n"));
   TAPIDEMO_PRINTF(("Legend:\n"));
   TAPIDEMO_PRINTF(("<c> - channel number started from 1 (1 means channel 0).\n"));
   TAPIDEMO_PRINTF(("<iii> - lowest part of external board IP address (a.b.c.<iii>)\n"));
   TAPIDEMO_PRINTF(("<d> - FXO channel, started from 1 (1 means channel 0).\n"));
   TAPIDEMO_PRINTF(("<xxx*xxx*xxx*xxx> - external board IP address "
                    "(xxx.xxx.xxx.xxx)\n"));
#else
   TAPIDEMO_PRINTF((" a) Local connection: <dcc>\n"));
   TAPIDEMO_PRINTF((" b) Extern connection: <0iiidcc>\n"));
   TAPIDEMO_PRINTF((" c) PCM connection: <09iiidcc>\n"));
   TAPIDEMO_PRINTF((" d) FXO connection: <05Ddcc>\n"));
   TAPIDEMO_PRINTF((" e) FEATURE selection first digit must be 'STAR (*),\n"));
   TAPIDEMO_PRINTF(("    afterthat feature number (4 digits).\n"));
   TAPIDEMO_PRINTF((" f) CONFERENCING first digit must be 'HASH (#)',\n"));
   TAPIDEMO_PRINTF(("    afterthat normal dialing sequence,\n"));
   TAPIDEMO_PRINTF((" g) Extern connection with full ip:\n"
                    "    <0*xxx*xxx*xxx*xxx*dcc> - each part separated with STAR (*).\n"));
   TAPIDEMO_PRINTF(("\n"));
   TAPIDEMO_PRINTF(("Legend:\n"));
   TAPIDEMO_PRINTF(("<d> - device number started from 1 (1 means device 0).\n"));
   TAPIDEMO_PRINTF(("<cc> - channel number started from 1\n"
                    "       (01 means channel 0, 02 means channel 1, etc.).\n"));
   TAPIDEMO_PRINTF(("<iii> - lowest part of external board IP address (a.b.c.<iii>)\n"));
   TAPIDEMO_PRINTF(("<D> - FXO channel, only one digit at the moment\n"));
   TAPIDEMO_PRINTF(("<xxx*xxx*xxx*xxx> - external board IP address "
                    "(xxx.xxx.xxx.xxx)\n"));
#endif /* TAPI_VERSION4 */
   TAPIDEMO_PRINTF(("\n"));

   TAPIDEMO_PRINTF(("\n"));
   TAPIDEMO_PRINTF(("FXS channels calling table:\n"));
   TAPIDEMO_PRINTF((" %s\n", sTableSeparate));
#ifndef TAPI_VERSION4
   TAPIDEMO_PRINTF(("| Dial |          Board          | Phone channel |\n"));
#else
   TAPIDEMO_PRINTF(("| Dial |          Board          | Device | Phone channel  |\n"));
#endif
   TAPIDEMO_PRINTF(("|%s|\n", sTableSeparate));
   for (j = 0; j < pCtrl->nBoardCnt; j++)
   {
      pBoard = &pCtrl->rgoBoards[j];
      for (nCh = 0; nCh < pBoard->nMaxAnalogCh; nCh++)
      {
#ifndef TAPI_VERSION4
         TAPIDEMO_PRINTF(("|  %3d |     %15s     |      %3d      |\n",
                          (int) pBoard->rgoPhones[nCh].nPhoneNumber,
                          pBoard->pszBoardName, (int) nCh));
#else
         TAPIDEMO_PRINTF(("|  %03d |     %15s     |   %2d   |      %3d       |\n",
                          pBoard->rgoPhones[nCh].nPhoneNumber,
                          pBoard->pszBoardName,
                          pBoard->rgoPhones[nCh].nDev,
                          pBoard->rgoPhones[nCh].nPhoneCh));
#endif
      }
   }
#ifdef TD_DECT
   if (!pCtrl->pProgramArg->oArgFlags.nNoDect)
   {
      TAPIDEMO_PRINTF((" %s\n", sTableSeparate));
      TAPIDEMO_PRINTF(("| Dial |          Board          |  DECT channel |\n"));
      TAPIDEMO_PRINTF((" %s\n", sTableSeparate));
   }
   /* if dect disable nothing should be printed (nMaxAnalogCh == nMaxPhones) */
   for (j = 0; j < pCtrl->nBoardCnt; j++)
   {
      pBoard = &pCtrl->rgoBoards[j];
      for (nCh = pBoard->nMaxAnalogCh; nCh < pBoard->nMaxPhones; nCh++)
      {
         TAPIDEMO_PRINTF(("|  %3d |     %15s     |      %3d      |\n",
                          pBoard->rgoPhones[nCh].nPhoneNumber,
                          pBoard->pszBoardName,
                          pBoard->rgoPhones[nCh].nDectCh));
      }
   }
#endif /* TD_DECT */
   TAPIDEMO_PRINTF((" %s\n", sTableSeparate));
#ifndef TAPI_VERSION4
#ifdef FXO
   /* check if FXO enabled */
   if (pCtrl->pProgramArg->oArgFlags.nFXO)
   {
      /* check if FXO number is set */
      if (0 < pCtrl->nMaxFxoNumber)
      {
         sTableSeparate = "-------------------------------------------------";
         TAPIDEMO_PRINTF(("\n"));
         TAPIDEMO_PRINTF(
            ("FXO channels calling table:\n"));
         TAPIDEMO_PRINTF(
            (" %s\n", sTableSeparate));
         TAPIDEMO_PRINTF(
            ("| Dial |            FXO Device           | FXO ch |\n"));
         TAPIDEMO_PRINTF(("|%s|\n", sTableSeparate));
         /* get next FXO device */
         while (0 < Common_GetNextFxoDevCtrlFd(pCtrl, &pFxoDevice, IFX_FALSE))
         {
            /* check if fxo device was found */
            if (IFX_NULL != pFxoDevice)
            {
               /* for all FXO channels */
               for (nCh=0; nCh < pFxoDevice->nNumOfCh; nCh++)
               {
                  /* get FXO channel structure */
                  pFXO = &pFxoDevice->rgoFXO[nCh];
                  TAPIDEMO_PRINTF(
                     ("|  05%d | %12s No %1d (%11s) |  %3d   |\n",
                      pFXO->nFXO_Number, pFxoDevice->pFxoTypeName,
                      pFxoDevice->nDevNum,
                      FXO_SetFxoNameString(
                         Common_Enum2Name(pFxoDevice->nFxoType, TD_rgFxoDevName),
                         pFxoDevice->nDevNum, TYPE_CHANNEL, pFXO->nFxoCh),
                      pFXO->nFxoCh));
               } /* for all FXO channels */
            } /* if (IFX_NULL != pFxoDevice) */
            else
            {
               TAPIDEMO_PRINTF((
                  "Err, Invalid FXO device pointer.(File: %s line:%d)\n",
                  __FILE__, __LINE__));
               break;
            }
         }
         TAPIDEMO_PRINTF((" %s\n", sTableSeparate));
         TAPIDEMO_PRINTF(("\n"));
      } /* if (0 < pCtrl->nMaxFxoNumber) */
      else
      {
         TAPIDEMO_PRINTF(("No FXO channels.\n"));
      }
   } /* if (oCtrlStatus.pProgramArg->oArgFlags.nFXO) */
#endif /* FXO */
#endif /* TAPI_VERSION4 */

   return IFX_SUCCESS;
} /* TAPIDEMO_Call_Info() */

#ifdef STREAM_1_1
/**
   Clear phone buffer containing DTMF, pulse digits.

   \param nDataCh_FD  - file descriptor for data channel
*/
static IFX_void_t TAPIDEMO_ClearDigitBuff(IFX_int32_t nDataCh_FD)
{
   IFX_int32_t ready = 0;
   IFX_int32_t digit = 0;


   /* Check for DTMF digit */
   /* Check the DTMF dialing status, but on data device not control device */
   do
   {
      ready = 0;
      TD_IOCTL(nDataCh_FD, IFX_TAPI_TONE_DTMF_READY_GET, (IFX_int32_t) &ready);
      if (1 == ready)
      {
         /* Digit arrived or was in buffer */
         TD_IOCTL(nDataCh_FD, IFX_TAPI_TONE_DTMF_GET, (IFX_int32_t) &digit);
      }
      else
      {
         /* No digit in the buffer, buffer is cleared */
         break;
      }
   }
   while (1 == ready);

   /* Check for Pulse digit */
   /* Check the Pulse dialing status, but on data device not control device */
   do
   {
      ready = 0;
      TD_IOCTL(nDataCh_FD, IFX_TAPI_PULSE_READY, (IFX_int32_t) &ready);
      if (1 == ready)
      {
         /* Digit arrived or was in buffer */
         TD_IOCTL(nDataCh_FD, IFX_TAPI_PULSE_GET, (IFX_int32_t) &digit);
      }
      else
      {
         /* No digit in the buffer, buffer is cleared */
         break;
      }
   }
   while (1 == ready);

   TRACE(TAPIDEMO, DBG_LEVEL_LOW,
        ("Buffer with DTMF, pulse digits cleared, flushed\n"));

} /* TAPIDEMO_ClearDigitBuff() */
#endif /* STREAM_1_1 */


/**
   Initialize main structures used for testing

   \param pBoard      - board handle
   \param pCtrl       - pointer to status control structure
   \param nPhoneNum   - number of phone

   \return IFX_SUCCESS if everything OK, otherwise IFX_ERROR
*/
static IFX_return_t TAPIDEMO_Setup(BOARD_t* pBoard,
                                   CTRL_STATUS_t* pCtrl,
                                   IFX_int32_t* nPhoneNum)
{
   IFX_int32_t ch = 0;
#ifndef TAPI_VERSION4
   IFX_TAPI_SIG_DETECTION_t dtmfDetection;
#endif

   /* check input parameters */
   TD_PTR_CHECK(pCtrl, IFX_ERROR);
   TD_PTR_CHECK(pCtrl->pProgramArg, IFX_ERROR);
   TD_PTR_CHECK(pBoard, IFX_ERROR);

   /* Add default ip address and demo arguments */
   pCtrl->oTapidemo_IP_Addr = pCtrl->pProgramArg->oMy_IP_Addr;
   pCtrl->nMy_IP_Addr =
      ntohl((int)pCtrl->pProgramArg->oMy_IP_Addr.sin_addr.s_addr) & 0xffffff00;

   if (pBoard->nMaxCoderCh > 0)
   {
      /* Set codec */
      if (IFX_SUCCESS != VOIP_SetCodec(pCtrl))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH, ("VOIP_SetCodec failed.\n"));
         return IFX_ERROR;
      }

      if (IFX_SUCCESS != VOIP_Init(pCtrl->pProgramArg, pBoard))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH, ("VOIP_Init failed.\n"));
         return IFX_ERROR;
      }
   }

   /* PCM is global and initialized in main() function at the moment */

#ifdef STREAM_1_1
   ABSTRACT_UnmapDefaults(pCtrl, pBoard);
#endif /* STREAM_1_1 */

   /* Do startup mapping of channels */
   ABSTRACT_DefaultMapping(pCtrl, pBoard);

#ifdef TD_DECT
   /* set DECT phones info and configure DECT KPI */
   if (IFX_SUCCESS != TD_DECT_SetPhonesStructureAndKpi(pBoard))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("TD_DECT_SetPhonesStructureAndKpi failed\n"));
      return IFX_ERROR;
   }
#endif /* TD_DECT */
   /* Channels are already open */
   for (ch = 0; ch < pBoard->nMaxPhones; ch++)
   {
      /* reset event fields */
      pBoard->rgoPhones[ch].nIntEvent_FXS = IE_NONE;
      pBoard->rgoPhones[ch].nIntEvent_Config = IE_NONE;
#ifndef TAPI_VERSION4
      pBoard->rgoPhones[ch].nPhoneNumber = *nPhoneNum;
      *nPhoneNum += 1;
#else
      pBoard->rgoPhones[ch].nPhoneNumber =
         (pBoard->rgoPhones[ch].nDev + 1) * 100 +
         pBoard->rgoPhones[ch].nPhoneCh + 1 + *nPhoneNum;
#endif
      if (ch < pBoard->nMaxAnalogCh)
      {
         /* Now setup FDs and sockets, because startup mapping is done. */
         pBoard->rgoPhones[ch].nPhoneCh_FD =
            Common_GetFD_OfCh(pBoard, pBoard->rgoPhones[ch].nPhoneCh);

         if (pBoard->nMaxCoderCh > 0)
         {
            pBoard->rgoPhones[ch].nDataCh_FD =
               Common_GetFD_OfCh(pBoard, pBoard->rgoPhones[ch].nDataCh);
#ifdef TAPI_VERSION4
            if (pBoard->fUseSockets)
#endif
            {
               /* Set sockets */
               pBoard->rgoPhones[ch].nSocket =
                  VOIP_GetSocket(pBoard->rgoPhones[ch].nDataCh, pBoard);
            }
         }
      }

      /* Add also board to phone */
      pBoard->rgoPhones[ch].pBoard = pBoard;

      pBoard->rgoPhones[ch].pCID_Msg = calloc(1, sizeof(IFX_TAPI_CID_MSG_t));

      if (IFX_NULL != pBoard->rgoPhones[ch].pCID_Msg)
      {
         pBoard->rgoPhones[ch].pCID_Msg->message = calloc(TD_CID_ELEM_COUNT,
                                           sizeof(IFX_TAPI_CID_MSG_ELEMENT_t));
      }
      else
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Error on calloc for phone->pCID_Msg"
                "File: %s, line: %d)\n",
                 __FILE__, __LINE__));
      }

      if (pCtrl->pProgramArg->oArgFlags.nCID)
      {
#ifndef TAPI_VERSION4
         CID_SetupData(ch, &pBoard->rgoPhones[ch]);
#else
         CID_SetupData(pBoard->rgoPhones[ch].nPhoneCh, &pBoard->rgoPhones[ch]);
#endif

         /* Shows CID data */
         CID_Display(&pBoard->rgoPhones[ch], pBoard->rgoPhones[ch].pCID_Msg,
                     CID_DISPLAY_SIMPLE);

         /* Configure CID for this channel */
         if (IFX_SUCCESS != CID_ConfDriver(&pBoard->rgoPhones[ch]))
         {
#ifdef TAPI_VERSION4
            if (pBoard->rgoPhones[ch].pBoard->fSingleFD)
            {
               TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                     ("CID initialization failed for device %d, channel %d. "
                      "(File: %s, line: %d)\n",
                      pBoard->rgoPhones[ch].nDev,
                      pBoard->rgoPhones[ch].nPhoneCh, __FILE__, __LINE__));
            }
            else
#endif
            {
               TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                     ("CID initialization failed for channel %d. "
                      "File: %s, line: %d)\n",
                      (int)ch, __FILE__, __LINE__));
            }
            Common_Close_FDs(pBoard);
            return IFX_ERROR;
         }
      }

      if (ch < pBoard->nMaxAnalogCh)
      {
         /* set initial states of state machines */
         pBoard->rgoPhones[ch].rgStateMachine[FXS_SM].nState = S_READY;
      }
#ifdef TD_DECT
      else if (ch < (pBoard->nMaxAnalogCh + pBoard->nMaxDectCh))
      {
         /* set initial states of state machines for DECT phones */
         pBoard->rgoPhones[ch].rgStateMachine[FXS_SM].nState =
            S_DECT_NOT_REGISTERED;
      }
#endif /* TD_DECT */
      else
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Err, %s invalid phone number %d. (File: %s, line: %d)\n",
                pBoard->pszBoardName, ch, __FILE__, __LINE__));
      }
      pBoard->rgoPhones[ch].rgStateMachine[FXS_SM].pStateCombination = ST_rgFXSStates;
      pBoard->rgoPhones[ch].nCallDirection = CALL_DIRECTION_NONE;
      pBoard->rgoPhones[ch].rgStateMachine[CONFIG_SM].nState = S_READY;
      pBoard->rgoPhones[ch].rgStateMachine[CONFIG_SM].pStateCombination = ST_rgConfigStates;
   } /* for */

#ifdef TAPI_VERSION4
   if (pBoard->nMaxAnalogCh != 0 && &pBoard->rgoPhones[pBoard->nMaxAnalogCh-1]
      != IFX_NULL)
      *nPhoneNum = (pBoard->rgoPhones[pBoard->nMaxAnalogCh-1].nDev + 1) * 100;
#endif

#ifdef EASY336
   if (Common_PCM_ConfigSet(pBoard, &pcm_cfg) != IFX_SUCCESS)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("PCM configuration set failed "
            "(File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
#endif

#ifndef TAPI_VERSION4
   memset(&dtmfDetection, 0, sizeof (IFX_TAPI_SIG_DETECTION_t));
   /* Stop detection of DTMF tones from local interface */
   dtmfDetection.sig = IFX_TAPI_SIG_DTMFTX;
   if (0 < pBoard->nMaxCoderCh)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_LOW,
            ("%s: Disable DTMF detection on all data channels \n",
             pBoard->pszBoardName));
   }
   for (ch = 0; ch < pBoard->nMaxCoderCh; ch++)
   {
      if (IFX_SUCCESS != TD_IOCTL(pBoard->nCh_FD[ch],
                                  IFX_TAPI_SIG_DETECT_DISABLE,
                                  (IFX_int32_t) &dtmfDetection))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err: IFX_TAPI_SIG_DETECT_DISABLE failed (File: %s, line: %d)\n",
            __FILE__, __LINE__));
         return IFX_ERROR;
      }
   }
#endif /* TAPI_VERSION4 */
   return IFX_SUCCESS;
} /* TAPIDEMO_Setup() */

/**
   Set file descriptors set for select on device.

   \param pCtrl   - pointer to control structure
   \param pRdFds  - pointer to file descriptors set

   \return IFX_SUCCESS if everything OK, otherwise IFX_ERROR
*/
static IFX_return_t TAPIDEMO_SetDeviceFdSet(CTRL_STATUS_t* pCtrl, 
                                            TD_OS_socFd_set_t* pRdFds)
{
   IFX_int32_t i, j;
   BOARD_t* pBoard;
   IFX_int32_t fd_ch, fd_dev_ctrl;
#ifdef FXO
   TAPIDEMO_DEVICE_FXO_t* pFxoDev = IFX_NULL;
#endif /* FXO */

   /* check input parameters */
   TD_PTR_CHECK(pCtrl, IFX_ERROR);
   TD_PTR_CHECK(pRdFds, IFX_ERROR);

   /* reset structure */
   TD_OS_DevFdZero(pRdFds);

   /* for all boards */
   for (j = 0; j < pCtrl->nBoardCnt; j++)
   {
      /* get board */
      pBoard = &pCtrl->rgoBoards[j];

      /* Add all board file descriptor */
      fd_dev_ctrl = pBoard->nDevCtrl_FD;
      if (0 < fd_dev_ctrl)
      {
         TD_FD_SET(fd_dev_ctrl, pRdFds, pCtrl->nMaxFdDevice, TD_SELECT_DEVICE);
      }
      else
      {
         TAPIDEMO_DEVICE_t  *pDevice = IFX_NULL;
         /* some boards could not have control device file descriptor. */
         if ((pDevice = Common_GetDevice_CPU(pBoard->pDevice)) != IFX_NULL)
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                  ("Error, Control device not open, %s. (File: %s, line: %d)\n",
                   pBoard->pszBoardName, __FILE__, __LINE__));
            return IFX_ERROR;
         }
      }

      /* set fds responsible for communication/calls that use data channels
         e.g. voip connections */
#ifdef TAPI_VERSION4
      if (!pBoard->fSingleFD)
#endif
      {
         /* Set up file descriptor table without sockets */
         for (i = 0; i < pBoard->nMaxCoderCh; i++)
         {
            fd_ch = Common_GetFD_OfCh(pBoard, i);
            if (fd_ch > 0)
            {
#ifdef STREAM_1_1
               /* Clear DTMF buffer for each phone channel */
               TAPIDEMO_ClearDigitBuff(fd_ch);
#endif /* STREAM_1_1 */
               /* Put file descriptors into the read set */
               TD_FD_SET(fd_ch, pRdFds, pCtrl->nMaxFdDevice, TD_SELECT_DEVICE);
            } /* if (fd_ch > 0) */
            else
            {
               TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                     ("Error, File descriptor for channel %d not open. "
                      "(File: %s, line: %d)\n",
                      (int) i, __FILE__, __LINE__));
               return IFX_ERROR;
            }
         } /* for - go through all coder channels */
      } /* if (!pBoard->fSingleFD */
   } /* for - go through all boards */
#ifdef FXO
   /* add FXO device control fd */
   if (pCtrl->pProgramArg->oArgFlags.nFXO)
   {
      if (0 < pCtrl->nMaxFxoNumber)
      {
         do
         {
            fd_dev_ctrl = Common_GetNextFxoDevCtrlFd(pCtrl, &pFxoDev,
                                                     IFX_TRUE);
            /* Add config fd to fd set */
            if (0 < fd_dev_ctrl)
            {
                  TD_FD_SET(fd_dev_ctrl, pRdFds, pCtrl->nMaxFdDevice, 
                            TD_SELECT_DEVICE);
            }
         } while (0 < fd_dev_ctrl);
         /* check return value, see Common_GetNextFxoDevCtrlFd description
            for details */
         if ((IFX_ERROR == fd_dev_ctrl) && (IFX_NULL == pFxoDev))
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                  ("Err, Number of FXOs %d but no FXO device FD was found."
                   " (File: %s, line: %d)\n",
                   pCtrl->nMaxFxoNumber, __FILE__, __LINE__));
         }
         else if (IFX_ERROR == fd_dev_ctrl)
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                  ("Err, Geting FXO device FD failed. (File: %s, line: %d)\n",
                   __FILE__, __LINE__));
         }
      }
   } /* if FXO */
#endif /* FXO */

#ifdef TD_DECT
   if (!pCtrl->pProgramArg->oArgFlags.nNoDect)
   {
      /* check and add timer FD */
      if (0 < pCtrl->nTimerFd)
      {
         TD_FD_SET(pCtrl->nTimerFd, pRdFds, pCtrl->nMaxFdDevice, 
                   TD_SELECT_DEVICE);
      }
      else
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Err, Invalid pCtrl->nTimerFd = %d\n(File: %s, line: %d)\n",
                pCtrl->nTimerFd, __FILE__, __LINE__));
         return IFX_ERROR;
      }
      /* check and add paging key FD */
      if (0 < pCtrl->viPagingKeyFd)
      {
         TD_FD_SET(pCtrl->viPagingKeyFd, pRdFds, pCtrl->nMaxFdDevice, 
                   TD_SELECT_DEVICE);
      }
      else
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Err, Invalid pCtrl->viPagingKeyFd = %d\n(File: %s, line: %d)\n",
                pCtrl->viPagingKeyFd, __FILE__, __LINE__));
         return IFX_ERROR;
      }
   } /* if (!pCtrl->pProgramArg->oArgFlags.nNoDect) */
#endif /* TD_DECT */

   return IFX_SUCCESS;
} /* TAPIDEMO_SetDeviceFdSet */

/**
   Set file descriptors set for select on sockets and pipes.

   \param pCtrl   - pointer to control structure
   \param pRdFds  - pointer to file descriptors set

   \return IFX_SUCCESS if everything OK, otherwise IFX_ERROR
*/
static IFX_return_t TAPIDEMO_SetSocketFdSet(CTRL_STATUS_t* pCtrl, 
                                            TD_OS_socFd_set_t* pRdFds)
{
   IFX_int32_t i, j;
   BOARD_t* pBoard;
   IFX_int32_t nSocket;

   /* check input parameters */
   TD_PTR_CHECK(pCtrl, IFX_ERROR);
   TD_PTR_CHECK(pRdFds, IFX_ERROR);

   /* reset structure */
   TD_OS_DevFdZero(pRdFds);

   /* for all boards */
   for (j = 0; j < pCtrl->nBoardCnt; j++)
   {
      /* get board */
      pBoard = &pCtrl->rgoBoards[j];

#ifdef TAPI_VERSION4
      if (!pBoard->fSingleFD)
#endif
      {
         /* Set up file descriptor table without sockets */
         for (i = 0; i < pBoard->nMaxCoderCh; i++)
         {
#ifdef TAPI_VERSION4
            if (pBoard->fUseSockets)
#else /* TAPI_VERSION4 */
            if (!pCtrl->pProgramArg->oArgFlags.nQos)
#endif /* TAPI_VERSION4 */
            {
               /* Set possible sockets */
               nSocket = VOIP_GetSocket(i, pBoard);
               if (NO_SOCKET != nSocket)
               {
                  TD_FD_SET(nSocket, pRdFds, pCtrl->nMaxFdSocket, 
                            TD_SELECT_SOCKET);
               }
            } /* if !QoS */
         } /* for - go through all coder channels */
      } /* if (!pBoard->fSingleFD */
   } /* for - go through all boards */

   /* Also add administration socket for handling calls */
   if (pCtrl->nAdminSocket == NO_SOCKET)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error, Couldn't get administration socket. "
             "(File: %s, line: %d)\n",
             __FILE__, __LINE__));
      /* Without this socket we cannot make calls. */
      return IFX_ERROR;
   }
   TD_FD_SET(pCtrl->nAdminSocket, pRdFds, pCtrl->nMaxFdSocket, TD_SELECT_SOCKET);

   if (pCtrl->oMultithreadCtrl.oPipe.oSocketSynchPipe.rgPipeFd[TD_PIPE_OUT] 
       == NO_SOCKET)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error, Couldn't get thread synch pipe.\n"
             "(File: %s, line: %d)\n",
             __FILE__, __LINE__));
      return IFX_ERROR;
   }
   TD_FD_SET(pCtrl->oMultithreadCtrl.oPipe.oSocketSynchPipe.rgPipeFd[TD_PIPE_OUT],
             pRdFds, pCtrl->nMaxFdSocket, TD_SELECT_SOCKET);

   /* Add communication socket for handling PC's requests (used for ITM) */
   COM_SOCKET_FD_SET(pRdFds, pCtrl->nMaxFdSocket);

#ifdef TD_FAX_MODEM
#if (defined(TD_T38_FAX_TEST) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW))
   /* Add communication socket for handling PC's requests */
   if(NO_SOCKET != pCtrl->oFaxTestCtrl.nComSocket)
   {
      /* check if using T.38 fax test, if not then leave function */
      TD_T38_FAX_TEST_CHECK_IF_WORKING(&pCtrl->oFaxTestCtrl);
      /* add socket */
      TD_FD_SET(pCtrl->oFaxTestCtrl.nComSocket, pRdFds, pCtrl->nMaxFdSocket, 
                TD_SELECT_SOCKET);
   }
#endif /* (defined(TD_T38_FAX_TEST) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW)) */
#endif /* TD_FAX_MODEM */


#ifdef TD_DECT
   if (!pCtrl->pProgramArg->oArgFlags.nNoDect)
   {
      /* add DECT communication pipe if available */
      if ((IFX_TRUE == pCtrl->oDectStackCtrl.nDectInitialized) ||
          (0 < pCtrl->nDectPipeFdOut))
      {
         TD_FD_SET(pCtrl->nDectPipeFdOut, pRdFds, pCtrl->nMaxFdSocket, 
                   TD_SELECT_SOCKET);
      }
      else
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Err, Unable to add DECT pipe FD (%d, %d)\n(File: %s, line: %d)\n",
                pCtrl->oDectStackCtrl.nDectInitialized,
                pCtrl->nDectPipeFdOut,
                __FILE__, __LINE__));
         return IFX_ERROR;
      }

      /* check and add CLI FD */
      if (0 < pCtrl->viFromCliFd)
      {
         TD_FD_SET(pCtrl->viFromCliFd, pRdFds, pCtrl->nMaxFdSocket, 
                   TD_SELECT_SOCKET);
      }
      else
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Err, Invalid pCtrl->viFromCliFd = %d\n(File: %s, line: %d)\n",
                pCtrl->viFromCliFd, __FILE__, __LINE__));
         return IFX_ERROR;
      }   } /* if (!pCtrl->pProgramArg->oArgFlags.nNoDect) */
#endif /* TD_DECT */

   return IFX_SUCCESS;
} /* TAPIDEMO_SetSocketFdSet */

/**
   Clear phone after connection.

   \param  pCtrl  - pointer to status control structure
   \param  pPhone - pointer to PHONE
   \param  pConn  - pointer to phone connection
   \param  nInternalEvent - new event information
   \param  pBoard - board handle

   \return Phone is cleared.
*/
IFX_void_t TAPIDEMO_ClearPhone(CTRL_STATUS_t* pCtrl, PHONE_t* pPhone,
                               CONNECTION_t* pConn,
                               INTERNAL_EVENTS_t nInternalEvent,
                               BOARD_t* pBoard)
{
   IFX_int32_t fd_data_ch = -1;

   /* check input parameters */
   TD_PTR_CHECK_VOID(pCtrl);
   TD_PTR_CHECK_VOID(pPhone);
   TD_PTR_CHECK_VOID(pBoard);

   TRACE(TAPIDEMO, DBG_LEVEL_LOW,
         ("Phone No %d: TAPIDEMO_ClearPhone.\n",
          pPhone->nPhoneNumber));

   /* set connection pointer for FXO_CALL*/
   if ((pPhone->fFXO_Call == IFX_TRUE) ||
       /* if pointer not set and connection in progress */
       ((pPhone->nConnCnt == 1) && (pConn == IFX_NULL)))
   {
      pConn = &pPhone->rgoConn[0];
   }
   /************************
    TURN OFF SPECIAL FEATURES
    ************************/
   /* VAD works only for Data channels */
   if (0 < pBoard->nMaxCoderCh)
   {
      Common_SetVad(pPhone, pBoard, RESTORE_DEFAULT);
   }
   /* if wideband was set then change back narrowband
      if AGC was enabled then disable AGC
      AGC must be disabled before disabling encoder */
   if ((pBoard != IFX_NULL) && (pConn != IFX_NULL))
   {
#ifdef TD_DECT
      if (PHONE_TYPE_DECT == pPhone->nType)
      {
         /* do nothing here for DECT phones */
      }
      else
#endif /** TD_DECT */
      /* Set back to default features */
      if ((pPhone->fWideBand_Cfg) || (pPhone->fAGC_Cfg))
      {
         /* Set line type, coders, ... to some default values */
         if (pBoard->nMaxCoderCh > 0)
         {
            fd_data_ch = VOIP_GetFD_OfCh(pConn->nUsedCh, pBoard);
         }
         else if (pBoard->nMaxAnalogCh > 0)
         {
            fd_data_ch = Common_GetFD_OfCh(pBoard, pConn->nUsedCh);
         }
         else
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                  ("Board %s without coders and analog channels. "
                   "(File: %s, line: %d)\n",
                   pBoard->pszBoardName, __FILE__, __LINE__));
         }
         /* change back to NarrowBand if WideBand was enabled */
         if (pPhone->fWideBand_Cfg)
         {
            Common_ConfigureTAPI(pCtrl, pBoard, pPhone, 2000,
                                 fd_data_ch, IFX_TRUE, IFX_FALSE);
            pPhone->fWideBand_Cfg = IFX_FALSE;
         }
         /* disable AGC if enabled */
         if (pPhone->fAGC_Cfg)
         {
            Common_ConfigureTAPI(pCtrl, pBoard, pPhone, 2010,
                                 fd_data_ch, IFX_TRUE, IFX_FALSE);
            pPhone->fAGC_Cfg = IFX_FALSE;
         }
      } /* if ((pPhone->fWideBand_Cfg) || (pPhone->fAGC_Cfg)) */

   } /* if ((pBoard != IFX_NULL) && (pConn != IFX_NULL)) */

   /************************
      END CONNECTION
    ************************/
   if (IFX_NULL != pConn)
   {
      if (UNKNOWN_CALL_TYPE == pConn->nType)
      {
         pConn = &pPhone->rgoConn[0];
      }
/* CONFERENCE CONFERENCE CONFERENCE CONFERENCE CONFERENCE CONFERENCE */
      /* Conference clean up*/
      if (pPhone->nConfStarter)
      {
         if (IFX_ERROR == CONFERENCE_End(pCtrl, pPhone,
                             &pCtrl->rgoConferences[pPhone->nConfIdx - 1]))
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                  ("Failed to end conference properly. Could cause "
                   "mapping related problems. (File: %s, line: %d)\n",
                   __FILE__, __LINE__));
         }
      }
#if !defined(STREAM_1_1) && !defined(VXWORKS) && defined(FXO)
/* FXO CALL FXO CALL FXO CALL FXO CALL FXO CALL FXO CALL FXO CALL FXO CALL */
      else if (FXO_CALL == pConn->nType)
      {
         if (pCtrl->pProgramArg->oArgFlags.nFXO)
         {
            if (pPhone->fFXO_Call)
            {
               /* stop FXO call */
               if (pPhone->pFXO == IFX_NULL)
               {
                  /* should not happend*/
                  TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                        ("Error, PHONE_t:pFXO is NULL. (File: %s, line: %d)\n",
                         __FILE__, __LINE__));
               }
               else if ((pPhone->pFXO->nState == S_ACTIVE) ||
                        (pPhone->pFXO->nState == S_IN_RINGING))
               {
                  FXO_EndConnection(pCtrl->pProgramArg, pPhone->pFXO,
                                    pPhone, pConn, pBoard);
               }
               else
               {
                  TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                        ("Wrong FXO state. (File: %s, line: %d)\n",
                         __FILE__, __LINE__));
               }
            }
            else
            {
               TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                     ("pPhone->fFXO_Call is not set. (File: %s, line: %d)\n",
                      __FILE__, __LINE__));
            }
         }
         else
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                  ("FXO calls are not enbled. (File: %s, line: %d)\n",
                   __FILE__, __LINE__));
         }
      }
#endif /* !defined(STREAM_1_1) && !defined(VXWORKS) && defined(FXO) */
/* PCM CALLS  PCM CALLS PCM CALLS PCM CALLS PCM CALLS PCM CALLS PCM CALLS */
#ifndef EASY336
      else if (PCM_CALL == pConn->nType ||
               LOCAL_PCM_CALL == pConn->nType ||
               LOCAL_BOARD_PCM_CALL == pConn->nType)
      {
         TAPIDEMO_PORT_DATA_t oPortData;
         oPortData.nType = PORT_FXS;
         oPortData.uPort.pPhopne = pPhone;
         if (PCM_EndConnection(&oPortData, pCtrl->pProgramArg, pBoard, pConn))
         {
#if (!defined(DXT) && !defined(XT16))
#ifdef TD_DECT
            /* DECT phone is unmapped from PCM at end of this function
               if going on-hook */
            if (PHONE_TYPE_DECT != pPhone->nType)
#endif /* TD_DECT */
            {
               /* Unmap phone and pcm channels */
               PCM_MapToPhone(pConn->nUsedCh,
                              pPhone->nPhoneCh,
                              IFX_FALSE, pBoard);
            }
#endif /* !DXT && !XT16 */
         }
      }
#endif /* EASY336 */
      else
      {
/* CALL THAT USES CODECS CALL THAT USES CODECS CALL THAT USES CODECS */
         /* Stop coder if voip call or local call that uses codec*/
         if ((EXTERN_VOIP_CALL == pConn->nType) ||
             (LOCAL_BOARD_CALL == pConn->nType) ||
             ((LOCAL_CALL == pConn->nType) &&
              (pCtrl->pProgramArg->oArgFlags.nUseCodersForLocal)))
         {
            if (IFX_TRUE == pConn->bExtDialingIn)
            {
               /* Clean external connection */
               pConn->bExtDialingIn = IFX_FALSE;
               TD_TIMEOUT_STOP(pCtrl->nUseTimeoutSocket);
            }
            /* codec is started when connection is active */
            if (IFX_TRUE == pConn->fActive)
            {
#ifndef TAPI_VERSION4
               VOIP_StopCodec(pConn->nUsedCh, pConn, pBoard);
#else
#ifdef EASY336
               TRACE(TAPIDEMO, DBG_LEVEL_LOW,
                     ("pPhone->nVoipIDs: %d\n", pPhone->nVoipIDs));
               /* if conference then codec is stopped in CONFERENCE_End() */
               if ((0 < pPhone->nVoipIDs) &&
                   (IFX_FALSE == pPhone->nConfStarter))
               {
#if defined(TD_FAX_MODEM) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW)
                  if (TD_FAX_MODE_FDP != pCtrl->nFaxMode)
#endif /* defined(TD_FAX_MODEM) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW) */
                  {
                     if (pBoard->fSingleFD)
                     {
                        /* check if vad settings were changed */
                        Common_SetVad(pPhone, pBoard, RESTORE_DEFAULT);
                     }
                     VOIP_StopCodec(pConn->nUsedCh, pConn,
                        ABSTRACT_GetBoard(pCtrl, IFX_TAPI_DEV_TYPE_VIN_SUPERVIP));
                  }
#if  defined(TD_FAX_MODEM) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW)
                  else
                  {
                     VOIP_FaxStop(pCtrl, pConn,
                        ABSTRACT_GetBoard(pCtrl, IFX_TAPI_DEV_TYPE_VIN_SUPERVIP));
                  }
#endif /* defined(TD_FAX_MODEM) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW) */
               } /* if (0 < pPhone->nVoipIDs) */
#endif /* EASY336 */
#endif /* TAPI_VERSION4 */
#ifdef QOS_SUPPORT
               if (pCtrl->pProgramArg->oArgFlags.nQos)
               {
/* no QoS for SVIP */
#ifndef EASY336
                  /* QoS Support */
                  QOS_StopSession(pConn, pPhone);
#endif /* EASY336 */
               }
#endif /* QOS_SUPPORT */
/* TAPI SIGNAL SUPPORT TAPI SIGNAL SUPPORT TAPI SIGNAL SUPPORT */
               /* fax modem only avaible for voip calls */
               if (EXTERN_VOIP_CALL == pConn->nType)
               {
                  /* reset signal handlers */
                  SIGNAL_HandlerReset(pPhone, pConn);
               }
            } /* if (IFX_TRUE == pConn->fActive) */

         } /* connection that uses codecs */
#ifndef EASY336
/* LOCAL CALL LOCAL CALL LOCAL CALL LOCAL CALL LOCAL CALL LOCAL CALL */
         else if (LOCAL_CALL == pConn->nType)
         {
            /* Unmap only on caller side, because its sufficient to map it also
               on caller side. */
            if (pPhone->fLocalPeerCalled)
            {
               /* Unmap phone channels */
               Common_MapLocalPhones(pPhone, pConn->oConnPeer.oLocal.pPhone,
                                     TD_MAP_REMOVE);
            }
         }  /* if ((LOCAL_CALL != pConn->nType) */
#endif
      }
#ifdef EASY336
      pConn->voipID = SVIP_RM_UNUSED;
#endif /* EASY336 */
      /* connection is no longer active */
      pConn->fActive = IFX_FALSE;
   } /* if */
#ifdef TD_DECT
   if (PHONE_TYPE_DECT == pPhone->nType)
   {
      if (TD_NOT_SET != pPhone->nDataCh)
      {
         TD_DECT_DataChannelRemove(pPhone);
      }
      if (TD_NOT_SET != pPhone->nPCM_Ch)
      {
         TD_DECT_PCM_ChannelRemove(pPhone);
      }
      if (IE_READY == nInternalEvent)
      {
         /* disconnect with DECT handset - "on-hook" on DECT handset */
         TD_DECT_HookOn(pPhone);
      }
   }
#endif /* TD_DECT */
   /************************
      RESET PHONE STRUCTURE
    ************************/
   if (nInternalEvent != IE_BUSY)
   {
      pPhone->nCallDirection = CALL_DIRECTION_NONE;
   }
   /* Here we should also clear connections. */
   pPhone->nDialNrCnt = 0;
   pPhone->fExtPeerCalled = IFX_FALSE;
   pPhone->fPCM_PeerCalled = IFX_FALSE;
   pPhone->fLocalPeerCalled = IFX_FALSE;
   pPhone->nConnCnt = 0;
   pPhone->fConfTAPI_InConn = IFX_FALSE;
   pPhone->fFXO_Call = IFX_FALSE;

   /************************
   TURN OFF UNNEEDED RESOURCES
    ************************/
   /* phone goes on hook - turn off unneeded resources and change line feed */
   if (nInternalEvent == IE_READY)
   {
#ifdef TAPI_VERSION4
      if (!pBoard->fSingleFD)
#endif /* TAPI_VERSION4 */
      {
#ifndef TAPI_VERSION4
         /* disable the DTMF detection */
         Common_DTMF_SIG_Disable(pCtrl, pPhone);
         /* Disable the LEC*/
         Common_LEC_Disable(pCtrl, pPhone);
#endif /* TAPI_VERSION4 */
      }
      /* Set line in standby */
      if (IFX_ERROR == Common_LineFeedSet(pPhone, IFX_TAPI_LINE_FEED_STANDBY))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Err, line feed set failed for ch %d, %s."
                "(File: %s, line: %d)\n",
                pPhone->nPhoneCh, pBoard->pszBoardName, __FILE__, __LINE__));
      } /* if (IFX_ERROR == Common_LineFeedSet()) */
   } /* if (nInternalEvent == IE_READY) */

} /* TAPIDEMO_ClearPhone() */

/**
   SetAction    : set action of the phone we are talking to.

   \param pCtrl   - pointer to status control structure
   \param pPhone  - pointer to PHONE
   \param pConn   - pointer to phone connection
   \param nAction - which action to send to other phone

   \return       IFX_SUCCESS - no error, otherwise IFX_ERROR
*/
IFX_return_t TAPIDEMO_SetAction(CTRL_STATUS_t* pCtrl,
                                PHONE_t* pPhone,
                                CONNECTION_t* pConn,
                                IFX_int32_t nAction)
{
   PHONE_t* p_dst_phone = IFX_NULL;
   IFX_int32_t ret = IFX_SUCCESS;
   TD_OS_sockAddr_t to_addr;
   TD_COMM_MSG_t msg = {0};
   CONNECTION_t* p_conn = IFX_NULL;
   IFX_char_t* psActionName = IFX_NULL;

   /* check input parameter */
   TD_PTR_CHECK(pCtrl, IFX_ERROR);
   TD_PTR_CHECK(pPhone, IFX_ERROR);
   TD_PTR_CHECK(pConn, IFX_ERROR);

   /* set pointer string with action name */
   psActionName = Common_Enum2Name(nAction, TD_rgIE_EventsName);
   /* check if name was set, if no name is set,
      then invalid value of TD_rgIE_EventsName is set */
   TD_PTR_CHECK(psActionName, IFX_ERROR);

   /* prepare message end and start */
   msg.nMarkStart = COMM_MSG_START_FLAG;
   msg.nMarkEnd = COMM_MSG_END_FLAG;
   /* set msg size and version */
   msg.nMsgLength = sizeof(msg);
   msg.nMsgVersion = TD_COMM_MSG_CURRENT_VERSION;
   /* set call type */
   if (EXTERN_VOIP_CALL == pConn->nType)
   {
      msg.fPCM = CALL_FLAG_VOIP;
   }
   else if (PCM_CALL == pConn->nType)
   {
      msg.fPCM = CALL_FLAG_PCM;
   }
   else
   {
      msg.fPCM = CALL_FLAG_UNKNOWN;
   }
   /* Set action to external phone, for example tell him that
      we are calling */
   msg.nAction = nAction;
   /* set feature ID */
   msg.nFeatureID = pConn->nFeatureID;

   /* For this two actions no phone and connection are selected,
      pPhone and pConn are phony only for message send use. */
   if ((nAction == IE_EXT_CALL_PROCEEDING) ||
       (nAction == IE_EXT_CALL_WRONG_NUM))
   {
      /* Action send to external phone, that tried to call phone on this board,
         tell him that phone number is incomplete or wrong */

      /* called phone info */
      msg.nBoard_IDX = NO_BOARD_IDX;
      msg.nSenderPhoneNum = pPhone->nPhoneNumber;
      /* no real connection set port number to 0*/
      msg.nSenderPort = 0;
      /* copy board MAC address */
      memcpy(msg.MAC, pConn->oUsedMAC, ETH_ALEN);

      /* caller phone info */
      msg.nReceiverPhoneNum = pConn->oConnPeer.nPhoneNum;
      /* set peer address */
      memset(&to_addr, 0, sizeof(TD_OS_sockAddr_t));
      to_addr.sin_addr = pConn->oConnPeer.oRemote.oToAddr.sin_addr;
      to_addr.sin_family = pConn->oConnPeer.oRemote.oToAddr.sin_family;
      to_addr.sin_port = htons(ADMIN_UDP_PORT);

      TRACE(TAPIDEMO, DBG_LEVEL_LOW,
            ("Sending MSG using socket %d to %s:%d:\n"
             " - call type %s\n"
             " - sender MAC %02X:%02X:%02X:%02X:%02X:%02X\n"
             " - sender phone number %d\n"
             " - receiver phone number %d\n"
             " - new internal event %s\n"
             " - feature id %d\n",
             (int) pCtrl->nAdminSocket,
             inet_ntoa(to_addr.sin_addr), ntohs(to_addr.sin_port),
             (CALL_FLAG_PCM == msg.fPCM)? "PCM" :
             (CALL_FLAG_VOIP == msg.fPCM) ? "EXTERN" : "UNKNOWN",
             msg.MAC[0],
             msg.MAC[1],
             msg.MAC[2],
             msg.MAC[3],
             msg.MAC[4],
             msg.MAC[5],
             msg.nSenderPhoneNum, msg.nReceiverPhoneNum,
             psActionName, msg.nFeatureID));

      ret = TD_OS_SocketSendTo(pCtrl->nAdminSocket, (IFX_char_t *) &msg, 
                               sizeof(msg), &to_addr);
      if (sizeof(msg) != ret)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Error sending data (command %d byte(s)), %s. "
               "(File: %s, line: %d)\n",
               sizeof(msg), strerror(errno), __FILE__, __LINE__));
         return IFX_ERROR;
      }
   }
   else if (nAction == IE_EXT_CALL_WORKAROUND_EASY508xx)
   {
      /* It is workaround, which avoid problem with first external/PCM connection
      to EASY508xx.

      We need to send any packet to external board ESY508xx before starting
      to establish external connection with it. If we don't do it, first packet
      during first external connection will be not detected.
      In this point, we neither know with what type of board we make connection
      nor if it is first connection, so we have to send this packet before every
      exernal connection. This packet will be just ignore by TAPIDEMO
      on secont board  */

      memset(&to_addr, 0, sizeof(TD_OS_sockAddr_t));

      /* set call type */
      if (EXTERN_VOIP_CALL == pConn->nType)
      {
         to_addr.sin_addr = pConn->oConnPeer.oRemote.oToAddr.sin_addr;
         to_addr.sin_family = pConn->oConnPeer.oRemote.oToAddr.sin_family;
         to_addr.sin_port = htons(ADMIN_UDP_PORT);
      }
      else if (PCM_CALL == pConn->nType)
      {
         to_addr.sin_addr = pConn->oConnPeer.oPCM.oToAddr.sin_addr;
         to_addr.sin_family = pConn->oConnPeer.oPCM.oToAddr.sin_family;
         to_addr.sin_port = htons(ADMIN_UDP_PORT);
      }
      /* reset feature ID */
      msg.nFeatureID = 0;

      /* called phone info */
      msg.nBoard_IDX = NO_BOARD_IDX;
      msg.nSenderPhoneNum = 0;
      /* no real connection set port number to 0*/
      msg.nSenderPort = 0;
      /* copy board MAC address */
      memcpy(msg.MAC, pConn->oUsedMAC, ETH_ALEN);

      /* caller phone info */
      msg.nReceiverPhoneNum = 0;

      ret = TD_OS_SocketSendTo(pCtrl->nAdminSocket, (IFX_char_t *) &msg, 
                               sizeof(msg),&to_addr);
      if (sizeof(msg) != ret)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Error sending data (command %d byte(s)), %s. "
               "(File: %s, line: %d)\n",
               sizeof(msg), strerror(errno), __FILE__, __LINE__));
         return IFX_ERROR;
      }
   } /* else if (nAction == IE_EXT_CALL_WORKAROUND_EASY508xx) */
   else
   {
#ifndef TAPI_VERSION4
      TRACE(TAPIDEMO, DBG_LEVEL_LOW,
            ("Phone No %d: SetAction: ConnType %s, Action %s\n"
             "%s(PCM Ch %d, DataCh %d, PhoneCh %d)\n",
             (int) pPhone->nPhoneNumber,
             Common_Enum2Name(pConn->nType, TD_rgCallTypeName), psActionName,
             pPhone->pBoard->pIndentPhone,
             (int) pPhone->nPCM_Ch, (int) pPhone->nDataCh,(int) pPhone->nPhoneCh));
#else /* TAPI_VERSION4 */
      TRACE(TAPIDEMO, DBG_LEVEL_LOW,
            ("Phone No %d: %s device %d, PhoneCh %d,\n"
             "%sSetAction: ConnType %s, Action %s\n",
             (int) pPhone->nPhoneNumber,
             pPhone->pBoard->pszBoardName, pPhone->nDev, pPhone->nPhoneCh,
             pPhone->pBoard->pIndentPhone,
             Common_Enum2Name(pConn->nType, TD_rgCallTypeName), psActionName));
#endif /* TAPI_VERSION4 */
      /* check event type */
      if (IE_RINGING == nAction &&
          ((PCM_CALL == pConn->nType) || (EXTERN_VOIP_CALL == pConn->nType)))
      {
         /* check if CID support is on */
         if (pCtrl->pProgramArg->oArgFlags.nCID)
         {
            /* get CID data */
            if (IFX_SUCCESS != CID_FromPeerMsgSet(pPhone, &msg))
            {
               TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                     ("Err, CID_FromPeerMsgGet failed. (File: %s, line: %d)\n",
                      __FILE__, __LINE__));
            }
         } /* if (pCtrl->pProgramArg->oArgFlags.nCID) */
      } /* if (IE_RINGING == event) */

      if ((LOCAL_CALL == pConn->nType) || (LOCAL_BOARD_CALL == pConn->nType)||
          (LOCAL_PCM_CALL == pConn->nType) ||
          (LOCAL_BOARD_PCM_CALL == pConn->nType))
      {
         /* Set action to local phone, for example tell him that
            we are calling */
         p_dst_phone = pConn->oConnPeer.oLocal.pPhone;

         if (IFX_NULL == p_dst_phone)
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                  ("Pointer to destination phone is NULL, target phone is missing!"
                   " (File: %s, line: %d)\n",
                   __FILE__, __LINE__));
            return IFX_ERROR;
         }
#ifdef TD_DECT
         /* print info for DECT phone */
         if (PHONE_TYPE_DECT == p_dst_phone->nType)
         {
            TRACE(TAPIDEMO, DBG_LEVEL_LOW,
                  ("SetAction on destination Phone No %d: DectCh %d\n",
                   p_dst_phone->nPhoneNumber, p_dst_phone->nDectCh));
         }
         else
#endif /* TD_DECT */
         {
#ifndef TAPI_VERSION4
            TRACE(TAPIDEMO, DBG_LEVEL_LOW,
                  ("SetAction on destination PhoneNo %d: DataCh %d, PhoneCh %d\n",
                   (IFX_int32_t)p_dst_phone->nPhoneNumber,
                   (int) p_dst_phone->nDataCh, (int) p_dst_phone->nPhoneCh));
#else /* TAPI_VERSION4 */
            TRACE(TAPIDEMO, DBG_LEVEL_LOW,
                  ("SetAction on destination PhoneNo %d: %s device %d, ch %d\n",
                   (IFX_int32_t)p_dst_phone->nPhoneNumber,
                   p_dst_phone->pBoard->pszBoardName,
                   p_dst_phone->nDev, p_dst_phone->nPhoneCh));
#endif /* TAPI_VERSION4 */
         }

         /* More phones can call us */
         /* Status from where we got command. */
         /* Called phone has always only one connection with caller phone */
         p_dst_phone->nIntEvent_FXS = nAction;
         /* set internal event flag */
         pCtrl->bInternalEventGenerated = IFX_TRUE;
         p_conn = ABSTRACT_GetConnOfLocal(p_dst_phone, pPhone, pConn->nType);
         if (IFX_NULL == p_conn)
         {
            /* Could not get phone connection */
            return IFX_ERROR;
         }
         /* internal event is generated on phone */
         p_dst_phone->pConnFromLocal = p_conn;

         /* set connection data only on first messsage */
         if (IE_RINGING == nAction)
         {
            p_conn->nType = pConn->nType;
            p_conn->oConnPeer.nPhoneNum = pPhone->nPhoneNumber;
            p_conn->oConnPeer.oLocal.pPhone = pPhone;
            p_conn->nFeatureID = pConn->nFeatureID;
#ifdef TD_DECT
            /* for now set narrowband */
            p_conn->oConnPeer.oLocal.fDectWideband = IFX_FALSE;
#endif /* TD_DECT */
            if ((LOCAL_PCM_CALL == pConn->nType) ||
                (LOCAL_BOARD_PCM_CALL == pConn->nType))
            {
               /* Caller phone is sending action */
               p_conn->oConnPeer.oLocal.fSlave = IFX_TRUE;
               /* cross timeslots numbers */
               p_conn->oPCM.nTimeslot_RX = pConn->oPCM.nTimeslot_TX;
               p_conn->oPCM.nTimeslot_TX = pConn->oPCM.nTimeslot_RX;

               p_conn->nUsedCh = p_dst_phone->nPCM_Ch;
               /* phone fd and pcm fd are the same */
               p_conn->nUsedCh_FD = p_dst_phone->nPhoneCh_FD;
            } /* if ((LOCAL_PCM_CALL == pConn->nType) */
            else if (LOCAL_CALL == pConn->nType)
            {
               p_conn->nUsedCh = p_dst_phone->nPhoneCh;
               p_conn->nUsedCh_FD = p_dst_phone->nPhoneCh_FD;
#ifdef TD_DECT
               TD_DECT_WideBandSet(pPhone, pConn, p_dst_phone, p_conn);
#endif /* TD_DECT */
            }
            else if (LOCAL_BOARD_CALL == pConn->nType)
            {
               p_conn->nUsedCh = p_dst_phone->nDataCh;
               p_conn->nUsedCh_FD = p_dst_phone->nDataCh_FD;
            } /* if ((LOCAL_PCM_CALL == pConn->nType) */
         } /* if (IE_RINGING == nAction) */
      } /* if ((LOCAL_CALL == pConn->nType) ... */
      else if (EXTERN_VOIP_CALL == pConn->nType)
      {
         /* message sender info */
         msg.nBoard_IDX = pPhone->pBoard->nBoard_IDX;
         msg.nSenderPhoneNum = pPhone->nPhoneNumber;
         msg.nSenderPort = ntohs(pConn->oUsedAddr.sin_port);
         memcpy(msg.MAC, pConn->oUsedMAC, ETH_ALEN);

         /* Receiver phone number */
         msg.nReceiverPhoneNum = pConn->oConnPeer.nPhoneNum;
         /* set receiver address */
         memset(&to_addr, 0, sizeof(TD_OS_sockAddr_t));
         to_addr.sin_addr = pConn->oConnPeer.oRemote.oToAddr.sin_addr;
         to_addr.sin_family = pConn->oConnPeer.oRemote.oToAddr.sin_family;
         to_addr.sin_port = htons(ADMIN_UDP_PORT);

         TRACE(TAPIDEMO, DBG_LEVEL_LOW,
               ("%s: Sending MSG using socket %d to %s:%d:\n"
                "%s  - call type %s\n"
                "%s  - sender UDP port %d\n"
                "%s  - sender MAC %02X:%02X:%02X:%02X:%02X:%02X\n"
                "%s  - sender phone number %d\n"
                "%s  - receiver phone number %d\n"
                "%s  - new internal event %s\n"
                "%s  - board idx %d\n"
                "%s  - feature id %d\n",
                pPhone->pCID_Msg->message[TD_CID_IDX_NAME].string.element,
                (int) pCtrl->nAdminSocket,
                inet_ntoa(to_addr.sin_addr), ntohs(to_addr.sin_port),
                pPhone->pBoard->pIndentBoard,(CALL_FLAG_PCM == msg.fPCM)? "PCM" :
                (CALL_FLAG_VOIP == msg.fPCM) ? "EXTERN" : "UNKNOWN",
                pPhone->pBoard->pIndentBoard, msg.nSenderPort,
                pPhone->pBoard->pIndentBoard, msg.MAC[0],
                msg.MAC[1],
                msg.MAC[2],
                msg.MAC[3],
                msg.MAC[4],
                msg.MAC[5],
                pPhone->pBoard->pIndentBoard, msg.nSenderPhoneNum,
                pPhone->pBoard->pIndentBoard, msg.nReceiverPhoneNum,
                pPhone->pBoard->pIndentBoard, psActionName,
                pPhone->pBoard->pIndentBoard, (int) msg.nBoard_IDX,
                pPhone->pBoard->pIndentBoard, msg.nFeatureID));
         /* send test message */
         COM_MOD_SEND_PHONE_MAPPING(msg, pPhone, pConn);

#ifdef TD_FAX_MODEM
#if (defined(TD_T38_FAX_TEST) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW))
         TD_T38_FAX_TEST_SET_ACTION(pCtrl, msg);
#endif /* (defined(TD_T38_FAX_TEST) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW)) */
#endif /* TD_FAX_MODEM */

         ret = TD_OS_SocketSendTo(pCtrl->nAdminSocket, (IFX_char_t *) &msg, 
                                  sizeof(msg),&to_addr);
         if (sizeof(msg) != ret)
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                  ("Error sending data (command %d byte(s)), %s. "
                   "(File: %s, line: %d)\n",
                   sizeof(msg), strerror(errno), __FILE__, __LINE__));
            return IFX_ERROR;
         }
      } /* else if (EXTERN_VOIP_CALL == pConn->nType) */
      else if (PCM_CALL == pConn->nType)
      {
         /* send board MAC address */
         memcpy(msg.MAC, pCtrl->oTapidemo_MAC_Addr, ETH_ALEN);
         /** \todo: check if remote side handles port in host format */
         msg.nSenderPort = ntohs(pConn->oUsedAddr.sin_port);
         msg.nBoard_IDX = pPhone->pBoard->nBoard_IDX;
         /* Get timeslot RX index */
         msg.nTimeslot_RX = pConn->oPCM.nTimeslot_RX;
         /* Get timeslot TX index */
         msg.nTimeslot_TX = pConn->oPCM.nTimeslot_TX;

         msg.nReceiverPhoneNum = pConn->oConnPeer.nPhoneNum;
         msg.nSenderPhoneNum = pPhone->nPhoneNumber;

         /* Set action to PCM phone, for example tell him that we are calling */
         memset(&to_addr, 0, sizeof(TD_OS_sockAddr_t));
         to_addr.sin_addr = pConn->oConnPeer.oPCM.oToAddr.sin_addr;
         to_addr.sin_family = pConn->oConnPeer.oPCM.oToAddr.sin_family;
         to_addr.sin_port = htons(ADMIN_UDP_PORT);

#ifndef TAPI_VERSION4
         TRACE(TAPIDEMO, DBG_LEVEL_LOW,
               ("%s: Sending MSG using PCM ch %d, socket %d to %s:%d:\n"
                "%s  - call type %s\n"
                "%s  - sender phone number %d\n"
                "%s  - receiver phone number %d\n"
                "%s  - new internal event %s\n"
                "%s  - board idx %d\n"
                "%s  - timeslot RX %d\n"
                "%s  - timeslot TX %d\n",
                pPhone->pCID_Msg->message[TD_CID_IDX_NAME].string.element,
                (int) pConn->nUsedCh, (int) pCtrl->nAdminSocket,
                inet_ntoa(to_addr.sin_addr), ntohs(to_addr.sin_port),
                pPhone->pBoard->pIndentBoard,
                (CALL_FLAG_PCM == msg.fPCM)? "PCM" :
                (CALL_FLAG_VOIP == msg.fPCM) ? "EXTERN" : "UNKNOWN",
                pPhone->pBoard->pIndentBoard, msg.nSenderPhoneNum,
                pPhone->pBoard->pIndentBoard, msg.nReceiverPhoneNum,
                pPhone->pBoard->pIndentBoard, psActionName,
                pPhone->pBoard->pIndentBoard,(int) msg.nBoard_IDX,
                pPhone->pBoard->pIndentBoard, msg.nTimeslot_RX,
                pPhone->pBoard->pIndentBoard, msg.nTimeslot_TX));
#endif /* TAPI_VERSION4 */

         ret = TD_OS_SocketSendTo(pCtrl->nAdminSocket, (IFX_char_t *) &msg, 
                                  sizeof(msg), &to_addr);

         if (sizeof(msg) != ret)
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                  ("Error sending data (command %d byte), %s. "
                   "(File: %s, line: %d)\n",
                   sizeof(msg), strerror(errno), __FILE__, __LINE__));
            return IFX_ERROR;
         }
      } /* else if (PCM_CALL == pConn->nType) */
      else if (pConn->nType == FXO_CALL)
      {
         /* Nothing at the moment */
      } /* else if (pConn->nType == FXO_CALL) */
      else
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Wrong call type, %d. (File: %s, line: %d)\n",
                (int) pConn->nType, __FILE__, __LINE__));
      }
   } /* if(nAction == IE_EXT_CALL_PROCEEDING||nAction == IE_EXT_CALL_WRONG_NUM)*/

   return IFX_SUCCESS;
} /* TAPIDEMO_SetAction() */

/**
   Initialize administration socket which will be used to manage phone
   connections.

   \return socket number or NO_SOCKET if error.
*/
TD_OS_socket_t TAPIDEMO_InitAdminSocket(IFX_void_t)
{
   TD_OS_socket_t socFd = NO_SOCKET;
   TD_OS_sockAddr_t my_addr;
   IFX_char_t ip_tos = 0x68;

   /* socket must be set SOCK_DGRAM to make handling messages easier,
      if SOCK_DGRAM only one message will be read at a time with recvfrom() */
   if(IFX_SUCCESS != TD_OS_SocketCreate(TD_OS_SOC_TYPE_DGRAM, &socFd)) 
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error: Can't create admin socket, %s.\n(File: %s, line: %d)\n",
             strerror(errno), __FILE__, __LINE__));
      return NO_SOCKET;
   }

   /* Socket Option Level IP, IP TOS field shall be set */
   setsockopt (socFd, IPPROTO_IP, IP_TOS, &ip_tos, sizeof(ip_tos));

   memset(&my_addr, 0, sizeof(TD_OS_sockAddr_t));
   my_addr.sin_family = AF_INET;
   my_addr.sin_port = htons(ADMIN_UDP_PORT);

   if (oCtrlStatus.pProgramArg->oMy_IP_Addr.sin_addr.s_addr)
   {
      my_addr.sin_addr.s_addr =
         oCtrlStatus.pProgramArg->oMy_IP_Addr.sin_addr.s_addr;
   }
   else
   {
      my_addr.sin_addr.s_addr = htonl(INADDR_ANY);
   }

   if(IFX_SUCCESS != TD_OS_SocketBind(socFd, &my_addr))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Error: Can't bind admin socket to port, %s.\n(File: %s, line: %d)\n",
            strerror(errno), __FILE__, __LINE__));
      TD_OS_SocketClose(socFd);
      return NO_SOCKET;
   }

#ifdef LINUX
   /* make the socket non blocking */
   fcntl(socFd, F_SETFL, O_NONBLOCK);
#endif /* LINUX */

   return socFd;
} /* TAPIDEMO_InitAdminSocket() */

/**
   Handle data from ADMIN socket.

   \param pCtrl  - pointer to status control structure

   \return IFX_SUCCESS message received and handled otherwise IFX_ERROR
*/
IFX_return_t TAPIDEMO_HandleCallSeq(CTRL_STATUS_t* pCtrl)
{
   static IFX_char_t buf[TD_MAX_COMM_MSG_SIZE];
   IFX_int32_t size = 0;
   IFX_int32_t ret = 0;
   IFX_int32_t event = 0;
   IFX_uint8_t from_MAC[ETH_ALEN];
   IFX_int32_t from_port = 0;
   IFX_int32_t board_idx = 0;
   TD_COMM_MSG_t msg = {0};
   TD_OS_sockAddr_t addr_from;
   PHONE_t* phone = IFX_NULL;
   CONNECTION_t* conn = IFX_NULL;
   IFX_boolean_t bFreeConn = IFX_FALSE;
   IFX_int32_t feature_id = 0;
   /* message receiver and sender phone numbers */
   IFX_int32_t nReciverPhoneNum;
   IFX_int32_t nSenderPhoneNum;
   IFX_char_t* psEventName = IFX_NULL;
#ifdef EASY336
   SVIP_libStatus_t SVIP_libRet;
#endif

   /* check input arguments */
   TD_PTR_CHECK(pCtrl, IFX_ERROR);

   size = sizeof(addr_from);

   /* Get data from socket */
   ret = TD_OS_SocketRecvFrom(pCtrl->nAdminSocket, buf, sizeof(buf), &addr_from);
   if (0 > ret)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err, No data read from ADMIN socket, %s. (File: %s, line: %d)\n",
             strerror(errno), __FILE__, __LINE__));
      return IFX_ERROR;
   }
   else if (0 == ret)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_LOW, ("No data read from ADMIN socket.\n"));
      return IFX_SUCCESS;
   }

   /* set and check message structure with received data */
   if (IFX_SUCCESS != Common_CommMsgSet(&msg, buf, ret))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err, Invalid data on ADMIN socket (%d). (File: %s, line: %d)\n",
             (int) pCtrl->nAdminSocket, __FILE__, __LINE__));
      return IFX_ERROR;

   }
   /* change peer address if needed for ITM */
   COM_ITM_SET_IP(addr_from);

   /* event to send to phone */
   event = msg.nAction;

   /* set pointer string with action name */
   psEventName = Common_Enum2Name(event, TD_rgIE_EventsName);

   /* check if name was set, if name was not set, then invalid value
      of TD_rgIE_EventsName is set */
   TD_PTR_CHECK(psEventName, IFX_ERROR);

   /* check if workaround */
   if(event  == IE_EXT_CALL_WORKAROUND_EASY508xx)
   {
      /* It is workaround to avoid problem with first external/PCM
         connection with EASY508xx. TAPIDEMO ignores this type of action */
      return IFX_SUCCESS;
   }

   /* get feature id */
   feature_id = msg.nFeatureID;

   /* Caller stuff */
   /* get board index (used to get board from pCtrl->rgoBoards[]) */
   board_idx = msg.nBoard_IDX;
   /* get message sending phone number */
   nSenderPhoneNum = msg.nSenderPhoneNum;
   memcpy(from_MAC, msg.MAC, ETH_ALEN);
   from_port = msg.nSenderPort;

   /* Called stuff */
   /* get message receiving phone number */
   nReciverPhoneNum = msg.nReceiverPhoneNum;

   /* Print message data */
   TRACE(TAPIDEMO, DBG_LEVEL_LOW,
         ("Received MSG from %s:%d with:\n"
          " - call type %s\n",
          inet_ntoa(addr_from.sin_addr), ntohs(addr_from.sin_port),
          (CALL_FLAG_PCM == msg.fPCM)? "PCM" : (CALL_FLAG_VOIP == msg.fPCM) ?
          "EXTERN" : "UNKNOWN"));
   /* messages currently are received only for PCM_CALL and EXTERN_VOIP_CALL,
      UDP port is only sent during EXTERN_VOIP_CALL, but if some error ocurres
      e.g. wrong call type, then print this data also. */
   if (CALL_FLAG_PCM != msg.fPCM)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_LOW,
            (" - sender UDP port %d\n", msg.nSenderPort));

   }
   TRACE(TAPIDEMO, DBG_LEVEL_LOW,
         (" - sender MAC %02X:%02X:%02X:%02X:%02X:%02X\n"
          " - message sender phone number %d\n"
          " - message receiver phone number %d\n"
          " - board idx %d\n"
          " - new internal event %s\n"
          " - feature id %d\n",
          msg.MAC[0],
          msg.MAC[1],
          msg.MAC[2],
          msg.MAC[3],
          msg.MAC[4],
          msg.MAC[5],
          msg.nSenderPhoneNum, msg.nReceiverPhoneNum,
          msg.nBoard_IDX,
          psEventName,
          msg.nFeatureID));
   /* messages currently are received only for PCM_CALL and EXTERN_VOIP_CALL,
      timeslots are only sent during PCM_CALL, but if some error ocurres
      e.g. wrong call type, then print this data also. */
   if (CALL_FLAG_VOIP != msg.fPCM)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_LOW,
            (" - timeslot RX %d\n"
             " - timeslot TX %d\n",
             msg.nTimeslot_RX, msg.nTimeslot_TX));
   }

   /* Check if phone number is avaible */
   phone = ABSTRACT_GetPHONE_OfCalledNum(pCtrl, msg.nReceiverPhoneNum);

   /* caller is dialing number but, didn't finish dialing full number */
   if (IFX_NULL == phone)
   {
      /* If receive call setup messsage, send answer */
      if (IE_EXT_CALL_SETUP == event)
      {
         /* phoney phone and connection to allow usage of TAPIDEMO_SetAction() */
         CONNECTION_t oConnTmp;
         PHONE_t oPhoneTmp;
         /* Received called number is incomplete or incorrect */
         /* Allocate temporary connection structure and send msg to caller */
         bFreeConn = IFX_TRUE;
         conn = &oConnTmp;
         memset(conn, 1, sizeof(CONNECTION_t));

         /* Allocate temporary phone structure and send msg to caller */
         phone = &oPhoneTmp;
         memset(phone, 1, sizeof(PHONE_t));

         conn->nType = EXTERN_VOIP_CALL;
         memcpy(conn->oConnPeer.oRemote.MAC, from_MAC, ETH_ALEN);
         conn->oConnPeer.oRemote.oToAddr = addr_from;
         conn->oConnPeer.nPhoneNum = msg.nSenderPhoneNum;

#ifndef EASY336
         memcpy(conn->oUsedMAC, pCtrl->oTapidemo_MAC_Addr, ETH_ALEN);
#else
         SVIP_libRet = SVIP_libMAC_AddressForVoIPGet(addr_from.sin_addr.s_addr,
            conn->oUsedMAC);
         if (SVIP_libRet != SVIP_libStatusOk)
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Err: SVIP_libMAC_AddressForVoIPGet returned error %d "
                "(File: %s, line: %d)\n",
                SVIP_libRet, __FILE__, __LINE__));
            return IFX_ERROR;
         }
#endif
         conn->oUsedAddr.sin_addr = pCtrl->oTapidemo_IP_Addr.sin_addr;

         /* We don't know phone number, so we don't know below values */
         conn->nUsedSocket = NO_SOCKET;
         conn->nUsedCh_FD = NO_CHANNEL_FD;
         conn->nUsedCh = NO_USED_CHANNEL;
         conn->oUsedAddr.sin_port = ADMIN_UDP_PORT;

         /* We need to send correct called phone number, it's needed in
         function ABSTRACT_GetConnOfPhone for second phone */
         phone->nPhoneNumber = nReciverPhoneNum;

         /* It is workaround,
         which avoid problem with first external/PCM connection to EASY508xx */
         if (IFX_ERROR == TAPIDEMO_SetAction(pCtrl, phone, conn,
                                             IE_EXT_CALL_WORKAROUND_EASY508xx))
         {
            return IFX_ERROR;
         }

         if (MIN_PHONE_NUM <= msg.nReceiverPhoneNum)
         {
            /* Received phone number is not correct, if another digit will be dialed, then dialed phone number ,
            inform about it phone on second board */
            if (IFX_ERROR == TAPIDEMO_SetAction(pCtrl, phone, conn,
                                                IE_EXT_CALL_WRONG_NUM))
            {
               return IFX_ERROR;
            }
         }
         else
         {
            /* Received phone number is not completed,
            so inform about it second board */
            if (IFX_ERROR == TAPIDEMO_SetAction(pCtrl, phone, conn,
                                                IE_EXT_CALL_PROCEEDING))
            {
               return IFX_ERROR;
            }
         } /* if (pCtrl->nSumPhone < msg.nCalledPhoneNum) */
         return IFX_SUCCESS;
      }
      else
      {
         /* For rest of the cases print error */
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Error get phone. (File: %s, line: %d)\n",
                __FILE__, __LINE__));
         return IFX_ERROR;
      } /* if (IE_EXT_CALL_SETUP == event) */
   } /* if (IFX_NULL == phone) */
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_LOW,
            ("%s: Received event %s on ADMIN socket\n",
             phone->pCID_Msg->message[TD_CID_IDX_NAME].string.element,
             psEventName));

      if (CALL_FLAG_PCM == msg.fPCM)
      {
         /** \todo: this function should be reviewed */
         conn = ABSTRACT_GetConnOfPhone(phone, PCM_CALL,
                                        nReciverPhoneNum, nSenderPhoneNum,
                                        addr_from, &bFreeConn);
         if (conn == IFX_NULL)
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                  ("Error get connection. (File: %s, line: %d)\n",
                   __FILE__, __LINE__));
            return IFX_ERROR;
         } /* if (conn == IFX_NULL) */

         if (bFreeConn)
         {
            conn->nType = PCM_CALL;
            conn->oConnPeer.oPCM.oToAddr = addr_from;
            conn->oConnPeer.oPCM.oToAddr.sin_port = from_port;
            conn->oConnPeer.nPhoneNum = msg.nSenderPhoneNum;

            conn->oUsedAddr.sin_addr = pCtrl->oTapidemo_IP_Addr.sin_addr;
            conn->nUsedSocket = phone->nSocket;
            conn->nUsedCh_FD = PCM_GetFD_OfCh(phone->nPCM_Ch, phone->pBoard);
            conn->nUsedCh = phone->nPCM_Ch;
            conn->nFeatureID = feature_id;
         } /* if (free_conn) */

         /* Set new event for phone */
         phone->nIntEvent_FXS = event;

         if (!pCtrl->pProgramArg->oArgFlags.nPCM_Master)
         {
            if (IE_RINGING == event || IE_RINGBACK == event)
            {
               /* Set PCM timeslots */
               conn->oPCM.nTimeslot_RX = msg.nTimeslot_TX;
               conn->oPCM.nTimeslot_TX = msg.nTimeslot_RX;
            }
         }
         /* check event type */
         if (IE_RINGING == event)
         {
            /* check if CID support is on */
            if (pCtrl->pProgramArg->oArgFlags.nCID)
            {
               /* get CID data */
               if (IFX_SUCCESS != CID_FromPeerMsgGet(phone, &msg))
               {

                  TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                        ("Err, CID_FromPeerMsgGet failed. "
                         "(File: %s, line: %d)\n",
                         __FILE__, __LINE__));
               }
            } /* if (pCtrl->pProgramArg->oArgFlags.nCID) */
         } /* if (IE_RINGING == event) */

#ifndef TAPI_VERSION4
         TRACE(TAPIDEMO, DBG_LEVEL_LOW,
               ("%s: PCM connection on channel %d:\n"
                "%s  - used socket: %d\n"
                "%s  - timeslot RX %d TX %d\n"
                "%s  - to: %s:%d\n",
                phone->pCID_Msg->message[TD_CID_IDX_NAME].string.element,
                (int) conn->nUsedCh,
                phone->pBoard->pIndentBoard,
                (int) conn->nUsedSocket,
                phone->pBoard->pIndentBoard,
                conn->oPCM.nTimeslot_RX, conn->oPCM.nTimeslot_TX,
                phone->pBoard->pIndentBoard,
                inet_ntoa(conn->oUsedAddr.sin_addr),
                ntohs(conn->oUsedAddr.sin_port)));
#endif /* TAPI_VERSION4 */
         TRACE(TAPIDEMO, DBG_LEVEL_LOW,
               ("%s  - from %s:%d\n",
                phone->pBoard->pIndentBoard,
                inet_ntoa(conn->oConnPeer.oPCM.oToAddr.sin_addr),
                ntohs(conn->oConnPeer.oPCM.oToAddr.sin_port)));
      } /* if (CALL_FLAG_PCM == msg.fPCM) */
      else if (CALL_FLAG_VOIP == msg.fPCM)
      {
         /* note: this function should be reviewed */
         conn = ABSTRACT_GetConnOfPhone(phone, EXTERN_VOIP_CALL,
                                        nReciverPhoneNum, nSenderPhoneNum,
                                        addr_from, &bFreeConn);
         if (IFX_NULL == conn)
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                  ("Error get connection. (File: %s, line: %d)\n",
                   __FILE__, __LINE__));
            return IFX_ERROR;
         } /* if (IFX_NULL == conn) */

         if (bFreeConn)
         {
            conn->nType = EXTERN_VOIP_CALL;

            memcpy(conn->oConnPeer.oRemote.MAC, from_MAC, ETH_ALEN);
            conn->oConnPeer.oRemote.oToAddr = addr_from;
            conn->oConnPeer.nPhoneNum = msg.nSenderPhoneNum;
#ifdef TD_DECT
            /* if DECT phone is available then add data channel */
            if (PHONE_TYPE_DECT == phone->nType && TD_NOT_SET == phone->nDataCh &&
                S_READY == phone->rgStateMachine[FXS_SM].nState &&
                IE_RINGING == event)
            {
               /* this is unlikely to happend but if next message is not received
                  then this data channel will not be freed untill this phone
                  goes off-hook and then on-hook */
               if (IFX_SUCCESS != TD_DECT_DataChannelAdd(phone))
               {
                  TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                        ("Err TD_DECT_DataChannelAdd failed. (File: %s, line: %d)\n",
                         __FILE__, __LINE__));
                  return IFX_ERROR;
               }
            } /* if DECT phone */
#endif /* TD_DECT */
#ifndef EASY336
            memcpy(conn->oUsedMAC, pCtrl->oTapidemo_MAC_Addr, ETH_ALEN);
#else
            SVIP_libRet = SVIP_libMAC_AddressForVoIPGet(
               addr_from.sin_addr.s_addr, conn->oUsedMAC);
            if (SVIP_libRet != SVIP_libStatusOk)
            {
               TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                  ("Err: SVIP_libMAC_AddressForVoIPGet returned error %d "
                   "(File: %s, line: %d)\n",
                   SVIP_libRet, __FILE__, __LINE__));
               return IFX_ERROR;
            }
#endif
            conn->oUsedAddr.sin_addr = pCtrl->oTapidemo_IP_Addr.sin_addr;
            conn->nUsedSocket = phone->nSocket;
#ifdef TD_DECT
            if (TD_NOT_SET != phone->nDataCh)
#endif /* TD_DECT */
#ifdef TAPI_VERSION4
            if (!phone->pBoard->fSingleFD)
#endif
            {
               conn->nUsedCh_FD = VOIP_GetFD_OfCh(phone->nDataCh,
                                                  phone->pBoard);
            }
            conn->nUsedCh = phone->nDataCh;

#ifdef TAPI_VERSION4
            if (phone->pBoard->fUseSockets)
#endif /* TAPI_VERSION4 */
            {
               if (!pCtrl->pProgramArg->oArgFlags.nQos)
               {
                  conn->oUsedAddr.sin_port =
                     VOIP_GetSocketPort(conn->nUsedSocket, pCtrl);
               }
            }
         } /* if (free_conn) */

#ifdef TD_FAX_MODEM
#if (defined(TD_T38_FAX_TEST) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW))
         if (IE_RINGBACK == event || IE_RINGING == event)
         {
            TD_T38_FAX_TEST_HANDLE_CALL_SEQUENCE(pCtrl, msg, phone);
         }
#endif /* (defined(TD_T38_FAX_TEST) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW)) */
#endif /* TD_FAX_MODEM */
         if (IE_EXT_CALL_SETUP == event)
         {
            /* It is workaround,
            which avoid problem with first external connection to EASY508xx */
            if (IFX_ERROR == TAPIDEMO_SetAction(pCtrl, phone, conn,
                                                IE_EXT_CALL_WORKAROUND_EASY508xx))
            {
               return IFX_ERROR;
            }
            /* Received phone number is correct,
               so inform about it second phone */
            if (IFX_ERROR == TAPIDEMO_SetAction(pCtrl, phone, conn,
                                                IE_EXT_CALL_ESTABLISHED))
            {
               return IFX_ERROR;
            }
            return IFX_SUCCESS;
         } /* if (IE_EXT_CALL_SETUP == event) */
         /* save port number, caller sends it with IE_RINGING,
            called phone sends it with IE_ACTIVE action */
         else if (IE_ACTIVE == event || IE_RINGING == event)
         {
            conn->oConnPeer.oRemote.oToAddr.sin_port = htons(from_port);
         }
         /* check event type */
         if (IE_RINGING == event)
         {
            /* check if CID support is on */
            if (pCtrl->pProgramArg->oArgFlags.nCID)
            {
               /* get CID data */
               if (IFX_SUCCESS != CID_FromPeerMsgGet(phone, &msg))
               {

                  TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                        ("Err, CID_FromPeerMsgGet failed. "
                         "(File: %s, line: %d)\n",
                         __FILE__, __LINE__));
               }
            } /* if (pCtrl->pProgramArg->oArgFlags.nCID) */
         } /* if (IE_RINGING == event) */

         memcpy(conn->oConnPeer.oRemote.MAC, from_MAC, ETH_ALEN);
         conn->nFeatureID = feature_id;

         /* Set new event for phone */
         phone->nIntEvent_FXS = event;

         if ((phone->rgStateMachine[CONFIG_SM].nState != S_READY) ||
             (event == IE_CONFIG_PEER))
         {
            phone->nIntEvent_Config = event;
         }

         if (IE_ACTIVE == event)
         {
            TRACE(TAPIDEMO, DBG_LEVEL_LOW,
                  ("%s: VOIP connection on channel %d:\n",
                   phone->pCID_Msg->message[TD_CID_IDX_NAME].string.element,
                   phone->nPhoneCh));
#ifndef TAPI_VERSION4
            TRACE(TAPIDEMO, DBG_LEVEL_LOW,
                  ("%s - used socket: %d\n",
                   phone->pBoard->pIndentBoard,
                   conn->nUsedSocket));
#else /* TAPI_VERSION4 */
            TRACE(TAPIDEMO, DBG_LEVEL_LOW,
                  ("%s - device: %d\n",
                   phone->pBoard->pIndentBoard,
                   phone->nDev));
#endif /* TAPI_VERSION4 */
            TRACE(TAPIDEMO, DBG_LEVEL_LOW,
                  ("%s - to: %s:%d\n",
                   phone->pBoard->pIndentBoard,
                   inet_ntoa(conn->oUsedAddr.sin_addr),
                   ntohs(conn->oUsedAddr.sin_port)));
            TRACE(TAPIDEMO, DBG_LEVEL_LOW,
                  ("%s - from: %s:%d\n",
                   phone->pBoard->pIndentBoard,
                   inet_ntoa(conn->oConnPeer.oRemote.oToAddr.sin_addr),
                   ntohs(conn->oConnPeer.oRemote.oToAddr.sin_port)));
         }
      } /* if (CALL_FLAG_PCM == msg.fPCM) */
      else
      {
         /* If we came here then wrong message was received */
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Received Message with UNKNOWN call flag. "
                "(File: %s, line: %d)\n", __FILE__, __LINE__));
      }

      /* Action from other phone received, handle it. */
      while ( phone->nIntEvent_Config != IE_NONE )
      {
         ST_HandleState_Config(pCtrl, phone, conn);
      }

      while ( phone->nIntEvent_FXS!= IE_NONE)
      {
         ST_HandleState_FXS(pCtrl, phone, conn);
      }

      return IFX_SUCCESS;
   } /* if (IFX_NULL == phone) */

} /* ADMIN_HandleCallSeq() */

#ifdef LINUX
/**
   Initialize socket for syslog.

   \return socket number or NO_SOCKET if error.
*/
IFX_int32_t TAPIDEMO_InitSyslogSocket(IFX_void_t)
{
   TD_OS_socket_t socFd = NO_SOCKET;
   TD_OS_sockAddr_t my_addr;

   if(IFX_SUCCESS != TD_OS_SocketCreate(TD_OS_SOC_TYPE_DGRAM, &socFd))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("InitSyslogSocket: Can't create UDP socket, %s. "
             "(File: %s, line: %d)\n",
             strerror(errno), __FILE__, __LINE__));
      return NO_SOCKET;
   }

   memset(&my_addr, 0, sizeof(TD_OS_sockAddr_t));
   my_addr.sin_addr.s_addr = INADDR_ANY;
   my_addr.sin_port = 0;
   my_addr.sin_family = AF_INET;

   if(IFX_SUCCESS != TD_OS_SocketBind(socFd, &my_addr))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("InitSyslogSocket: Can't bind to port, %s. (File: %s, line: %d)\n",
             strerror(errno), __FILE__, __LINE__));
      TD_OS_SocketClose(socFd);
      return NO_SOCKET;
   }

   return socFd;
} /* TAPIDEMO_InitSyslogSocket() */

/**
   Send data to remote syslog

   \param pBuffer - pointer to message
   \param nSyslogSocket - socket used for communication with remote Syslog

   \return  IFX_SUCCESS if everything OK, otherwise IFX_ERROR
*/
IFX_return_t TAPIDEMO_SendLogRemote(IFX_char_t* pBuffer,
                                    TD_OS_socket_t nSyslogSocket)
{
   IFX_int32_t ret = 0;
   IFX_int32_t nLength;

   /* It is used to build external syslog message */
   IFX_char_t pExtSyslogMsg[MAX_EXT_SYSLOG_MSG_LEN] = {0};

   TD_OS_sockAddr_t server_addr;

   TD_PTR_CHECK(pBuffer, IFX_ERROR);

   memset(&server_addr, 0, sizeof(TD_OS_sockAddr_t));
   server_addr.sin_addr.s_addr =
      oTraceRedirection.oSyslog_IP_Addr.sin_addr.s_addr;
   server_addr.sin_port = oTraceRedirection.oSyslog_IP_Addr.sin_port;
   server_addr.sin_family = AF_INET;

   /* Correct syslog message has The Priority part (PRI) at the beginning.*/
   /* The PRI part MUST have three, four, or five characters and will be */
   /* bound with angle brackets as the first and last characters. */
   /* The code set used in this part MUST be seven-bit ASCII in an eight-bit field. */
   /* The number contained within these angle brackets is known as */
   /* the Priority value and represents both the Facility and */
   /* Severity. The Priority value is calculated by first multiplying the Facility */
   /* number by 8 and then adding the numerical value of the Severity. */
   /* Source: RFC3164 */

   /* In TAPIDEMO, Facility is set on 1 (user-level messages) and */
   /* Severity is set on 7 (Debug: debug-level messages): 1*8+7 = 15 */
   /* We need buffer with lenght + PRI' length + '\n' */

   /* Add PRI part */
   strcpy(pExtSyslogMsg, "<15>");
   /* Add message part */
   strcat(pExtSyslogMsg, pBuffer);
   nLength = strlen(pExtSyslogMsg);

   ret = TD_OS_SocketSendTo(nSyslogSocket, pExtSyslogMsg, nLength, &server_addr);
   if (ret != nLength)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error calling sendto(), %s, socket %d"
             " (File: %s, line: %d)\n",
             strerror(errno), nSyslogSocket,
             __FILE__, __LINE__));

      return IFX_ERROR;
   }

   return IFX_SUCCESS;
}

/**
   Split data on smaller packets and send them.
   Data are splitted according to end of line. Every line is separate packet.

   \param pBuffer - pointer to message

   \return  IFX_SUCCESS if everything OK, otherwise IFX_ERROR
*/
IFX_return_t TAPIDEMO_SplitAndSendLog(IFX_char_t* pBuffer)
{
   IFX_return_t ret;
   IFX_char_t* pPresentCh;
   IFX_char_t* pPreviusCh;

   /* It is used to keep split data */
   IFX_char_t pTmpBuffer[MAX_LOG_LEN] = {0};

   TD_PTR_CHECK(pBuffer, IFX_ERROR);
   TD_PARAMETER_CHECK((MAX_LOG_LEN <= strlen(pBuffer)),
                      strlen(pBuffer), IFX_ERROR);

   /* If there isn't char '\n' at the end of string, some data can be missed */
   pBuffer[strlen(pBuffer)] = TD_EOL;

   /* Locate first occurrence of character '\n' */
   pPresentCh = strchr(pBuffer, TD_EOL);
   pPreviusCh = pBuffer;

   while (pPresentCh != NULL)
   {
      if (pPresentCh - pPreviusCh + 1 < MAX_LOG_LEN)
      {
         strncpy(pTmpBuffer, pPreviusCh, pPresentCh - pPreviusCh + 1);

         if (oCtrlStatus.pProgramArg->nTraceRedirection ==
             TRACE_REDIRECTION_SYSLOG_LOCAL)
         {
            /* Redirect log to local syslog */
            syslog(LOG_DEBUG, pTmpBuffer);
         } /* TRACE_REDIRECTION_SYSLOG_LOCAL */
         else if (oCtrlStatus.pProgramArg->nTraceRedirection ==
                  TRACE_REDIRECTION_SYSLOG_REMOTE)
         {
            /* Redirect log to remote syslog */
            ret = TAPIDEMO_SendLogRemote(pTmpBuffer,
                                         oTraceRedirection.nSyslogSocket);
            if (ret == IFX_ERROR)
            {
               TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                     ("Error sending log to remote syslog. "
                      "(File: %s, line: %d)\n",
                      __FILE__, __LINE__));
               return IFX_ERROR;
            }
         } /* TRACE_REDIRECTION_SYSLOG_REMOTE */
         else if (oCtrlStatus.pProgramArg->nTraceRedirection ==
                  TRACE_REDIRECTION_FILE)
         {
            ret = TD_OS_FWrite(pTmpBuffer, strlen(pTmpBuffer), 1,
                               oTraceRedirection.pFileLogFD);
            if (0 > ret)
            {
               TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                     ("Error writing log to file, %s. (File: %s, line: %d)\n",
                      strerror(errno), __FILE__, __LINE__));
               return IFX_ERROR;
            }
         } /* TRACE_REDIRECTION_FILE */
         else
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                  ("Unknow type of log redirection (File: %s, line: %d)\n",
                   __FILE__, __LINE__));
            return IFX_ERROR;
         }
         /* Clean tmp buffer */
         memset(pTmpBuffer, 0, MAX_LOG_LEN);
      }
      else
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Message too long. (File: %s, line: %d)\n",
                __FILE__, __LINE__));
      }

      pPreviusCh = pPresentCh+1;
      /* Locate next occurrence of character '\n' */
      pPresentCh = strchr(pPreviusCh, TD_EOL);
   } /* while */

   return IFX_SUCCESS;
}
/**
   Prepare logs' redirection, e.g. set up syslog, open file.

   \return IFX_SUCCESS if everything OK, otherwise IFX_ERROR
*/
IFX_return_t TAPIDEMO_PreapareLogRedirection(IFX_void_t)
{
   IFX_char_t full_filename[MAX_FULL_FILE_LEN] = {0};

   if (oCtrlStatus.pProgramArg->nTraceRedirection ==
       TRACE_REDIRECTION_SYSLOG_LOCAL)
   {
      /* Start redirecting logs to local syslog */
      openlog("Tapidemo", LOG_PID, LOG_USER);

      oTraceRedirection.fStartRedirectLogs = IFX_TRUE;

      TRACE(TAPIDEMO, DBG_LEVEL_LOW,
            ("Start redirecting logs to local syslog\n"));
   }
   else if (oCtrlStatus.pProgramArg->nTraceRedirection ==
            TRACE_REDIRECTION_SYSLOG_REMOTE)
   {
      /* Start redirecting logs to remote syslog */
      oTraceRedirection.nSyslogSocket = TAPIDEMO_InitSyslogSocket();
      if (oTraceRedirection.nSyslogSocket == NO_SOCKET)
      {
         oTraceRedirection.fStartRedirectLogs = IFX_TRUE;

         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Can not redirect logs to remote syslog "
                "on address %s:%d\n",
                inet_ntoa(oTraceRedirection.oSyslog_IP_Addr.sin_addr),
                oTraceRedirection.oSyslog_IP_Addr.sin_port));
         return IFX_ERROR;
      }
      else
      {
         oTraceRedirection.fStartRedirectLogs = IFX_TRUE;

         TRACE(TAPIDEMO, DBG_LEVEL_LOW,
               ("Start redirecting logs to remote syslog "
                "on address %s:%d\n",
                inet_ntoa(oTraceRedirection.oSyslog_IP_Addr.sin_addr),
                oTraceRedirection.oSyslog_IP_Addr.sin_port));
      }
   }
   else if (oCtrlStatus.pProgramArg->nTraceRedirection ==
            TRACE_REDIRECTION_FILE)
   {
      /* Start redirecting logs to file */
      strcpy(full_filename, oTraceRedirection.sPathToLogFiles);
      oTraceRedirection.pFileLogFD = TD_OS_FOpen(full_filename, "a");
      if (oTraceRedirection.pFileLogFD < 0)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Can not create / open log file, %s\n"
                "File: %s, line: %d)\n",
                 strerror(errno), __FILE__, __LINE__));
         return IFX_ERROR;
      }
      else
      {
         oTraceRedirection.fStartRedirectLogs = IFX_TRUE;

         TRACE(TAPIDEMO, DBG_LEVEL_LOW,
               ("Start redirecting logs to file %s ",
                full_filename));
      }
   }
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error on setting redirection. "
             "File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   return IFX_SUCCESS;
}

/**
   Run Tapidemo as daemon

   \return IFX_SUCCESS if everything OK, otherwise IFX_ERROR
*/
IFX_return_t TAPIDEMO_CreateDaemon(CTRL_STATUS_t* pCtrl)
{
   pid_t pid1, sid;

   TD_PTR_CHECK(pCtrl, IFX_ERROR);

   /*******************************************************************
         Create daemon
    *******************************************************************/
   /* Create new process - TAPIDEMO */
   pid1  = fork();
   if (pid1 < 0)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Can not create daemon. Error on fork %s\n"
             "File: %s, line: %d)\n",
             strerror(errno), __FILE__, __LINE__));

      return IFX_ERROR;
   }
   else if (pid1 != 0)
   {
      /* Close parent process */
      exit(0);
   }

   /*******************************************************************
         Prepare pipe for redirect log
    *******************************************************************/
   strcpy(pCtrl->oMultithreadCtrl.oPipe.oRedirectLogPipe.rgPipeName, REDIRECT_LOG_PIPE);
   if (IFX_SUCCESS != 
       Common_PreparePipe(&pCtrl->oMultithreadCtrl.oPipe.oRedirectLogPipe))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error: Failed to prepare pipe for redirect log.\n"
             "(File: %s, line: %d)\n", __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /********************************************************
         Start thread for handle log redirection
    ********************************************************/
   if (IFX_SUCCESS != 
       Common_StartThread(&pCtrl->oMultithreadCtrl.oThread.oHandleLogThreadCtrl,
                            TAPIDEMO_HandleLogThread,
                            "HandleLog",
                            &pCtrl->oMultithreadCtrl.oPipe.oLogSynchPipe,
                            &pCtrl->oMultithreadCtrl.oLock.oLogThreadStopLock))
   {
       TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
             ("Error: Failed to start thread for handle log redirection\n."
              "(File: %s, line: %d)\n", __FILE__, __LINE__));
       return IFX_ERROR;
   }
   
   /********************************************************
         Redirect standard output and error output stream
    ********************************************************/
   /* Redirect stdout to logger via pipe */
   if (-1 == dup2(pCtrl->oMultithreadCtrl.oPipe.oRedirectLogPipe.rgPipeFd[TD_PIPE_IN], 
                  STDOUT_FILENO))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Can not redirect standard output. Error on dup2 %s.\n"
             "File: %s, line: %d)\n",
             strerror(errno), __FILE__, __LINE__));

      return IFX_ERROR;
   }
   /* Redirect stderr to logger via pipe */
   if (-1 == dup2(pCtrl->oMultithreadCtrl.oPipe.oRedirectLogPipe.rgPipeFd[TD_PIPE_IN], 
                  STDERR_FILENO))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Can not redirect standard error output. Error on dup2 %s.\n"
             "File: %s, line: %d)\n",
             strerror(errno), __FILE__, __LINE__));

      return IFX_ERROR;
   }
   /* Start new session */
   sid = setsid();
   umask(0);

   return IFX_SUCCESS;
}
#endif /* defined(LINUX) */

/* ============================= */
/* Global function definition    */
/* ============================= */

/**
   Get ip address of board and set it.

   \param pProgramArg - pointer to program arguments structure

   \return IFX_ERROR if an error, otherwise IFX_SUCCESS (no error occurred)
*/
IFX_return_t TAPIDEMO_SetDefaultAddr(PROGRAM_ARG_t* pProgramArg)
{
#ifdef EASY336
   SVIP_libStatus_t SVIP_libRet;
   IFX_char_t pIP[INET_ADDR_LEN];
#else
   /* In man page of hostname is written, that host names are limited
      to 255 bytes. */
   IFX_char_t buffer[256];
#ifdef LINUX
   /** Value for local ip address */
   const IFX_char_t* LOCAL_IP_ADDRESS = "127.0.0.1";
   struct hostent* he = IFX_NULL;
   TD_OS_socket_t sockt_fd = NO_SOCKET;
   struct ifreq ifr;
   IFX_boolean_t bAskSocket = IFX_FALSE;
#else /* LINUX */
   IFX_int32_t addr = 0;
#endif /* LINUX */
#endif /* EASY336 */

   TD_PTR_CHECK(pProgramArg, IFX_ERROR);

   if (pProgramArg->oMy_IP_Addr.sin_addr.s_addr != 0)
   {
      return IFX_SUCCESS;
   }

#ifndef EASY336
   if (gethostname(buffer, sizeof(buffer)) != 0)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err: gethostname(), %s. (File: %s, line: %d)\n",
             strerror(errno), __FILE__, __LINE__));
      return IFX_ERROR;
   }

   TRACE(TAPIDEMO, DBG_LEVEL_LOW, ("gethostname() returned %s.\n", buffer));

#ifdef LINUX
   he = gethostbyname(buffer);

   if (IFX_NULL == he)
   {
      bAskSocket = IFX_TRUE;
   }
   else
   {
      if (he->h_addr_list[0] != IFX_NULL)
      {
         pProgramArg->oMy_IP_Addr.sin_addr.s_addr =
            *(IFX_uint32_t *)he->h_addr_list[0];

         if (strcmp(inet_ntoa(pProgramArg->oMy_IP_Addr.sin_addr),
                    LOCAL_IP_ADDRESS) == 0)
         {
            TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
                  ("Local address (127.0.0.1) read, "
                   "will now try to check eth0 for right address.\n"
                   "On error local address will be used.\n"));
            bAskSocket = IFX_TRUE;
         }
      }
   }

   if (bAskSocket)
   {
      /* TD_OS_SocketSelect() can't be used because, we need third argument */
      sockt_fd = socket(AF_INET, SOCK_DGRAM, IPPROTO_IP);
      if (sockt_fd == -1)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Err: socket(), %s. (File: %s, line: %d)\n",
                strerror(errno), __FILE__, __LINE__));
         return IFX_ERROR;
      }

      memset(&ifr, 0, sizeof (struct ifreq));
      strcpy (ifr.ifr_name, "eth0");
      ifr.ifr_addr.sa_family = AF_INET;
      if (ioctl(sockt_fd, SIOCGIFADDR, &ifr) == -1)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Err with ioctl cmd SIOCGIFADDR %s. "
                "(File: %s, line: %d)\n",
                strerror(errno), __FILE__, __LINE__));
         TD_OS_SocketClose(sockt_fd);
         return IFX_ERROR;
      }

      pProgramArg->oMy_IP_Addr.sin_addr.s_addr =
         ((TD_OS_sockAddr_t *) &ifr.ifr_addr)->sin_addr.s_addr;

      close(sockt_fd);
   } /* if (bAskSocket) */

  TRACE(TAPIDEMO, DBG_LEVEL_LOW,
        ("Default IP Address for host %s is %s\n",
         buffer, inet_ntoa(pProgramArg->oMy_IP_Addr.sin_addr)));

#elif VXWORKS /* LINUX */

   /* There is also possibility to use resolvGetHostByName(), but
      in file configAll.h support for library resolvLib,
      #define INCLUDE_DNS_RESOLVER must be added and new image
      prepared */
   addr = hostGetByName(buffer);

   pProgramArg->oMy_IP_Addr.sin_addr.s_addr = addr;

   TRACE(TAPIDEMO, DBG_LEVEL_LOW,
         ("Default IP Address for host %s is %s\n",
          buffer, inet_ntoa(pProgramArg->oMy_IP_Addr.sin_addr)));
#else
   TRACE(TAPIDEMO, DBG_LEVEL_LOW,
         ("Unsupported function TAPIDEMO_SetDefaultAddr()\n"));   
#endif /* LINUX */

#else /* EASY336 */
   SVIP_libRet = SVIP_libGetAddress("eth0", pIP, IFX_NULL);
   if (SVIP_libRet != SVIP_libStatusOk)
      return IFX_ERROR;

   if (IFX_SUCCESS != TD_OS_SocketAton(pIP, &pProgramArg->oMy_IP_Addr))
      return IFX_ERROR;

   TRACE(TAPIDEMO, DBG_LEVEL_LOW,
         ("Default IP-Address is %s\n", pIP));
#endif /* EASY336 */

   return IFX_SUCCESS;
} /* TAPIDEMO_SetDefaultAddr() */

/**
   Get MAC address of board and set it.

   \return IFX_ERROR if an error, otherwise IFX_SUCCESS (no error occurred)
*/
static IFX_int32_t TAPIDEMO_SetMACAddr()
{
#ifndef EASY336
#ifdef LINUX
   IFX_int32_t sockt_fd = 0;
   struct ifreq ifr;
#endif /* LINUX */

#ifdef LINUX
   /* TD_OS_SocketCreate() can't be used because, we need third argument */
   sockt_fd = socket(AF_INET, SOCK_DGRAM, IPPROTO_IP);
   if (sockt_fd == -1)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err: socket(), %s. (File: %s, line: %d)\n",
             strerror(errno), __FILE__, __LINE__));
      return IFX_ERROR;
   }

   memset(&ifr, 0, sizeof (struct ifreq));
   strcpy (ifr.ifr_name, "eth0");
   ifr.ifr_addr.sa_family = AF_INET;
   if (ioctl(sockt_fd, SIOCGIFHWADDR, &ifr) == -1)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err with ioctl cmd SIOCGIFHWADDR %s. "
             "(File: %s, line: %d)\n",
             strerror(errno), __FILE__, __LINE__));
      close(sockt_fd);
      return IFX_ERROR;
   }

   memcpy(oCtrlStatus.oTapidemo_MAC_Addr, ifr.ifr_hwaddr.sa_data, ETH_ALEN);

   close(sockt_fd);

#endif /* LINUX */
#else
   SVIP_libStatus_t SVIP_libRet;

   SVIP_libRet = SVIP_libGetAddress("eth0", IFX_NULL,
      oCtrlStatus.oTapidemo_MAC_Addr);
   if (SVIP_libRet != SVIP_libStatusOk)
      return IFX_ERROR;

#endif /* EASY336 */

   return IFX_SUCCESS;
} /* TAPIDEMO_SetMACAddr() */

#if (defined EASY336 || defined XT16)
/**
   Get IP and MAC addresses of veth0 interface of board and set it.

   \return IFX_ERROR if an error, otherwise IFX_SUCCESS (no error occurred)
*/
static IFX_return_t TAPIDEMO_SetVeth0Addr()
{
#ifdef EASY336
   SVIP_libStatus_t SVIP_libRet;
   IFX_char_t pIP[INET_ADDR_LEN];

   SVIP_libRet = SVIP_libGetAddress("veth0", pIP,
      oCtrlStatus.oVeth0_MAC_Addr);
   if (SVIP_libRet != SVIP_libStatusOk)
   {
      return IFX_ERROR;
   }

  if (IFX_SUCCESS != TD_OS_SocketAton(pIP, &oCtrlStatus.oVeth0_IP_Addr))
  {
      return IFX_ERROR;
  }

  TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
        ("veth0 IP-Address: %s, MAC %02X:%02X:%02X:%02X:%02X:%02X\n", pIP,
         oCtrlStatus.oVeth0_MAC_Addr[0],
         oCtrlStatus.oVeth0_MAC_Addr[1],
         oCtrlStatus.oVeth0_MAC_Addr[2],
         oCtrlStatus.oVeth0_MAC_Addr[3],
         oCtrlStatus.oVeth0_MAC_Addr[4],
         oCtrlStatus.oVeth0_MAC_Addr[5]));
#endif /* EASY336 */

   return IFX_SUCCESS;
} /* TAPIDEMO_SetVeth0Addr() */
#endif

/**
   Set default program settings.

   \param pProgramArg - program arguments

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.
*/
IFX_return_t TAPIDEMO_SetDefaultProgramArgs(PROGRAM_ARG_t* pProgramArg)
{
   /* check input parameters */
   TD_PTR_CHECK(pProgramArg, IFX_ERROR);

   /* Trace level is set to high by default */
   pProgramArg->nDbgLevel = DBG_LEVEL_HIGH;
   /* Trace redirection is disabled by default. See also function
      TAPIDEMO_SetDefault(). */
   pProgramArg->nTraceRedirection = TRACE_REDIRECTION_NONE;
#ifdef TD_DECT
   /* default DECT debug level */
   pProgramArg->nDectDbgLevel = DBG_LEVEL_OFF;
#endif /* TD_DECT */

   pProgramArg->nBoardComb = NO_EXT_BOARD;

   /* Voice Activity Detection */
   pProgramArg->nVadCfg = DEFAULT_VAD_CFG;

   /* LEC */
   pProgramArg->nLecCfg = DEFAULT_LEC_CFG;

   /* CID is turned on by default and set to ETSI FSK */
   pProgramArg->oArgFlags.nCID = 1;

   pProgramArg->nCIDStandard = CID_GetStandard();

   /* set default options */
#if (!defined XT16)
   /* Turn conferencing on */
   pProgramArg->oArgFlags.nConference = 1;
#else
   /* enable local PCM loop */
   pProgramArg->oArgFlags.nPCM_Loop = 1;
   pProgramArg->oArgFlags.nPCM_Master = 1;
#endif /* XT16 */

#ifdef EASY336
   /* enable PCM slave */
   pProgramArg->oArgFlags.nPCM_Slave = 1;
#endif

#ifdef DXT
   /* EASY3201 by default uses PCM local loop */
   pProgramArg->oArgFlags.nPCM_Master = 1;
   pProgramArg->oArgFlags.nPCM_Slave = 0;
   pProgramArg->oArgFlags.nPCM_Loop = 1;
#endif /* DXT */
#ifdef TD_DECT
   /* dect support is disabled by default */
   pProgramArg->oArgFlags.nNoDect = 1;
#endif /* TD_DECT */
   /* Default path to download files */
   strncpy(pProgramArg->sPathToDwnldFiles, DEF_DOWNLOAD_PATH, MAX_PATH_LEN - 1);

   /* default encoder and packetisation time */
   pProgramArg->nEnCoderType = TD_DEFAULT_CODER_TYPE;
   pProgramArg->nPacketisationTime = TD_DEFAULT_PACKETISATION_TIME;

   /* karol -workaround for teridian extension board */
   /* grzegorz: workaround for teridian extension board that was originally
                in function TAPIDEMO_SetDefault() is now splitted between two
                functions: TAPIDEMO_SetDefault() (setting
                CTRL_STATUS_t->hUseDuslicFxo) and TAPIDEMO_SetDefaultProgramArgs()
                (setting PROGRAM_ARG_t->hUseDuslicFxo). */
   pProgramArg->hUseDuslicFxo = IFX_TRUE;

   return IFX_SUCCESS;
} /* TAPIDEMO_SetDefaultProgramArgs() */

/**
   Set default program settings.

   \param  pCtrl   - pointer to status control structure
   \param pProgramArg - program arguments

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.
*/
IFX_return_t TAPIDEMO_SetDefault(CTRL_STATUS_t* pCtrl)
{
   /* check input parameters */
   TD_PTR_CHECK(pCtrl, IFX_ERROR);

   /* Trace level is set to high by default */
   pCtrl->pProgramArg->nDbgLevel = DBG_LEVEL_HIGH;
   TD_OS_PRN_USR_LEVEL_SET(TAPIDEMO, pCtrl->pProgramArg->nDbgLevel);


   /* configure program arguments */
   if (IFX_ERROR == TAPIDEMO_SetDefaultProgramArgs(pCtrl->pProgramArg))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Setting program options to default values failed. "
             "File: %s, line: %d)\n",
             __FILE__, __LINE__));
      return IFX_ERROR;
   }
   /* save default setting before reading program command line arguments */
   if (IFX_ERROR == TAPIDEMO_SetDefaultProgramArgs(
                       &pCtrl->oITM_Ctrl.oDefaultProgramArg))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Setting program options to default values failed. "
             "File: %s, line: %d)\n",
             __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* Trace redirection is disabled by default. See also program
      arguments (function TAPIDEMO_SetDefaultProgramArgs()). */
   oTraceRedirection.fStartRedirectLogs = IFX_FALSE;
   oTraceRedirection.pFileLogFD = IFX_NULL;
   oTraceRedirection.nSyslogSocket = NO_SOCKET;

#ifdef TD_FAX_MODEM
   pCtrl->nFaxMode = TD_FAX_MODE_DEAFAULT;
#endif /* TD_FAX_MODEM */

   /* karol -workaround for teridian extension board */
   /* grzegorz: workaround for teridian extension board that was originally
                in function TAPIDEMO_SetDefault() is now splitted between two
                functions: TAPIDEMO_SetDefault() (setting
                CTRL_STATUS_t->hUseDuslicFxo) and TAPIDEMO_SetDefaultProgramArgs()
                (setting PROGRAM_ARG_t->hUseDuslicFxo). */
   pCtrl->hUseDuslicFxo = IFX_TRUE;

   return IFX_SUCCESS;
} /* TAPIDEMO_SetDefault() */


/**
   Set and register the main board.

   The main board is set based on compilation flag '--enable-boardname'. 
   The main board's ID is always 1. 

   \param pBoard - pointer to board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.
*/
IFX_return_t TAPIDEMO_SetMainBoard(BOARD_t* pBoard)
{
   TD_PTR_CHECK(pBoard, IFX_ERROR);

#if (defined(EASY3332) && defined(HAVE_DRV_EASY3332_HEADERS))
   pBoard->nType = TYPE_VINETIC;
   pBoard->nID = 1;
#elif (defined(DANUBE) && defined(HAVE_DRV_VMMC_HEADERS))   
   pBoard->nType = TYPE_DANUBE;
   pBoard->nID = 1;
#elif DXT
   pBoard->nType = TYPE_DUSLIC_XT;
   pBoard->nID = 1;
#elif (defined(VINAX) && defined(HAVE_DRV_VMMC_HEADERS))
   pBoard->nType = TYPE_VINAX;
   pBoard->nID = 1;
#elif (defined(AR9) && defined(HAVE_DRV_VMMC_HEADERS))
   pBoard->nType = TYPE_AR9;
   pBoard->nID = 1;
#elif (defined(VR9) && defined(HAVE_DRV_VMMC_HEADERS))
   pBoard->nType = TYPE_VR9;
   pBoard->nID = 1;
#elif EASY336
   pBoard->nType = TYPE_SVIP ;
   pBoard->nID = 1;
#elif XT16
   pBoard->nType = TYPE_XT16;
   pBoard->nID = 1;
#else
   /* Unknown board type or miss driver's headers.*/
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error: Unknown board type. (File: %s, line: %d)\n",
             __FILE__, __LINE__));
   return IFX_ERROR;
#endif /* EASY3332 */

   if (IFX_SUCCESS != Common_RegisterBoard(pBoard))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error: The main board registration failed. (File: %s, line: %d)\n",
             __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* Increase number of boards */
   oCtrlStatus.nBoardCnt++;

   return IFX_SUCCESS;
} /* TAPIDEMO_SetMainBoard */

/**
   Check board combination.

   This function is used only by obsolete argument '-b'. It checks if user's
   board combination is supported based on compilation flags.

   \param pBoard - pointer to extension board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.
*/
IFX_return_t TAPIDEMO_CheckBoardCombination(BOARD_t* pBoard)
{
   IFX_return_t ret = IFX_ERROR;
   /* Set PCM mode must be checked */
   IFX_boolean_t hCheckPcmIfMode = IFX_FALSE;
   IFX_int32_t nID = -1;

   TD_PTR_CHECK(oCtrlStatus.pProgramArg, IFX_ERROR);
   TD_PTR_CHECK(pBoard, IFX_ERROR);

   switch (oCtrlStatus.pProgramArg->nBoardComb)
   {
      case ONE_CPE:
      {
         /* TYPE_VINETIC and NO_EXT_BOARD has got the same value '0'.
            Tapidemo'll never enter this section. However, it's correct because,
            ONE_CPE means single board, so Tapidemo doesn't check for extension board */
         if (TYPE_VINETIC == oCtrlStatus.rgoBoards[0].nType) 
         {
            return IFX_SUCCESS;
         }
      }
      case ONE_VMMC:
         {
         if (TYPE_DANUBE == oCtrlStatus.rgoBoards[0].nType) 
            {
            return IFX_SUCCESS;
         }
         break;
            }
      case ONE_VMMC_ONE_CPE:
         {
            if (TYPE_DANUBE == oCtrlStatus.rgoBoards[0].nType) 
            {
#if (defined(HAVE_DRV_VINETIC_HEADERS) && !defined(EASY3332))
               nID = BOARD_Easy50510_DetectNode(&oCtrlStatus);
               if (NO_EXT_BOARD_DETECT != nID) 
               {
                  /* An extesion board has been detected. */ 
                  pBoard->nType = TYPE_VINETIC;
                  pBoard->nID = nID;
                  hCheckPcmIfMode = IFX_TRUE;
                  ret = IFX_SUCCESS;
               }
               else
               {
                  TAPIDEMO_PRINTF(("Error: Missing device node for board EASY50510.\n"
                                   "       Please check if driver is loaded.\n"));
               }
#else
            TAPIDEMO_PRINTF(("Error: During compilation, require driver "
                             "       VINETIC-CPE wasn't found.\n"
                             "       Please recompile with missing driver.\n"));   
#endif /* HAVE_DRV_VINETIC_HEADERS */
            }
            break;
         }
      case ONE_DXT:
         {
            if (TYPE_DUSLIC_XT == oCtrlStatus.rgoBoards[0].nType) 
            {
               return IFX_SUCCESS;            
            }
         }
      case TWO_DXTS:
         {
            if (TYPE_DUSLIC_XT == oCtrlStatus.rgoBoards[0].nType) 
            {
#ifdef DXT
               nID = BOARD_Easy3111_DetectNode(&oCtrlStatus);
               if (NO_EXT_BOARD_DETECT != nID) 
               {
                  /* An extesion board has been detected. */
                  pBoard->nType = TYPE_DUSLIC_XT;
                  pBoard->nID = nID;
                  ret = IFX_SUCCESS;
               }
               else
               {
                  TAPIDEMO_PRINTF(("Error: Missing device node for board EASY3111.\n"));
               }
#endif /* DXT */               
            }
            break;
         }
      case ONE_VMMC_VINAX:
         {
            if (TYPE_VINAX == oCtrlStatus.rgoBoards[0].nType) 
            {
               return IFX_SUCCESS;
            }
            break;
         }
      case ONE_VMMC_EASY508XX:
         {
            if (TYPE_AR9 == oCtrlStatus.rgoBoards[0].nType) 
            {
               return IFX_SUCCESS;
            }
            break;
         }
      case ONE_EASY80910:
         {
            if (TYPE_VR9 == oCtrlStatus.rgoBoards[0].nType) 
            {
               return IFX_SUCCESS;
            }
            break;
         }
      case X_SVIP:
         {
            if (TYPE_SVIP == oCtrlStatus.rgoBoards[0].nType) 
            {
               return IFX_SUCCESS;
            }
            break;
         }
      case X_SVIP_X_XT16:
         {
            if (TYPE_SVIP == oCtrlStatus.rgoBoards[0].nType) 
            {
               /* Extension board XT16 is detected by svipbox.
                  Check if TAPIDEMO was compiled with support for that board */
#ifdef WITH_VXT
               /* An extesion board has been detected. */
               pBoard->nType = TYPE_XT16;
               pBoard->nID = 1;
               ret = IFX_SUCCESS;
#else
               TAPIDEMO_PRINTF(("Error: TAPIDEMO is compiled without support for XT16.\n"
                             "       Please use compilation flag --with-vxt.\n"));               
#endif /* WITH_VXT */
            }
            break;
         }
      case X_XT16:
         {
            if (TYPE_XT16 == oCtrlStatus.rgoBoards[0].nType) 
            {
               return IFX_SUCCESS;
            }
            break;
         }
      case BOARD_COMB_RESERVED:
      default:
         {
            TAPIDEMO_PRINTF(("Unknown board combination.\nFile: %s, line: %d)\n",
                             __FILE__, __LINE__));
            return IFX_ERROR;
          }
   } /* switch */

   /* To prevent compilation warning */
   nID = -1;

   if (IFX_SUCCESS != ret)
   {
      TAPIDEMO_PRINTF(
         ("User board combination: %s is unsupported.\n"
          "Compilation for: %s.\nFile: %s, line: %d)\n", 
          BOARD_COMB_NAMES[oCtrlStatus.pProgramArg->nBoardComb],
          oCtrlStatus.rgoBoards[0].pszBoardName,
          __FILE__, __LINE__));
      return IFX_ERROR;
   }
   else
   {
      /* Register extension board */
      if (IFX_SUCCESS != Common_RegisterBoard(pBoard))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Error: The extension board registration failed.\n"
                "(File: %s, line: %d)\n", __FILE__, __LINE__));
         return IFX_ERROR;
      }

      /* Increase number of boards */
      oCtrlStatus.nBoardCnt++;

      /* Set PCM mode */
      if (hCheckPcmIfMode)
      {
         if (oCtrlStatus.pProgramArg->oArgFlags.nPCM_Slave)
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Main board - the PCM i/f can not be configured as a SLAVE when \n"
                "extension board is used.\n"
                "PCM i/f will be set as MASTER\n"));
            oCtrlStatus.pProgramArg->oArgFlags.nPCM_Slave = 0;
         }
         oCtrlStatus.pProgramArg->oArgFlags.nPCM_Master = 1;
      }  /* if */
   }

   return IFX_SUCCESS;
} /* TAPIDEMO_CheckBoardCombination() */

/**
   Set and register extension board.

   Check which extension boards are avaiable. It's done by checking 
   if require device node exists (e.g. for EASY50510 needs vin1x) 
   in directory /dev/. Device node is created during board's start-up, 
   regardless if extension board is connected to main board or not.
 
   \param pBoard - pointer to board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.
*/
IFX_return_t TAPIDEMO_SetExtBoard(BOARD_t* pBoard)
{
   IFX_return_t ret = IFX_ERROR;
   /* Set PCM mode must be checked */
   IFX_boolean_t hCheckPcmIfMode = IFX_FALSE;
   IFX_int32_t nID = -1;

   TD_PTR_CHECK(pBoard, IFX_ERROR);

   do {
#ifdef DXT
      nID = BOARD_Easy3111_DetectNode(&oCtrlStatus);
      if (NO_EXT_BOARD_DETECT != nID) 
      {
         /* An extesion board has been detected. 
            Check if Tapidemo doesn't detect the same board twice time. */
         if (IFX_TRUE != TAPIDEMO_ExtBoardCheckForDuplicate(TYPE_DUSLIC_XT, nID))
         {
            pBoard->nType = TYPE_DUSLIC_XT;
            pBoard->nID = nID;
            ret = IFX_SUCCESS;
            break;
         }
      }
#endif /* DXT */

#if (defined(HAVE_DRV_VINETIC_HEADERS) && !defined(EASY3332))
      nID = BOARD_Easy50510_DetectNode(&oCtrlStatus);
      if (NO_EXT_BOARD_DETECT != nID) 
      {
         /* An extesion board has been detected. 
            Check if Tapidemo doesn't detect the same board twice time. */
         if (IFX_TRUE != TAPIDEMO_ExtBoardCheckForDuplicate(TYPE_VINETIC, nID))
         {
            hCheckPcmIfMode = IFX_TRUE;
            pBoard->nType = TYPE_VINETIC;
            pBoard->nID = nID;
            ret = IFX_SUCCESS;
            break;
         }
      } 
#endif /* HAVE_DRV_VINETIC_HEADERS */

#ifdef WITH_VXT
      /* Extension board XT16 is detected by svipbox.
         Check if TAPIDEMO was compiled with support for that board */
      if (IFX_TRUE != TAPIDEMO_ExtBoardCheckForDuplicate(TYPE_XT16, 1))
      {
         pBoard->nType = TYPE_XT16;
         pBoard->nID = 1;
         ret = IFX_SUCCESS;
         break;
      }
#endif /* WITH_VXT */
   } while (0);

   /* To prevent compilation warning */
   nID = -1;

   if (IFX_SUCCESS == ret) 
   {

      if (IFX_SUCCESS != Common_RegisterBoard(pBoard))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Error: Board registration failed. (File: %s, line: %d)\n",
                __FILE__, __LINE__));
         return IFX_ERROR;
      }

      /* Increase number of boards */
      oCtrlStatus.nBoardCnt++;

      /* Set PCM mode */
      if (hCheckPcmIfMode)
      {
         if (oCtrlStatus.pProgramArg->oArgFlags.nPCM_Slave)
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Main board - the PCM i/f can not be configured as a SLAVE when \n"
                "extension board is used.\n"
                "PCM i/f will be set as MASTER\n"));
            oCtrlStatus.pProgramArg->oArgFlags.nPCM_Slave = 0;
         }
         oCtrlStatus.pProgramArg->oArgFlags.nPCM_Master = 1;
      }  /* hCheckPcmIfMode */
   } /* ret */

   return ret;
} /* TAPIDEMO_SetExtBoard */

/**
   Check if extension board has been detected before.

   Search all registered extension boards for detected board by comparing 
   board type and ID.

   \param nType - type
   \param nType - ID

   \return IFX_TRUE if board is already detected, otherwise IFX_FALSE.
*/
IFX_boolean_t TAPIDEMO_ExtBoardCheckForDuplicate(IFX_int32_t nType, 
                                                 IFX_int32_t nID)
{
   IFX_int32_t i;
   /* Check if detected board has been alread set */ 
   for(i = 1; i < oCtrlStatus.nBoardCnt; i++)
   {
      if ((nType == oCtrlStatus.rgoBoards[i].nType) &&
          (nID == oCtrlStatus.rgoBoards[i].nID))
      {
         /* This board has been already set */
         return IFX_TRUE;
      }
   } /* for */
   return IFX_FALSE;
} /* TAPIDEMO_CheckIfExtBoardDetected */

/**
   Start board.

   Initilize all registered board. Firstly the main board is processed.
   Next, all extension board are processed. If extension board isn't 
   connected to the main board, its initialization is stopped and trace is printed.
   Tapidemo shouldn't stop working, when extension board's initialization failed.

   \param  nIndex - index of board which is started

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.
*/
IFX_return_t TAPIDEMO_StartBoard(IFX_int32_t nIndex)
{
   BOARD_t* pBoard = IFX_NULL;

   TD_PARAMETER_CHECK((0 > nIndex), nIndex, IFX_ERROR);

   pBoard = &oCtrlStatus.rgoBoards[nIndex];

   pBoard->nBoard_IDX = nIndex;
   pBoard->pCtrl = &oCtrlStatus;

   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
         ("----- Start board %s initialization. -----\n",
          pBoard->pszBoardName));

   /* Init board */
   if (IFX_SUCCESS != pBoard->Init(pBoard, &oProgramArg))
   {
      if (nIndex == 0) 
      {
         /* The main board is always initialized firstly. */ 
         /* Main board initialization failed. */
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error: The main board initialization failed (Idx %d).\n(File: %s, line: %d)\n",
            pBoard->nBoard_IDX, __FILE__, __LINE__));
         /* Cleanup previous stuff */
#ifdef VXWORKS
         abort();
#else /* VXWORKS */
         TAPIDEMO_CleanUp(SIGABRT);
#endif /* VXWORKS */
         return IFX_ERROR;
      }
      else
      {
         /* The extension board initialization failed. It might happen when
         extension board's driver is installed and nodes are created but device 
         is not connected to main board. */

         TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
               ("-----      Board %s not initialized.     -----\n",
                pBoard->pszBoardName));

         /* Make clean in board array */
         if (IFX_SUCCESS != TAPIDEMO_RebuildBoardArray(nIndex))
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Error: Rebuilding board array failed.\n(File: %s, line: %d)\n",
                __FILE__, __LINE__));
            /* Cleanup previous stuff */
#ifdef VXWORKS
            abort();
#else /* VXWORKS */
            TAPIDEMO_CleanUp(SIGABRT);
#endif /* VXWORKS */
            return IFX_ERROR;
         }
         return IFX_SUCCESS;
      } /*  if */
   }

   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
         ("-----      Board %s initialized.     -----\n",
          pBoard->pszBoardName));

   /* Save sum of all channels */
   oCtrlStatus.nSumCoderCh += pBoard->nMaxCoderCh;
   oCtrlStatus.nSumPhones += pBoard->nMaxPhones;
   oCtrlStatus.nSumPCM_Ch += pBoard->nMaxPCM_Ch;

   return IFX_SUCCESS;
} /* TAPIDEMO_StartBoard */

/**
   Rebuild board array.

   Initialization of extension board failed. All registered boards 
   are kept in one array. Tapidemo must rebuild this array. 

   \param  nIndex - index of board which is failed

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.
*/
IFX_return_t TAPIDEMO_RebuildBoardArray(IFX_int32_t nIndex)
{
   /* Pointer on board */
   BOARD_t* pBoard = IFX_NULL;
   BOARD_t* pNextBoard = IFX_NULL;
   IFX_int32_t i;

   TD_PARAMETER_CHECK((0 >= nIndex), nIndex, IFX_ERROR);

   /* Remove board which initilization failed */

   /* Get extension board which initialization failed */
   pBoard = &oCtrlStatus.rgoBoards[nIndex];
   /* Release board and free memory. */
   pBoard->RemoveBoard(pBoard);
   memset(pBoard, 0, sizeof (BOARD_t));
   pBoard = IFX_NULL;

   /* If board was on last position in array, don't do anything more */
   if (nIndex == (oCtrlStatus.nBoardCnt-1)) 
   {
      oCtrlStatus.nBoardCnt--;
      return IFX_SUCCESS;
   }

   /* If board wasn't on last position in array, we need to rebuild array */

   /* Build new array of boards without board which initilization failed */
   for (i=nIndex; i < oCtrlStatus.nBoardCnt-1; i++)
   {
      /* Pointer on new position in array */
      pBoard = &oCtrlStatus.rgoBoards[i];
      /* Pointer on old position in array */
      pNextBoard = &oCtrlStatus.rgoBoards[i+1];

      /* Copy elements to new position */
      pBoard->nType = pNextBoard->nType;
      pBoard->nID = pNextBoard->nID;
      pBoard->pszBoardName = pNextBoard->pszBoardName;
      pBoard->Init = pNextBoard->Init;
      pBoard->RemoveBoard = pNextBoard->RemoveBoard;
   } /* for */

   /* Remove last position in array, because it has been copied to new position */
   pBoard = &oCtrlStatus.rgoBoards[oCtrlStatus.nBoardCnt-1];
   /* Release board and free memory. */
   memset(pBoard, 0, sizeof (BOARD_t));
   pBoard = IFX_NULL;
   oCtrlStatus.nBoardCnt--;

   return IFX_SUCCESS;
} /* TAPIDEMO_RebuildBoardArray */

#ifdef FXO
/**
   Check timeouts for FXO call

   Check if timeout for FXO call elapsed. If yes, perform the appropriate action.
   If not, adjust timeout value.

   \param  pCtrl - pointer to status control structure
   \param pTimeOut   - pointer to current timeout for select

   \return status (IFX_SUCCESS or IFX_ERROR)
*/
IFX_return_t TAPIDEMO_CheckTimeoutFxoCall(CTRL_STATUS_t* pCtrl, IFX_time_t* pTimeOut)
{
   BOARD_t* pBoard = IFX_NULL;
   PHONE_t* pPhone = IFX_NULL;
   IFX_int32_t nBoard;
   FXO_t* pFXO = IFX_NULL;
   IFX_int32_t nPhone;
   IFX_int32_t nFxoNum = 0;
   BOARD_t* pRingBoard = IFX_NULL;
   PHONE_t oDummyPhone;
   IFX_boolean_t fFxoEnd = IFX_FALSE;
   IFX_time_t nDiffMSec;
   IFX_time_t time_new;

   TD_PTR_CHECK(pCtrl, IFX_ERROR);

   /* for all boards */
   for(nBoard = 0; nBoard < pCtrl->nBoardCnt; nBoard++)
   {
      /* get board */
      pBoard = &pCtrl->rgoBoards[nBoard];
      /* check if FXO is enabled */
      if (pCtrl->pProgramArg->oArgFlags.nFXO)
      {
         if (0 != nBoard)
         {
            /* check FXO only for first board */
         }
         else
         {
            /* for all fxo channels */
            for (nFxoNum = 1; nFxoNum <= pCtrl->nMaxFxoNumber; nFxoNum++)
            {
               /* get fxo of fxo number */
               pFXO = ABSTRACT_GetFxo(nFxoNum, pCtrl);
               if (pFXO == IFX_NULL)
               {
                  TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                        ("FXO not found for fxo channel %d. "
                         "(File: %s, line: %d)\n",
                         nFxoNum, __FILE__, __LINE__));
                  break;
               }
               /* if ringing on FXO channel */
               if (S_IN_RINGING == pFXO->nState)
               {
                  /* Check if timeout for ringing event - it means
                     caller phone went ONHOOK and ringing must be stopped on
                     all the called phones. */;
                  nDiffMSec = TD_OS_ElapsedTimeMSecGet(pFXO->timeRingEventTimeout);
                  /* check timeout */
                  if (nDiffMSec > TIMEOUT_FOR_ONHOOK)
                  {
                     /* if timeout occured then FXO call has ended */
                     TRACE(TAPIDEMO, DBG_LEVEL_LOW,
                           ("Timeout diff %lu sec for ringing elapsed\n"
                            "on fxo No %d, stop it on all called phones.\n",
                            nDiffMSec, nFxoNum));
                     /* set board that should be ringing */
                     pRingBoard = pBoard;
                     /* check if phones are present on this board */
                     if ((pBoard->nMaxAnalogCh == 0) && (pBoard->nBoard_IDX > 0))
                     {
                        /* This is the extension board without phone channels.
                           Stop ringing on the evaluation board. */
                        pRingBoard = &pBoard->pCtrl->rgoBoards[0];
                     }
                     fFxoEnd = IFX_FALSE;
                     /* for all phones on ring board */
                     for (nPhone=0; nPhone < pRingBoard->nMaxPhones; nPhone++)
                     {
                        /* get phone */
                        pPhone = &pRingBoard->rgoPhones[nPhone];
                        /* check if phone ringing caused by fxo call */
                        if ((pPhone->rgoConn[0].nType == FXO_CALL) &&
                            (pPhone->rgStateMachine[FXS_SM].nState == S_IN_RINGING))
                        {
                           fFxoEnd = IFX_TRUE;
                           /* fxo call ended - stop ringing */
                           pPhone->nIntEvent_FXS = IE_READY;

                           while (pPhone->nIntEvent_FXS != IE_NONE)
                           {
                              ST_HandleState_FXS(pCtrl, pPhone, &pPhone->rgoConn[0]);
                           }
                        } /* if ringing uring FXO call*/
                     } /*for all phones on ring board */
                     /* check if any phone was ringing */
                     if (IFX_TRUE != fFxoEnd)
                     {
                        FXO_EndConnection(pCtrl->pProgramArg, pFXO,
                                          &oDummyPhone, oDummyPhone.rgoConn,
                                          pBoard);
                     }
                  } /* if (time_diff > TIMEOUT_FOR_ONHOOK) */
                  else
                  {
                     /* Check if timeout should be smaller than it is currently set */
                     time_new = TIMEOUT_FOR_ONHOOK - nDiffMSec;
                     if (time_new < *pTimeOut) 
                     {
                        *pTimeOut = time_new;
                     }
                  }
               } /* if (S_IN_RINGING == pFXO->fRingStopOccured) */
            } /* for all FXOs on board */\
         } /* if 0 != nBoard */
      } /* if (oCtrlStatus.pProgramArg->oArgFlags.nFXO) */
   } /* for all boards */
   return IFX_SUCCESS;
} /* TAPIDEMO_CheckTimeoutFxoCall */
#endif /* FXO */

/**
   Check timeouts for external call

   Check if timeout for external call elapsed. 
   If yes, perform the appropriate action.
   If not, adjust timeout value.

   \param  pCtrl   - pointer to status control structure
   \param pTimeOut   - pointer to current timeout for select

   \return status (IFX_SUCCESS or IFX_ERROR)
*/
IFX_return_t TAPIDEMO_CheckTimeoutExtCall(CTRL_STATUS_t* pCtrl, IFX_time_t* pTimeOut)
{
   BOARD_t* pBoard = IFX_NULL;
   PHONE_t* pPhone = IFX_NULL;
   CONNECTION_t* pConn = IFX_NULL;
   IFX_int32_t nBoard, nPhone, nConn;
   IFX_time_t nDiffMSec;
   IFX_time_t time_new;

   TD_PTR_CHECK(pCtrl, IFX_ERROR);

   /* for all boards */
   for(nBoard = 0; nBoard < pCtrl->nBoardCnt; nBoard++)
   {
      /* get board */
      pBoard = &pCtrl->rgoBoards[nBoard];
      /* for all phones on board */
      for (nPhone=0; nPhone < pBoard->nMaxPhones; nPhone++)
      {
         pPhone = &pBoard->rgoPhones[nPhone];
         /* for all connections */
         for (nConn=0; nConn < pPhone->nConnCnt; nConn++)
         {
            pConn = &pPhone->rgoConn[nConn];
            /* check connection (must be external call in dialing) */
            if ((pConn->nType == EXTERN_VOIP_CALL) && 
                (pConn->bExtDialingIn == IFX_TRUE))
            {
               nDiffMSec = TD_OS_ElapsedTimeMSecGet(pConn->timeStartExtCalling);
               if (nDiffMSec >= TIMEOUT_FOR_EXTERNAL_CALL)
               {
                  TRACE(TAPIDEMO, DBG_LEVEL_LOW,
                        ("Phone No %d: Timeout diff %lu msec for external connection\n"
                         "%selapsed, start to play busy tone.\n",
                         pPhone->nPhoneNumber, nDiffMSec,
                         pPhone->pBoard->pIndentPhone));

                  /* check phone state */
                  if ((pPhone->rgStateMachine[FXS_SM].nState == S_OUT_DIALING) ||
                      (pPhone->rgStateMachine[FXS_SM].nState == S_CF_DIALING))
                  {
                     pPhone->nIntEvent_FXS = IE_EXT_CALL_NO_ANSWER;

                     while (pPhone->nIntEvent_FXS != IE_NONE)
                     {
                        ST_HandleState_FXS(pCtrl, pPhone, pConn);
                     }
                  } /* if */
                  else
                  {
                     TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                           ("Error, Phone No %d: Detected incorrect event for state: %s\n",
                            (IFX_int32_t) pPhone->nPhoneNumber,
                            Common_Enum2Name(pPhone->rgStateMachine[FXS_SM].nState,
                                             TD_rgStateName)));
                  } /* check phone state */
               } /* check time difference */
               else
               {
                  /* Check if timeout should be smaller than it is currently set */
                  time_new = TIMEOUT_FOR_EXTERNAL_CALL - nDiffMSec;
                  if (time_new < *pTimeOut) 
                  {
                     *pTimeOut = time_new;
                  }
               }
            } /* check connection - if (pConn->... == ...) */
         }/* for all connections */
      }/* for all phones on board */
   } /* for all boards */
   return IFX_SUCCESS;
} /* TAPIDEMO_CheckTimeoutExtCall */

/**
   State machine for PBX - handle events from devices

   \param  pCtrl - pointer to status control structure

   \return status (IFX_SUCCESS or IFX_ERROR)
*/
static IFX_int32_t TAPIDEMO_HandleDeviceThread(CTRL_STATUS_t* pCtrl)
{
   IFX_return_t ret = IFX_SUCCESS;
   BOARD_t* pBoard = IFX_NULL;
   PHONE_t* pPhone = IFX_NULL;
   CONNECTION_t* pConn = IFX_NULL;
   IFX_int32_t nBoard;
   FXO_t* pFXO = IFX_NULL;
   IFX_boolean_t bIsFxo = IFX_FALSE;
   TD_OS_socFd_set_t rfds, trfds;
   /* Wait forever by default */
   IFX_time_t nTimeOut = TD_OS_WAIT_FOREVER;

#if (!defined EASY336 && !defined TAPI_VERSION4)
   IFX_int32_t nDataCh;
   IFX_int32_t nDataChFd;
#endif /* (!defined EASY336 && !defined TAPI_VERSION4) */

#if !defined(STREAM_1_1) && !defined(VXWORKS) && defined(FXO)
   IFX_int32_t nFxoDevCtrl_FD;
   TAPIDEMO_DEVICE_FXO_t* pFxoDevice;
#endif /* !defined(STREAM_1_1) && !defined(VXWORKS) && defined(FXO) */

   /* check input parameters */
   TD_PTR_CHECK(pCtrl, IFX_ERROR);
 
   /*****************************
      Prepare structures
    *****************************/
   pCtrl->bInternalEventGenerated = IFX_FALSE;
   /* do not use timeout untill it is needed */
   TD_TIMEOUT_RESET(pCtrl->nUseTimeoutDevice);

   /********************************************************
         Sets the set of file descriptors for select
    ********************************************************/
   if (IFX_ERROR == TAPIDEMO_SetDeviceFdSet(pCtrl, &rfds))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error: Failed to set fd_set for select.on devices.\n"
             "(File: %s, line: %d)\n", __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /********************************************************
         Wait for events and handle them
    ********************************************************/
   TD_OS_MutexGet(&pCtrl->oMultithreadCtrl.oMutex.mutexStateMachine);
   while (1)
   {
      /* Check if timeout should be used */
      if (TD_TIMEOUT_CHECK(pCtrl->nUseTimeoutSocket))
      {
         /* There are two threads which are waiting for events from diffrent 
            type of sources. By default, they are waiting indefinitely on events.
            It might happen that select on device may receive event which
            requires to activate timeout for select on sockets (e.g. during dialing
            external call). In such situation, the second thread must be waked-up.
            It is done by sending message via pipe. */
         TD_OS_PipePrintf(pCtrl->oMultithreadCtrl.oPipe.oSocketSynchPipe.rgPipeFp[TD_PIPE_IN], 
                          "%d", TD_SYNCH_THREAD_MSG_SET_TIMEOUT);
      }

      if (TD_TIMEOUT_CHECK(pCtrl->nUseTimeoutDevice))
      {
         /* Time out used for FXO calls */
         if ((nTimeOut == 0) || (nTimeOut == IFXOS_WAIT_FOREVER)) 
         {
            nTimeOut = SELECT_TIMEOUT_MSEC;
         }
         else
         {
#ifdef FXO
/* start                check timeout for FXO ringing                   start */
            TAPIDEMO_CheckTimeoutFxoCall(pCtrl, &nTimeOut);
/*   end                check timeout for FXO ringing                   end   */
#endif /* FXO */
         }
      }
      else
      {
         /* Select should wait without timeout */
         nTimeOut = TD_OS_SOC_WAIT_FOREVER;
      }
      /* Release mutex and wait on new events */
      TD_OS_MutexRelease(&pCtrl->oMultithreadCtrl.oMutex.mutexStateMachine);

      /* Update the local file descriptor by the copy in the task parameter */
      memcpy((void *) &trfds, (void*) &rfds, sizeof(fd_set));
      /* Wait for event or timeout */
      ret = TD_OS_DeviceSelect(pCtrl->nMaxFdDevice + 1, &trfds, &trfds, nTimeOut);
      TD_OS_MutexGet(&pCtrl->oMultithreadCtrl.oMutex.mutexStateMachine);
      /* Check if select waked up by event */
      if (0 < ret)
      {
         /* for all boards */
         for(nBoard = 0; nBoard < pCtrl->nBoardCnt; nBoard++)
         {
            /* get board */
            pBoard = &pCtrl->rgoBoards[nBoard];
/* start             Handling tapi events on FXS                        start */
            /* if pBoard->nDevCtrl_FD not set skip this part without error */
            if (0 < pBoard->nDevCtrl_FD)
            {
               /* check device file descriptor */
               if (TD_OS_DevFdIsSet(pBoard->nDevCtrl_FD, &trfds))
               {
                  /* get event */
                  if (IFX_ERROR == EVENT_Check(pBoard->nDevCtrl_FD, pBoard))
                  {
                     /* no event for this device */
                     TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                           ("Error, Board %s failed to get event. "
                            "(File: %s, line: %d)\n",
                            pBoard->pszBoardName, __FILE__, __LINE__));
                     continue;
                  }
                  else
                  {
                     /* Get event owner. Checking if event is dedicated for phone
                        or for fxo. There are some situations that data channel
                        is used to detect CPT for fxo. In such situation event
                        is dedicated for fxo */
                     if (IFX_ERROR == EVENT_GetOwnerOfEvent(pBoard, pBoard->nDevCtrl_FD,
                                         &pPhone, &pConn, &bIsFxo, &pFXO))
                     {
                        /* failed to get owner or event doesn't need handling */
                        /*TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                              ("Error, Failed to get event owner."
                               " (File: %s, line: %d)\n",
                               __FILE__, __LINE__));*/
                         break;
                     }
                     else
                     {
                        /* event for phone channel */
                        if (IFX_FALSE == bIsFxo)
                        {
                           /* check if pointers are set */
                           if ((IFX_NULL == pPhone) || (IFX_NULL == pConn))
                           {
                              TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                                    ("Error, Invalid data: phone(%d) and connection(%d)."
                                     " (File: %s, line: %d)\n", 
                                     (IFX_int32_t)pPhone,
                                     (IFX_int32_t)pConn, __FILE__, __LINE__));
                              break;
                           }
                           else
                           {
                              /* handle detected event */
                              EVENT_Handle(pPhone, pCtrl, pBoard);
                              /* if internal event generated then handle it
                                 (both config and FXS state machine ) */
                              while (IE_NONE != pPhone->nIntEvent_Config)
                              {
                                 ST_HandleState_Config(pCtrl, pPhone, pConn);
                              }
                              while (IE_NONE != pPhone->nIntEvent_FXS)
                              {
                                 ST_HandleState_FXS(pCtrl, pPhone, pConn);
                              }
                           }
                        } /* IFX_FALSE == bIsFxo */
                        else if (IFX_TRUE == bIsFxo)
                        {
                           if (IFX_NULL == pFXO)
                           {
                              TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                                    ("Error, Failed to get fxo."
                                     " (File: %s, line: %d)\n", __FILE__, __LINE__));
                              break;
                           }
#ifdef SLIC121_FXO
                           if ((pFXO->pFxoDevice->oTapiEventOnFxo.id & 
                                IFX_TAPI_EVENT_TYPE_MASK) ==
                                IFX_TAPI_EVENT_TYPE_FXO)
                           {
                              /* fxo event detected on fxo line in SLIC121. */
                              EVENT_FXO_Handle(pFXO, pCtrl, pBoard);
                           }
                           else
#endif /* SLIC121_FXO */
                           {
                              /* handle event on data channel connected to FXO */
                              EVENT_Handle_DataCH(pFXO->nDataCh, IFX_NULL, 
                                                  pBoard, pFXO);
                           }
                        } /* IFX_TRUE == bIsFxo */
                        else
                        {
                           TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                                 ("Error, Unpredicted behaviour of EVENT_GetOwnerOfEvent()."
                                  " (File: %s, line: %d)\n", __FILE__, __LINE__));
                        }
                        /* check if internal event generated */
                        if (IFX_TRUE == pCtrl->bInternalEventGenerated)
                        {
                           EVENT_InternalEventHandle(pCtrl);
                        }
                     } /* EVENT_GetOwnerOfEvent */
                  } /* EVENT_Check */
               } /* fd is set */
            } /* if pBoard->nDevCtrl_FD is used for this board */
         }/* for all boards */
/*   end             Handling tapi events on FXS                        end   */
/* start             Handling tapi events on FXO                        start */
#if !defined(STREAM_1_1) && !defined(VXWORKS) && defined(FXO)
         /* FXO events handling - check if FXO enabled */
         if (pCtrl->pProgramArg->oArgFlags.nFXO)
         {
            /* get first board */
            nBoard = 0;
            pBoard = &pCtrl->rgoBoards[nBoard];
            /* check number of FXOs */
            if (0 < pCtrl->nMaxFxoNumber)
            {
               /* reset pointer */
               pFxoDevice = IFX_NULL;
               do
               {
                  /* get fxo device fd */
                  nFxoDevCtrl_FD =  Common_GetNextFxoDevCtrlFd(pCtrl, 
                                                               &pFxoDevice,
                                                               IFX_TRUE);
                  /* not all boards have FXO device,
                     but no error should be printed because of that */
                  if (0 < nFxoDevCtrl_FD)
                  {
                     /* check if fd is set */
                     if (TD_OS_DevFdIsSet(nFxoDevCtrl_FD, &trfds))
                     {
                        /* get FXO event */
                        ret = EVENT_Check(nFxoDevCtrl_FD, pBoard);
                        if (IFX_ERROR == ret)
                        {
                           /* no event for this device */
                           TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                                 ("Error, Board %s failed to get event. "
                                  "(File: %s, line: %d)\n",
                                  pBoard->pszBoardName, __FILE__, __LINE__));
                           continue;
                        }
                        else
                        {
                           /* get owner of event, here can be only fxo */
                           if (IFX_ERROR == EVENT_GetOwnerOfEvent(pBoard, nFxoDevCtrl_FD,
                                               &pPhone, &pConn, &bIsFxo, &pFXO))
                           {
                              /* failed to get fxo or event doesn't need handling */
                              /*TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                                    ("Error, Failed to get event owner."
                                     " (File: %s, line: %d)\n",
                                     __FILE__, __LINE__));*/
                              continue;
                           }
                           else
                           {
                              if (IFX_TRUE != bIsFxo)
                              {
                                 TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                                       ("Error, Not a FXO event. (File: %s, line: %d)\n",
                                        __FILE__, __LINE__));
                                 continue;
                              }
                              else
                              {
                                 if (IFX_NULL == pFXO)
                                 {
                                    TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                                          ("Error, Failed to Get FXO. (File: %s, line: %d)\n",
                                           __FILE__, __LINE__));
                                    continue;
                                 }
                                 else
                                 {
                                    /* handle FXO event */
                                    if (IFX_SUCCESS == EVENT_FXO_Handle(pFXO, pCtrl, pBoard))
                                    {
                                       /* EVENT_FXO_Handle() can generate internal events on
                                          phones, check if such events are generated
                                          and handle them */
                                       if (IFX_TRUE == pCtrl->bInternalEventGenerated)
                                       {
                                          EVENT_InternalEventHandle(pCtrl);
                                       }
                                    } /* EVENT_FXO_Handle */
                                 } /* IFX_NULL == pFXO */
                              } /* bIsFxo */
                           } /* EVENT_GetOwnerOfEvent */
                        } /* EVENT_Check */
                     } /* nFxoDevCtrl_FD is set */
                  } /* if (0 < nFxoDevCtrl_FD) */
               }  while (0 < nFxoDevCtrl_FD);
               /* check return value, see Common_GetNextFxoDevCtrlFd description
                  for details */
               if (IFX_ERROR == nFxoDevCtrl_FD && IFX_NULL == pFxoDevice)
               {
                  TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                        ("Err, Number of FXOs %d but no FXO device FD was found."
                         " (File: %s, line: %d)\n",
                         pCtrl->nMaxFxoNumber, __FILE__, __LINE__));
               }
               else if (IFX_ERROR == nFxoDevCtrl_FD)
               {
                  TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                        ("Err, Geting FXO device FD failed. (File: %s, line: %d)\n",
                         __FILE__, __LINE__));
               }
            } /* if (0 < pCtrl->nMaxFxoNumber) */
         } /* if (oCtrlStatus.pProgramArg->oArgFlags.nFXO) */
#endif /* !defined(STREAM_1_1) && !defined(VXWORKS) && defined(FXO) */
/*   end             Handling tapi events on FXO                        end   */
#ifdef TD_DECT
/* start               DECT event handling                              start */
         /* check if DECT support is on */
         if (!pCtrl->pProgramArg->oArgFlags.nNoDect)
         {
            /* check for timer events */
            if (TD_OS_DevFdIsSet(pCtrl->nTimerFd, &trfds))
            {
               IFX_TLIB_CurrTimerExpiry(0); 
            }
            /* check for paging key events */
            if (TD_OS_DevFdIsSet(pCtrl->viPagingKeyFd, &trfds))
            {
               TD_DECT_HandlePaging(pCtrl->viPagingKeyFd); 
            }
         } /* if (!pCtrl->pProgramArg->oArgFlags.nNoDect) */
/*   end               DECT event handling                              end   */
#endif /* TD_DECT */
/* start      HANDLING voip packets on data channels                    start */
/* packet handling must after event handling otherwise there are errors
   from drivers eg. INFO: Buffer pool growth limit reached */
         /* for all boards */
         for(nBoard = 0; nBoard < pCtrl->nBoardCnt; nBoard++)
         {
            /* get board */
            pBoard = &pCtrl->rgoBoards[nBoard];
#ifdef TAPI_VERSION4
            /* check if single fd is used */
            if (!pBoard->fSingleFD)
#endif
            {
#if (!defined EASY336 && !defined TAPI_VERSION4)
               /* for all data channels */
               for (nDataCh = 0; nDataCh < pBoard->nMaxCoderCh; nDataCh++)
               {
                  /* HANDLING PACKETS FROM DATA CHANNELS */
                  /* get data channel fd to check for voice packets */
                  nDataChFd = VOIP_GetFD_OfCh(nDataCh, pBoard);
                  /* check if valid data chennel fd was returned */
                  if (NO_DEVICE_FD == nDataChFd)
                  {
                     TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                           ("Error, Board %s Failed to get fd for "
                            "data channel %d. (File: %s, line: %d)\n",
                            pBoard->pszBoardName, nDataCh,
                            __FILE__, __LINE__));
                     break;
                  }
                  else
                  {
                  /* check if packets can be read from data channel  */
                     if (TD_OS_DevFdIsSet(nDataChFd, &trfds))
                     {
                        /* get connection that uses this data channel */
                        if (IFX_SUCCESS == ABSTRACT_GetConnOfDataCh(&pPhone, &pConn,
                                                                    pBoard, nDataCh))
                        {
                           /* We received data from phone */
                           VOIP_HandleData(pCtrl, pPhone, nDataChFd, pConn,
                                           QOS_HandleService(pCtrl, pConn));
                        }
                        else
                        {
                         /*  TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                                 ("Error, Unable to handle packets on Data channel"
                                  "%d - discard data. (File: %s, line: %d)\n",
                                  nDataCh, __FILE__, __LINE__)); */
                           /* because connection is not stopped at the same time on
                              both phones, packet from ended connection can
                              be still received and must be discarded */
                           VOIP_DiscardData(nDataChFd);
                        }
                     } /* check if data channel is readable*/
                  } /* NO_DEVICE_FD == nDataChFd */
               } /* for all data channels */
#endif /* (!defined EASY336 && !defined TAPI_VERSION4) */
            } /* if (!pBoard->fSingleFD) */
/*   end      HANDLING voip packets on data channels and sockets        end   */
         }/* for all boards */
      } /* if (0 < ret)*/
      /* check if select waked up by timeot */
      else if (0 == ret)
      {
#ifdef FXO
         TAPIDEMO_CheckTimeoutFxoCall(pCtrl, &nTimeOut);
#endif /* FXO */
      } /* ret of select() */
      /* if ret < 0 select returned error */
      else if (0 > ret)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Error in select for devices. (File: %s, line: %d)\n",
              __FILE__, __LINE__));
         break;
      }
      
   } /* while */

   /* We leave loop, so we don't need keep mutex anymore */
   TD_OS_MutexRelease(&pCtrl->oMultithreadCtrl.oMutex.mutexStateMachine);

   return IFX_SUCCESS;
} /* TAPIDEMO_HandleDeviceThread() */

/**
   State machine for PBX - handle events from sockets

   \param  pThread - pointer to thread control structure

   \return status (IFX_SUCCESS or IFX_ERROR)
*/
static IFX_int32_t TAPIDEMO_HandleSocketThread(TD_OS_ThreadParams_t *pThread)
{
   IFX_return_t ret = IFX_SUCCESS;
   BOARD_t* pBoard = IFX_NULL;
   CTRL_STATUS_t* pCtrl;
   IFX_int32_t nBoard;
   TD_OS_socFd_set_t rfds, trfds;
   /* Wait forever by default */
   IFX_time_t nTimeOut = TD_OS_WAIT_FOREVER;
   /* Read data from thread synch pipe */
   IFX_char_t pBuffer[2] = {0};
   TD_SYNCH_THREAD_MSG_t oSychThreadMsg;

#if (!defined EASY336 && !defined TAPI_VERSION4)
   CONNECTION_t* pConn = IFX_NULL;
   PHONE_t* pPhone = IFX_NULL;
   IFX_int32_t nDataCh;
   IFX_int32_t nSocket = NO_SOCKET;
#endif /* (!defined EASY336 && !defined TAPI_VERSION4) */

   pCtrl = &oCtrlStatus;

   /* Block all signals on this thread */
   if (IFX_SUCCESS != TAPIDEMO_BlockSignals())
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
            ("Error: Can't block signals for  socket thread.\n"
             "(File: %s, line: %d)\n", __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /*****************************
      Prepare structures
    *****************************/
   pCtrl->bInternalEventGenerated = IFX_FALSE;
   /* do not use timeout untill it is needed */
   TD_TIMEOUT_RESET(pCtrl->nUseTimeoutSocket);

   /********************************************************
         Sets the set of file descriptors for select
    ********************************************************/
   if (IFX_SUCCESS != TAPIDEMO_SetSocketFdSet(pCtrl, &rfds))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error: Failed to set fd_set for select on sockets.\n"
             "(File: %s, line: %d)\n", __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* This lock is used for synchronization with the main thread and it's used
      when Tapidemo is closed down. 
      When lock is released it means thread stops working. */
   TD_OS_LockGet(&pCtrl->oMultithreadCtrl.oLock.oSocketThreadStopLock);

   /********************************************************
         Wait for events and handle them
    ********************************************************/
   TD_OS_MutexGet(&pCtrl->oMultithreadCtrl.oMutex.mutexStateMachine);
   while (1)
   {
      /* check if timeout should be used */
      if (TD_TIMEOUT_CHECK(pCtrl->nUseTimeoutSocket))
      {
         /* set time out used for external call dialing (for dialing timeout). */
         if ((nTimeOut == 0) || (nTimeOut == TD_OS_WAIT_FOREVER)) 
         {
            /* Timeout is not set. Set default one */
            nTimeOut = SELECT_TIMEOUT_MSEC;
         }
         else
         {
            /* Check if timeout is set correctly */
            TAPIDEMO_CheckTimeoutExtCall(pCtrl, &nTimeOut);
         }
#ifdef TD_DECT
         if (!pCtrl->pProgramArg->oArgFlags.nNoDect)
         {
            TD_DECT_DialtoneTimeoutSelectSet(pCtrl, &nTimeOut);
         } /* if (!pCtrl->pProgramArg->oArgFlags.nNoDect) */
#endif /* TD_DECT */
      }
      else
      {
         /* select should wait without timeout */
         nTimeOut = TD_OS_SOC_WAIT_FOREVER;
      }

      /* if request to change IP address via test scripts */
      COM_COMMUNICATION_SOCKETS_RESET_IF_IP_CHANGED(pCtrl, rfds);

      /* Release mutex and wait on new events */
      TD_OS_MutexRelease(&pCtrl->oMultithreadCtrl.oMutex.mutexStateMachine);

      /* Update the local file descriptor by the copy in the task parameter */
      memcpy((void *) &trfds, (void*) &rfds, sizeof(fd_set));
      /* Wait for event or timeout */
      ret = TD_OS_SocketSelect(pCtrl->nMaxFdSocket + 1, &trfds, IFX_NULL, IFX_NULL, 
                               nTimeOut);      
      TD_OS_MutexGet(&pCtrl->oMultithreadCtrl.oMutex.mutexStateMachine);
      /* check if select waked up by event */
      if (0 < ret)
      {
/* start         HANDLING events from administration sockets            start */
         /* Check socket for handling calls between boards */
         if (TD_OS_SocFdIsSet(pCtrl->nAdminSocket, &trfds))
         {
            /* Got something on admin socket
               (actions sent by TAPIDEMO_SetAction()) */
            TRACE(TAPIDEMO, DBG_LEVEL_LOW, ("New action on ADMIN socket\n"));
            TAPIDEMO_HandleCallSeq(pCtrl);
            /* check if internal event generated */
            if (IFX_TRUE == pCtrl->bInternalEventGenerated)
            {
               EVENT_InternalEventHandle(pCtrl);
            }
         }
/*   end         HANDLING events from administration socket             end   */
/* start         HANDLING events from thread synch pipe                 start */
         if (TD_OS_SocFdIsSet(
            pCtrl->oMultithreadCtrl.oPipe.oSocketSynchPipe.rgPipeFd[TD_PIPE_OUT], 
            &trfds))
         {
            /* Clean up buffer */
            memset(&pBuffer, 0, sizeof(pBuffer)); 
            ret = TD_OS_PipeRead(pBuffer, sizeof(pBuffer)-1, 1, 
                                 pCtrl->oMultithreadCtrl.oPipe.oSocketSynchPipe.rgPipeFp[TD_PIPE_OUT]);
            if (Common_IsNumber(pBuffer))
            {
               oSychThreadMsg = atoi(pBuffer);
               if (TD_SYNCH_THREAD_MSG_SET_TIMEOUT == oSychThreadMsg) 
               {
                  /* In order to set new timeout for this thread, 
                     we need to wake-up thread. It is done by receiving message from
                     another thread. The thread is wake-up, so we don't need to do
                     anything more */
               }
               else if (TD_SYNCH_THREAD_MSG_STOP == oSychThreadMsg)
               {
                  /* Receive msg from another thread to stop working. Exit thread */
                  TRACE(TAPIDEMO, DBG_LEVEL_LOW,
                        ("Receive message from synchronization pipe: stop thread.\n"));
                  break;
               }
            }
            else
            {
               TRACE(TAPIDEMO, DBG_LEVEL_LOW,
                     ("Receive unknown message from synchronization pipe: %s\n", 
                     pBuffer));
               break;
            }
         }
/*   end         HANDLING events from thread synch pipe                 end   */

/* start       HANDLING events from ITM communication socket            start */
         /* ITM message handling e.g. choosing test */
         if (IFX_TRUE == g_pITM_Ctrl->fITM_Enabled)
         {
            /* check if socket available */
            if(NO_SOCKET != g_pITM_Ctrl->nComInSocket)
            {
               /* Check if there is a command pending on communication socket */
               if (TD_OS_SocFdIsSet(g_pITM_Ctrl->nComInSocket, &trfds))
               {
                  /* Receive command */
                  TRACE(TAPIDEMO, DBG_LEVEL_LOW,
                        ("ITM: Command on communication socket.\n"));
                  COM_ReceiveCommand(pCtrl);
               }
            } /* if(NO_SOCKET != pCtrl->nComInSocket) */
         }
/*   end       HANDLING events from ITM communication socket            end   */
#ifdef TD_FAX_MODEM
#if (defined(TD_T38_FAX_TEST) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW))
         /* Add communication socket for handling PC's requests */
         if(NO_SOCKET != pCtrl->oFaxTestCtrl.nComSocket)
         {
            /* Check if there is a command pending on communication socket */
            if (TD_OS_SocFdIsSet(pCtrl->oFaxTestCtrl.nComSocket, &trfds))
            {
               /* Receive command */
               /*TRACE(TAPIDEMO, DBG_LEVEL_LOW,
                     ("Command on fax test communication socket.\n"));*/
               TD_T38_FAX_TEST_HandleSocketData(pCtrl);
            }
         }
#endif /* (defined(TD_T38_FAX_TEST) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW)) */
#endif /* TD_FAX_MODEM */

/* start      HANDLING voip packets on sockets        start */
         /* for all boards */
         for(nBoard = 0; nBoard < pCtrl->nBoardCnt; nBoard++)
         {
            /* get board */
            pBoard = &pCtrl->rgoBoards[nBoard];
#ifdef TAPI_VERSION4
            /* check if single fd is used */
            if (!pBoard->fSingleFD)
#endif
            {
#if (!defined EASY336 && !defined TAPI_VERSION4)
               /* for all data channels */
               for (nDataCh = 0; nDataCh < pBoard->nMaxCoderCh; nDataCh++)
               {
                  /* HANDLING PACKETS FROM SOCKETS */
                  /* check if QoS is used */
                  if (!pCtrl->pProgramArg->oArgFlags.nQos)
                  {
                     /* get socket */
                     nSocket = VOIP_GetSocket(nDataCh, pBoard);
                     /* check if valid data chennel fd was returned */
                     if (NO_SOCKET == nSocket)
                     {
                        TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                              ("Error, Board %s Failed to get socket for "
                               "data channel %d. (File: %s, line: %d)\n",
                               pBoard->pszBoardName, nDataCh,
                               __FILE__, __LINE__));
                     }
                     /* check if packets can be read from socket */
                     if (TD_OS_SocFdIsSet(nSocket, &trfds))
                     {
                        pPhone = ABSTRACT_GetPHONE_OfSocket(nSocket, &pConn, pBoard);
                        if (IFX_NULL == pPhone || IFX_NULL == pConn)
                        {
                         /*  TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                                 ("Error, %s not found for socket %d. "
                                  "(File: %s, line: %d)\n",
                                  IFX_NULL == pPhone ? "Phone" : "Connection",
                                  nSocket, __FILE__, __LINE__));*/
                           /* because connection is not stopped at the same time
                              on both phones, packet from ended connection can
                              be still received and must be discarded */
                           VOIP_DiscardData(nSocket);
                        }
                        else
                        {
                           /* We received data over socket, external phone */
                           VOIP_HandleSocketData(pPhone, pConn,
                                                 QOS_HandleService(pCtrl, pConn));
                        }
                     } /* check if socket is readable*/
                  } /* check if QoS is used */
               } /* for all data channels */
#endif /* (!defined EASY336 && !defined TAPI_VERSION4) */
            } /* if (!pBoard->fSingleFD) */
         } /* for all boards */
/*   end      HANDLING voip packets on data channels and sockets        end   */

#ifdef TD_DECT
/* start               DECT event handling                              start */
         /* check if DECT support is on */
         if (!pCtrl->pProgramArg->oArgFlags.nNoDect)
         {
            /* check if message received */
            if (TD_OS_SocFdIsSet(pCtrl->nDectPipeFdOut, &trfds))
            {
               pPhone = IFX_NULL;
               pConn = IFX_NULL;
               /* check if there is an event to handle */
               while (IFX_SUCCESS == TD_DECT_EventHandle(
                       pCtrl->oMultithreadCtrl.oPipe.oDectPipe.rgPipeFp[TD_PIPE_OUT],
                       &pCtrl->rgoBoards[0], &pPhone, &pConn))
               {
                  while (IE_NONE != pPhone->nIntEvent_Config)
                  {
                     ST_HandleState_Config(pCtrl, pPhone, pConn);
                  }
                  while (IE_NONE != pPhone->nIntEvent_FXS)
                  {
                     ST_HandleState_FXS(pCtrl, pPhone, pConn);
                  }
                  if (IFX_TRUE == pCtrl->bInternalEventGenerated)
                  {
                     EVENT_InternalEventHandle(pCtrl);
                  }
               }
            }
            /* check for message on DECT debug module socket */
            if (TD_OS_SocFdIsSet(pCtrl->viFromCliFd, &trfds))
            {
               TD_DECT_HandleCliCmd(pCtrl->viFromCliFd, pCtrl); 
            }
         } /* if (!pCtrl->pProgramArg->oArgFlags.nNoDect) */
/*   end               DECT event handling                              end   */
#endif /* TD_DECT */
      } /* if (0 < ret)*/
      /* check if select waked up by timeot */
      else if (0 == ret)
      {
/* start            check timeout for external call dialing             start */
         TAPIDEMO_CheckTimeoutExtCall(pCtrl, &nTimeOut);
/* stop             check timeout for external call dialing              stop */
#ifdef TD_DECT
/* start                check timeout for DECT dialtone                 start */
      /* check if DECT is enabled */
      if (!pCtrl->pProgramArg->oArgFlags.nNoDect)
      {
         /* check for timeout for DECT phones and handle it if needed */
         if (IFX_SUCCESS != 
             TD_DECT_DialtoneTimeoutSelectSet(pCtrl, &nTimeOut))
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                  ("Err, TD_DECT_DialtoneTimeoutSelectSet() failed.\n"
                   "(File: %s, line: %d)\n",
                   __FILE__, __LINE__));
         }
      } /* if (!pCtrl->pProgramArg->oArgFlags.nNoDect) */
/* end                  check timeout for DECT dialtone                   end */
#endif /* TD_DECT */
      }
      /* if ret < 0 select returned error */
      else if (0 > ret)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Error in select for sockets. (File: %s, line: %d)\n",
              __FILE__, __LINE__));
         break;
      }
   } /* while(1) */

   /* We leave loop, so we don't need keep mutex anymore */
   TD_OS_MutexRelease(&pCtrl->oMultithreadCtrl.oMutex.mutexStateMachine);

   /* Release lock, which means this thread stops working */
   TD_OS_LockRelease(&pCtrl->oMultithreadCtrl.oLock.oSocketThreadStopLock);

   return IFX_SUCCESS;
} /* TAPIDEMO_HandleSocketThread */

/**
   Run logs' redirection - read data from pipe and redirect them.

   \param  pThread - pointer to thread control structure
 
   \return status (IFX_SUCCESS or IFX_ERROR)
*/
static IFX_int32_t TAPIDEMO_HandleLogThread(TD_OS_ThreadParams_t *pThread)
{
   /* Read data from pipe and redirect them. */
   IFX_char_t pBuffer[MAX_LOG_LEN] = {0};
   IFX_int32_t nMaxFdSocket = 0;

   /* Read data from thread synch pipe */
   TD_SYNCH_THREAD_MSG_t oSychThreadMsg;

   CTRL_STATUS_t* pCtrl;
   IFX_return_t ret = IFX_SUCCESS;
   fd_set rfds, trfds;

   pCtrl = &oCtrlStatus;

   /* Block all signals on this thread */
   if (IFX_SUCCESS != TAPIDEMO_BlockSignals())
   {
       TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
             ("Error: Can't block signals for redirect log thread.\n"
              "(File: %s, line: %d)\n", __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* This lock is used for synchronization with the main thread and it's used
      when Tapidemo is closed down. 
      When lock is released it means thread stops working. */
   TD_OS_LockGet(&pCtrl->oMultithreadCtrl.oLock.oLogThreadStopLock);

   /********************************************************
         Sets the set of file descriptors for select
    ********************************************************/
   TD_OS_SocFdZero(&rfds);
   if (NO_PIPE == pCtrl->oMultithreadCtrl.oPipe.oLogSynchPipe.rgPipeFd[TD_PIPE_OUT])
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error, Couldn't get thread synch pipe.\n"
             "(File: %s, line: %d)\n",
             __FILE__, __LINE__));
      return IFX_ERROR;
   }
   TD_FD_SET(pCtrl->oMultithreadCtrl.oPipe.oLogSynchPipe.rgPipeFd[TD_PIPE_OUT], &rfds, 
             nMaxFdSocket, TD_SELECT_SOCKET);

   if (NO_PIPE == pCtrl->oMultithreadCtrl.oPipe.oRedirectLogPipe.rgPipeFd[TD_PIPE_OUT])
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error, Couldn't get redirect log pipe.\n"
             "(File: %s, line: %d)\n",
             __FILE__, __LINE__));
      return IFX_ERROR;
   }
   TD_FD_SET(pCtrl->oMultithreadCtrl.oPipe.oRedirectLogPipe.rgPipeFd[TD_PIPE_OUT], &rfds, 
             nMaxFdSocket, TD_SELECT_SOCKET);

   /********************************************************
         Wait for logs and handle them
    ********************************************************/
   while(1)
   {
      /* Update the local file descriptor by the copy in the task parameter */
      memcpy((void *) &trfds, (void*) &rfds, sizeof(fd_set));
      ret = TD_OS_SocketSelect(nMaxFdSocket+1, &trfds, IFX_NULL, IFX_NULL, 
                               TD_OS_WAIT_FOREVER);
      if (ret > 0)
      {
/* start         HANDLING events from thread synch pipe                 start */
         if (TD_OS_SocFdIsSet(
            pCtrl->oMultithreadCtrl.oPipe.oLogSynchPipe.rgPipeFd[TD_PIPE_OUT], 
            &trfds))
         {
            TD_OS_PipeRead(pBuffer, MAX_LOG_LEN - 1, 1, 
                           pCtrl->oMultithreadCtrl.oPipe.oLogSynchPipe.rgPipeFp[TD_PIPE_OUT]);
            if (Common_IsNumber(pBuffer))
            {
               oSychThreadMsg = atoi(pBuffer);
               if (TD_SYNCH_THREAD_MSG_SET_TIMEOUT == oSychThreadMsg) 
               {
                  /* In order to set new timeout for this thread, 
                     we need to wake-up thread. It is done by receiving message from
                     another thread. The thread is wake-up, so we don't need to do
                     anything more */
               }
               else if (TD_SYNCH_THREAD_MSG_STOP == oSychThreadMsg)
               {
                  /* Receive msg from another thread to stop working. Exit thread */
                  break;
               }
            }
            else
            {
               /* Receive unknown message from synchronization pipe */
               break;
            }
         }
/*   end         HANDLING events from thread synch pipe                 end   */
/* start                       HANDLING logs                            start */
         if (TD_OS_SocFdIsSet(
            pCtrl->oMultithreadCtrl.oPipe.oRedirectLogPipe.rgPipeFd[TD_PIPE_OUT], 
            &trfds))
         {
            TD_OS_PipeRead(pBuffer, MAX_LOG_LEN - 1, 1, 
                           pCtrl->oMultithreadCtrl.oPipe.oRedirectLogPipe.rgPipeFp[TD_PIPE_OUT]);
            /* Check how many bytes were read */
            if (0 >= strlen(pBuffer))
            {
               TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                     ("Message read error. (File: %s, line: %d)\n",
                      __FILE__, __LINE__));
               break;
            }
            else if (MAX_LOG_LEN < strlen(pBuffer))
            {
               TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                     ("Message too long. (File: %s, line: %d)\n",
                      __FILE__, __LINE__));
            }
            else
            {
               /* Split and send log to new destination */
               TAPIDEMO_SplitAndSendLog(pBuffer);
            } /* if (0 > bytes_read) */
         } /* if(TD_OS_SocFdIsSet(nPipeFd, &fds)) */
         /* Clean up buffer */
         memset(&pBuffer, 0, MAX_LOG_LEN);
/* stop                        HANDLING logs                            stop */
      }
      else if (0 == ret)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Timeout on select for logs (File: %s, line: %d)\n",
              __FILE__, __LINE__));
      }
      else
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Error in select for logs. (File: %s, line: %d)\n",
                __FILE__, __LINE__));
         break;
      } /* if (ret > 0) */
   } /* while(1) */

   /* Release lock, which means this thread stops working */
   TD_OS_LockRelease(&pCtrl->oMultithreadCtrl.oLock.oLogThreadStopLock);
                           
   /* End with task */
   return IFX_SUCCESS;
} /* TAPIDEMO_HandleLogThread */

/**
   Main function.

   \param argc - number of arguments
   \param argv - array of arguments

   \return 0 or err code
*/
#if defined(LINUX)

int main(int argc, char *argv [])
{

#else /* defined(LINUX) */

IFX_int32_t tapidemo(char* cmd_args)
{
   IFX_int32_t argc = 0;
   IFX_char_t** argv = IFX_NULL;

#endif /* defined(LINUX) */

   IFX_return_t ret = IFX_SUCCESS;
   IFX_int32_t i = 0;
   IFX_int32_t j = 0;
   IFX_int32_t phone_num = 0;
   IFX_int32_t tmp_coder_sum = 0;
   IFX_boolean_t check_arg = IFX_FALSE;
   BOARD_t* pBoard = IFX_NULL;
   IFX_int32_t nPreviousBoardCnt;

   /* Reset parameters */
   memset((IFX_uint8_t *) &oProgramArg, 0, sizeof(PROGRAM_ARG_t));
   /* Reset control status */
   memset(&oCtrlStatus, 0, sizeof(CTRL_STATUS_t));
   /* Set all variables and all used structures to zero. */
   memset(&oTraceRedirection, 0, sizeof(TRACE_REDIRECTION_t));

   /* set pointer to program arguments structure */
   oCtrlStatus.pProgramArg = &oProgramArg;

#if defined(VXWORKS)
   /* Set application priority level */
   taskPrioritySet((int) IFX_NULL, (int) 40);

   /* Get TID of application */
   nTapiDemo_TID = taskIdSelf();

   /* Read arguments and setup application accordingly */
   if ((cmd_args != IFX_NULL) && (strlen(cmd_args) > 0))
   {
      argc = GetArgCnt(cmd_args, GetCmdArgsArr());
      argv = GetCmdArgsArr();
      check_arg = IFX_TRUE;
   }
#elif LINUX
   if (argc > 1)
   {
      check_arg = IFX_TRUE;
   }
#endif /* VXWORKS */

   /* set ITM default */
   COM_SetDefault(&oCtrlStatus.oITM_Ctrl, argc, argv,
                  TAPIDEMO_VERSION, ITM_VERSION);

   /* Set some default arguments */
   if (IFX_SUCCESS != TAPIDEMO_SetDefault(&oCtrlStatus))
   {
      /* Can't set default configuration */
      TAPIDEMO_PRINTF(("Setting default configuration failed.\n"
                       "File: %s, line: %d\n",
                       __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (check_arg)
   {
      if (ReadOptions(argc, argv, oCtrlStatus.pProgramArg) != IFX_SUCCESS)
      {
         /* Wrong arguments */
         TAPIDEMO_PRINTF((" Wrong command line argument(s).\n"));
         return IFX_ERROR;
      }
      /* karol - workaround for teridian extension board */
      oCtrlStatus.hUseDuslicFxo = oProgramArg.hUseDuslicFxo;
      /* end workaround */
   }

/* ------ Start Prepare trace redirection and run application as a daemon ------- */
#ifdef LINUX
   if (oProgramArg.nTraceRedirection != TRACE_REDIRECTION_NONE)
   {
      /* Set up log redirection - e.g. sockets, syslog or open file */
      if (IFX_SUCCESS != TAPIDEMO_PreapareLogRedirection())
      {
         /* Can't start redirecting logs */
         TAPIDEMO_PRINTF((" Can not start redirecting logs.\n"));
         return IFX_ERROR;
      }
   } /* trace redirection */

   if (oProgramArg.oArgFlags.nDaemon)
   {
      TAPIDEMO_PRINTF(("....Tapidemo runs as a daemon....\n\n"));
      /* Create process responsible for log redirection and TAPIDEMO daemon */
      if (TAPIDEMO_CreateDaemon(&oCtrlStatus) != IFX_SUCCESS)
      {
         /* Can't create daamon */
         TAPIDEMO_PRINTF((" Can not run Tapidemo as a daemon.\n"));
         return IFX_ERROR;
      }
   } /* daemon */
#endif /* defined(LINUX) */
/* ------ Stop Prepare trace redirection and run application as a daemon ------- */

/* ------ Start Handle program argument --help, --version --configure ------- */
   if (oCtrlStatus.pProgramArg->oArgFlags.nHelp)
   {
      return ProgramHelp();
   } /* program help */
   TAPIDEMO_PRINTF(("TAPIDEMO version %s\n", TAPIDEMO_VERSION));
   TAPIDEMO_PRINTF(("ITM version %s.\n", ITM_VERSION));

#ifdef TD_FAX_MODEM
#if (defined(TD_T38_FAX_TEST) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW))
   /* FAX_TEST version */
   TD_T38_FAX_TEST_PrintVersion();
#endif /* (defined(TD_T38_FAX_TEST) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW)) */
#endif /* TD_FAX_MODEM */
   /* check if version command line argument was used */
   if (oCtrlStatus.pProgramArg->oArgFlags.nVersion)
   {
      /* end program after printing version of application */
      return IFX_SUCCESS;
   } /* program version */
/* -------------------- Stop Print application's versions ------------------- */
#ifdef LINUX
   if (oCtrlStatus.pProgramArg->oArgFlags.nConfigure)
   {
      TAPIDEMO_PRINTF(("TAPIDEMO configuration:\n %s\n", TAPIDEMO_CONFIGURE_STR));
      return IFX_SUCCESS;
   } /* program configuration */
#endif /* LINUX */
/* ------ Stop Handle program argument --help, --version --configure -------- */

/* ----- Start Detect and register the main board and extension board(s) ---- */

   /* Set main board */
   if (IFX_SUCCESS != TAPIDEMO_SetMainBoard(&oCtrlStatus.rgoBoards[0]))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error: Set main board. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* Check if Tapidemo should detect any extension board */
   if (oCtrlStatus.pProgramArg->oArgFlags.nUseCombinationBoard) 
   {
      /* Program argument '-b' is used. It might be used in two situation:
         - Run Tapidemo only for the main board. In this case, 
           disable detection of extension boards. Use: '-b 0'
         - Sets board combination - obsolete feature. In this case, 
           don't autodetect all extension boards. Detect specific 
           extension board based on board combination. Use: -b num */

      if (NO_EXT_BOARD != oCtrlStatus.pProgramArg->nBoardComb)
      {
         /* Second situation: '-b num' (where num > 0) is used 
            to set board combination */ 
         /* Check backward compatibility for flag -b */
         if (IFX_SUCCESS != TAPIDEMO_CheckBoardCombination(&oCtrlStatus.rgoBoards[1]))
         {
            return IFX_ERROR;
         }
      } /* if */
   }
   else
   {
      /* Autodetect all extension boards. */
      /* Max number of extension board is limited by constant MAX_BOARDS */
      for (i=1; i < MAX_BOARDS; i++) 
      {
         if (IFX_SUCCESS != TAPIDEMO_SetExtBoard(&oCtrlStatus.rgoBoards[i]))
         {
            break;
         }
      } /* for */
   } /* if */
/* ----- Stop Detect and register the main board and extension board(s) ----- */

/* ---------- Start Print information about used compilation flag ----------- */
   TAPIDEMO_PRINTF(("Compiled for %s", oCtrlStatus.rgoBoards[0].pszBoardName));
#ifdef FXO
   /* print: "with FXO(device_list)" */
   TAPIDEMO_PRINTF((" with FXO ("));

#ifdef DUSLIC_FXO
   TAPIDEMO_PRINTF(("%s", DUSLIC_FXO_NAME));
#endif /* DUSLIC_FXO */
#if defined(DUSLIC_FXO) && defined(TERIDIAN_FXO)
   TAPIDEMO_PRINTF((" and "));
#endif /* defined(DUSLIC_FXO) && defined(TERIDIAN_FXO) */
#ifdef TERIDIAN_FXO
   TAPIDEMO_PRINTF(("%s", TERIDIAN_FXO_NAME));
#endif /* TERIDIAN_FXO */
#if defined(SLIC121_FXO) && defined(TERIDIAN_FXO)
   TAPIDEMO_PRINTF((" and "));
#endif /* defined(SLIC121_FXO) && defined(TERIDIAN_FXO) */
#ifdef SLIC121_FXO
   TAPIDEMO_PRINTF(("%s", SLIC121_FXO_NAME));
#endif /* SLIC121_FXO */
   TAPIDEMO_PRINTF((")"));
#endif /* FXO */

#if defined(FXO) && defined(TD_DECT)
   TAPIDEMO_PRINTF((" and"));
#endif /* defined(FXO) && defined(TD_DECT) */
#ifdef TD_DECT
   /* dect support */
   TAPIDEMO_PRINTF((" with DECT"));
#endif /* TD_DECT */
   TAPIDEMO_PRINTF((".\n"));
/* ----------- Stop Print information about used compilation flag ----------- */

   /* Set default IP-Address  */
   TAPIDEMO_SetDefaultAddr(oCtrlStatus.pProgramArg);

   /* Register signal handler */
   if (IFX_SUCCESS != TAPIDEMO_RegisterSigHandler())
   {
      /* Could not register signal handler */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Could not register signal handler. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* initialize arrary with pointers to codecs names */
   VOIP_SetCodecNames();

   /* Turn on services */
   TAPIDEMO_RunServices(&oCtrlStatus);

   /* Set MAC-Address  */
   TAPIDEMO_SetMACAddr();

   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
         ("Using network interface: %s, %02X:%02X:%02X:%02X:%02X:%02X.\n",
          inet_ntoa(oProgramArg.oMy_IP_Addr.sin_addr),
          oCtrlStatus.oTapidemo_MAC_Addr[0],
          oCtrlStatus.oTapidemo_MAC_Addr[1],
          oCtrlStatus.oTapidemo_MAC_Addr[2],
          oCtrlStatus.oTapidemo_MAC_Addr[3],
          oCtrlStatus.oTapidemo_MAC_Addr[4],
          oCtrlStatus.oTapidemo_MAC_Addr[5]));

#if (defined EASY336 || defined XT16)
   /* get IP and MAC addresses of veth0 interface */
   TAPIDEMO_SetVeth0Addr();
#endif /* (defined EASY336 || defined XT16) */

   COM_COMMUNICATION_SOCKET_INIT(oProgramArg.oMy_IP_Addr);

#ifdef TD_FAX_MODEM
#if (defined(TD_T38_FAX_TEST) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW))
   /* Initialize FAX T.38 test */
   TD_T38_FAX_TEST_Init(&oCtrlStatus);
#endif /* (defined(TD_T38_FAX_TEST) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW)) */
#endif /* TD_FAX_MODEM */

   /* Start all boards */
   for (i = 0; i < oCtrlStatus.nBoardCnt; i++)
   { 
      /* Keep number of boards before initialization */
      nPreviousBoardCnt = oCtrlStatus.nBoardCnt; 
      if ( IFX_SUCCESS != TAPIDEMO_StartBoard(i))
      {
         return IFX_ERROR;
      }
      /* Check if any board was removed from array. 
         Number of board is decreased when extension board initialization failed.
         E.g. when extension board isn't connected to the main board. */
      if (nPreviousBoardCnt > oCtrlStatus.nBoardCnt)
      {
         /* One of the board has been removed from array, so we need to fix iteration */
         i--;
      }
   } /* for */

/* -------------- Start Print information about using settings -------------- */
   TAPIDEMO_PRINTF(("Using board %s with:\n", 
                   oCtrlStatus.rgoBoards[0].pszBoardName));
   if (oCtrlStatus.nBoardCnt > 1)
   {
     for (i=1; i < oCtrlStatus.nBoardCnt; i++) 
     {
        TAPIDEMO_PRINTF(("- Extension board: %s.\n", 
                         oCtrlStatus.rgoBoards[i].pszBoardName));
     }
   }
   PrintUsedOptions(&oProgramArg);
/* --------------- Stop Print information about using settings -------------- */

   /* end verify system initialization test */
   COM_MOD_VERIFY_SYSTEM_INIT_SEND_DONE(0);

   /* Prepare socket for making/ending calls. */
   oCtrlStatus.nAdminSocket = TAPIDEMO_InitAdminSocket();

   if (oCtrlStatus.nSumCoderCh > 0)
   {
      /* Prepare UDP sockets for VoIP external calls */
      oCtrlStatus.rgnSockets = calloc(oCtrlStatus.nSumCoderCh,
                                      sizeof(SOCKET_STAT_t));
      if (oCtrlStatus.rgnSockets == IFX_NULL)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("ERR allocate memory for UDP sockets. (File: %s, line: %d)\n",
               __FILE__, __LINE__));
         /* Cleanup previous stuff */
#ifdef VXWORKS
         abort();
#else /* VXWORKS */
         TAPIDEMO_CleanUp(SIGABRT);
#endif /* VXWORKS */
      }

      if (!oCtrlStatus.pProgramArg->oArgFlags.nQos)
      {
         IFX_int32_t k = 0;

         tmp_coder_sum = 0;
         j = 0;
         pBoard = &oCtrlStatus.rgoBoards[j];
         tmp_coder_sum = pBoard->nMaxCoderCh;
         for (i = 0; i < oCtrlStatus.nSumCoderCh; i++, k++)
         {
            /* Also initialize sockets */
#ifdef TAPI_VERSION4
            if (!pBoard->fUseSockets)
            {
               oCtrlStatus.rgnSockets[i].nSocket = NO_SOCKET;
            }
            else /* if (!pBoard->fUseSockets) */
#endif /* TAPI_VERSION4 */
            {
               oCtrlStatus.rgnSockets[i].nSocket =
                  VOIP_InitUdpSocket(oCtrlStatus.pProgramArg, i,
                                     &oCtrlStatus.rgnSockets[i].nPort);
               if (NO_SOCKET == oCtrlStatus.rgnSockets[i].nSocket)
               {
                  /* Error initializing VoIP sockets */
                  /* Cleanup previous stuff */
   #ifdef VXWORKS
                  abort();
   #else /* VXWORKS */
                  TAPIDEMO_CleanUp(SIGABRT);
   #endif /* VXWORKS */
                  return IFX_ERROR;
               }

                /* Connect this sockets for each board, data ch of this board is
                  index in socket array. */
               if (tmp_coder_sum <= i)
               {
                  j++;
                  pBoard = &oCtrlStatus.rgoBoards[j];
                  tmp_coder_sum += pBoard->nMaxCoderCh;
                  k = 0;
               }
               pBoard->rgnSockets[k] = oCtrlStatus.rgnSockets[i].nSocket;
            } /* if (!pBoard->fUseSockets) */
         } /* for */
      } /* if (!oCtrlStatus.pProgramArg->oArgFlags.nQos) */
   } /* if (oCtrlStatus.nSumCoderCh > 0) */

   /* Holds called phone number, which will be increased in *_Setup() */
#ifndef TAPI_VERSION4
   phone_num = 1;
#endif

   /* If support for multiple boards then initialize all boards. */
   for (i = 0; i < oCtrlStatus.nBoardCnt; i++)
   {
      /* Prepare objects, structure */
      if (IFX_ERROR == TAPIDEMO_Setup(&oCtrlStatus.rgoBoards[i],
                                      &oCtrlStatus, &phone_num))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("TAPIDEMO_Setup() failed. (File: %s, line: %d)\n",
               __FILE__, __LINE__));
         /* Cleanup previous stuff */
#ifdef VXWORKS
         abort();
#else /* VXWORKS */
         TAPIDEMO_CleanUp(SIGABRT);
#endif /* VXWORKS */
         return IFX_ERROR;
      }
      /* If support for multiple boards then initialize all boards. */
   } /* for all boards*/

#ifndef EASY336
   if (oCtrlStatus.nSumPCM_Ch > 0)
   {
      if (PCM_Init(&oProgramArg) != IFX_SUCCESS)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("PCM_Init() failed. (File: %s, line: %d)\n",
               __FILE__, __LINE__));

         /* Cleanup previous stuff */
#ifdef VXWORKS
         abort();
#else /* VXWORKS */
         TAPIDEMO_CleanUp(SIGABRT);
#endif /* VXWORKS */
         return IFX_ERROR;
      } /* if */
   } /* if (oCtrlStatus.nSumPCM_Ch > 0) */
#endif /* EASY336 */

#ifdef EASY336
   if (IFX_ERROR == BOARD_Easy336_Init_RM(&oCtrlStatus, &pcm_cfg))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("DEVICE_SVIP_Init_RM() failed. (File: %s, line: %d)\n",
             __FILE__, __LINE__));
      /* Cleanup previous stuff */
#ifdef VXWORKS
      abort();
#else /* VXWORKS */
      TAPIDEMO_CleanUp(SIGABRT);
#endif /* VXWORKS */
      return IFX_ERROR;
   }
#endif /* EASY336 */

   TAPIDEMO_Call_Info(&oCtrlStatus);

   /* Actions are handled by two threads. The first thread is responsible
      for handling actions from sockets. The second one is resposible for 
      handling actions from devices. Two separate threads are needed because of 
      library IFX OS, which has diffrent functions for supporting sockets and 
      devices. 
      Actions might be requested from different sources: devices or sockets.
      The synchronization between threads is using mutex (protect access to a 
      shared resources) and synch pipe (used for closing thread and refresh 
      timeout).*/

   /********************************************************
         Init mutex using for lock state machine
    ********************************************************/
   if (IFX_SUCCESS != 
       TD_OS_MutexInit(&oCtrlStatus.oMultithreadCtrl.oMutex.mutexStateMachine))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error: Failed to init mutex using for lock state machine.\n"
             "(File: %s, line: %d)\n", __FILE__, __LINE__));
   }

   /********************************************************
         Start thread for handle sockets
    ********************************************************/
   if (IFX_SUCCESS != 
       Common_StartThread(&oCtrlStatus.oMultithreadCtrl.oThread.oHandleSocketThreadCtrl,
                          TAPIDEMO_HandleSocketThread,
                          "HandleSocketThread",
                          &oCtrlStatus.oMultithreadCtrl.oPipe.oSocketSynchPipe,
                          &oCtrlStatus.oMultithreadCtrl.oLock.oSocketThreadStopLock))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error: Failed to start thread for handle sockets.\n"
             "(File: %s, line: %d)\n", __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /********************************************************
         Start handle devices
    ********************************************************/
   TAPIDEMO_HandleDeviceThread(&oCtrlStatus);


   /* This part never executes. */

   return ret;
} /* main() */

